@@ -1,37 +1,80 @@
 /*
- Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- 
- Permission is hereby granted, free of charge, to any person obtaining a copy
- of this software and associated documentation files (the "Software"), to
- deal in the Software without restriction, including without limitation the
- rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
- sell copies of the Software, and to permit persons to whom the Software is
- furnished to do so, subject to the following conditions:
- 
- The above copyright notice and this permission notice shall be included in
- all copies of the Software, its documentation and marketing & publicity
- materials, and acknowledgment shall be given in the documentation, materials
- and software packages that this Software was used.
-
- THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
- IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
- FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
- THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
- IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
- CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
+ * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
+ * Copyright (C) 2007-2008 Kim Woelders
+ *
+ * Permission is hereby granted, free of charge, to any person obtaining a copy
+ * of this software and associated documentation files (the "Software"), to
+ * deal in the Software without restriction, including without limitation the
+ * rights to use, copy, modify, merge, publish, distribute, sublicense, and/or
+ * sell copies of the Software, and to permit persons to whom the Software is
+ * furnished to do so, subject to the following conditions:
+ *
+ * The above copyright notice and this permission notice shall be included in
+ * all copies of the Software, its documentation and marketing & publicity
+ * materials, and acknowledgment shall be given in the documentation, materials
+ * and software packages that this Software was used.
+ *
+ * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
+ * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
+ * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL
+ * THE AUTHORS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER
+ * IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN
+ * CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.
  */
-
 #include "dox.h"
 
+#define DEFAULT_LINKCOLOR_R 30
+#define DEFAULT_LINKCOLOR_G 50
+#define DEFAULT_LINKCOLOR_B 160
+
+typedef enum _type {
+   IMG,
+   BR,
+   FONT,
+   P,
+   TEXT,
+   PAGE
+} Type;
+
+typedef struct _img {
+   char               *src;
+   char               *src2;
+   char               *src3;
+   int                 x, y;
+   char               *link;
+   int                 w, h;
+} Img_;
+
+typedef struct _font {
+   char               *face;
+   int                 r, g, b;
+} Font_;
+
+typedef struct _p {
+   float               align;
+} P_;
+
+typedef struct _object {
+   Type                type;
+   void               *object;
+} Object;
+
+typedef struct _page {
+   char               *name;
+   int                 count;
+   Object             *obj;
+   int                 columns;
+   int                 padding;
+   int                 linkr, linkg, linkb;
+   char               *background;
+} Page;
+
 static int          num_pages = 0;
 static Page        *page = NULL;
 static char        *fdat_ptr = NULL;
 static int          fdat_size = 0;
 static char        *fdat = NULL;
 
-static int          fdgetc(void);
-static void         fdjump(int count);
-
 static int
 fdgetc(void)
 {
@@ -54,7 +97,7 @@ fdjump(int count)
       fdat_ptr = (fdat + fdat_size) - 1;
 }
 
-void
+static void
 AddPage(Object * obj)
 {
    num_pages++;
@@ -86,7 +129,7 @@ AddPage(Object * obj)
      }
 }
 
-void
+static void
 AddObject(Object * obj)
 {
    page[num_pages - 1].count++;
@@ -96,7 +139,7 @@ AddObject(Object * obj)
    page[num_pages - 1].obj[page[num_pages - 1].count - 1].object = obj->object;
 }
 
-void
+static void
 BuildObj(Object * obj, char *var, char *param)
 {
    static Page        *pg = NULL;
@@ -225,7 +268,7 @@ BuildObj(Object * obj, char *var, char *param)
      }
 }
 
-int
+static int
 GetNextTag(Object * obj)
 {
    char                s[65536];
@@ -304,7 +347,7 @@ GetNextTag(Object * obj)
    return 1;
 }
 
-char               *
+static char        *
 GetTextUntilTag(void)
 {
    char                s[65536];
@@ -375,29 +418,22 @@ GetObjects(FILE * f)
 	  {
 	     int                 j;
 
-	     if (page[i].name)
-		free(page[i].name);
-	     if (page[i].background)
-		free(page[i].background);
+	     Efree(page[i].name);
+	     Efree(page[i].background);
 	     for (j = 0; j < page[i].count; j++)
 	       {
 		  switch (page[i].obj[j].type)
 		    {
 		    case IMG:
-		       if (((Img_ *) page[i].obj[j].object)->src)
-			  free(((Img_ *) page[i].obj[j].object)->src);
-		       if (((Img_ *) page[i].obj[j].object)->src2)
-			  free(((Img_ *) page[i].obj[j].object)->src2);
-		       if (((Img_ *) page[i].obj[j].object)->src3)
-			  free(((Img_ *) page[i].obj[j].object)->src3);
-		       if (((Img_ *) page[i].obj[j].object)->link)
-			  free(((Img_ *) page[i].obj[j].object)->link);
+		       Efree(((Img_ *) page[i].obj[j].object)->src);
+		       Efree(((Img_ *) page[i].obj[j].object)->src2);
+		       Efree(((Img_ *) page[i].obj[j].object)->src3);
+		       Efree(((Img_ *) page[i].obj[j].object)->link);
 		       break;
 		    case BR:
 		       break;
 		    case FONT:
-		       if (((Font_ *) page[i].obj[j].object)->face)
-			  free(((Font_ *) page[i].obj[j].object)->face);
+		       Efree(((Font_ *) page[i].obj[j].object)->face);
 		       break;
 		    case P:
 		       break;
@@ -406,13 +442,11 @@ GetObjects(FILE * f)
 		    case PAGE:
 		       break;
 		    }
-		  if (page[i].obj[j].object)
-		     free(page[i].obj[j].object);
+		  Efree(page[i].obj[j].object);
 	       }
-	     if (page[i].obj)
-		free(page[i].obj);
+	     Efree(page[i].obj);
 	  }
-	free(page);
+	Efree(page);
 	num_pages = 0;
 	page = NULL;
 	have_font = 0;
@@ -434,8 +468,7 @@ GetObjects(FILE * f)
 	       {
 		  if (!GetNextTag(&obj))
 		    {
-		       if (fdat)
-			  free(fdat);
+		       Efree(fdat);
 		       return 0;
 		    }
 	       }
@@ -444,8 +477,7 @@ GetObjects(FILE * f)
 	  {
 	     if (!GetNextTag(&obj))
 	       {
-		  if (fdat)
-		     free(fdat);
+		  Efree(fdat);
 		  return 0;
 	       }
 	  }
@@ -465,7 +497,7 @@ GetObjects(FILE * f)
 	   have_font = 1;
 	obj.object = NULL;
      }
-   free(fdat);
+   Efree(fdat);
 }
 
 int
@@ -521,15 +553,7 @@ RenderPage(Window win, int page_num, int w, int h)
    Imlib_Image         im;
    int                 wastext = 0;
 
-   ts.fontname = NULL;
-   ESetColor(&ts.fg_col, 0, 0, 0);
-   ESetColor(&ts.bg_col, 0, 0, 0);
-   ts.effect = 0;
-   ts.efont = NULL;
-   ts.xfont = NULL;
-   ts.xfontset = 0;
-   ts.xfontset_ascent = 0;
-   ts.height = 0;
+   memset(&ts, 0, sizeof(ts));
    pg = &(page[page_num]);
    x = pg->padding;
    y = pg->padding;
@@ -537,11 +561,7 @@ RenderPage(Window win, int page_num, int w, int h)
    col_h = h - (pg->padding * 2);
    if (pg->background)
      {
-	char                tmp[4096];
-
-	sprintf(tmp, "%s/%s", docdir, pg->background);
-	findLocalizedFile(tmp);
-	im = imlib_load_image(tmp);
+	im = ImageLoadDoc(pg->background);
 	if (im)
 	  {
 	     imlib_context_set_image(im);
@@ -565,10 +585,7 @@ RenderPage(Window win, int page_num, int w, int h)
 	     img = (Img_ *) pg->obj[i].object;
 	     if (img->src)
 	       {
-		  char                tmp[4096];
-
-		  sprintf(tmp, "%s/%s", docdir, img->src);
-		  im = imlib_load_image(tmp);
+		  im = ImageLoadDoc(img->src);
 		  if (im)
 		    {
 		       imlib_context_set_image(im);
@@ -608,9 +625,11 @@ RenderPage(Window win, int page_num, int w, int h)
 	     if (ts.efont)
 		Efont_free(ts.efont);
 	     ts.efont = NULL;
+#if USE_XFONT
 	     if (ts.xfont)
 		XFreeFont(disp, ts.xfont);
 	     ts.xfont = NULL;
+#endif
 	     if (ts.xfontset)
 		XFreeFontSet(disp, ts.xfontset);
 	     ts.xfontset = NULL;
@@ -999,8 +1018,10 @@ RenderPage(Window win, int page_num, int w, int h)
 
    if (ts.efont)
       Efont_free(ts.efont);
+#if USE_XFONT
    if (ts.xfont)
       XFreeFont(disp, ts.xfont);
+#endif
    if (ts.xfontset)
       XFreeFontSet(disp, ts.xfontset);
 
