@@ -3,7 +3,7 @@
 /*****************************************************************************/
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -37,8 +37,7 @@
 
 #define USE_EXT_INIT_WIN 1
 
-#include <X11/Xlib.h>
-#include <X11/extensions/shape.h>
+#include <X11/X.h>
 
 #ifdef HAVE_SM
 #define USE_SM 1
@@ -66,28 +65,14 @@
 #define USE_GLX 1
 #endif
 
-#include "xwin.h"
+#include "xtypes.h"
 
 #define ENABLE_COLOR_MODIFIERS 0	/* Not functional */
+#define ENABLE_DESKRAY         0	/* Not functional */
 
 #define ENABLE_TRANSPARENCY 1
 #define ENABLE_THEME_TRANSPARENCY 1
 
-/* sgi's stdio.h has:
- * 
- * #if _SGIAPI && _NO_ANSIMODE
- * extern int      vsnprintf(char *, ssize_t, const char *, char *);
- * #endif
- * 
- * so workaround...
- */
-
-#ifdef __sgi
-#ifdef _NO_ANSIMODE
-#undef _NO_ANSIMODE
-#endif
-#endif
-
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -95,19 +80,6 @@
 #include <sys/types.h>
 #include <unistd.h>
 
-#ifndef HAVE_GETCWD
-#error "ERROR: Enlightenment needs a system with getcwd() in it's libs."
-#error "You may have to upgrade your Operating system, Distribution, base"
-#error "system libraries etc. Please see the the instructions for your"
-#error "particular Operating System or Distribution"
-#endif
-#ifndef HAVE_MKDIR
-#error "ERROR: Enlightenment needs a system with mkdir() in it's libs."
-#error "You may have to upgrade your Operating system, Distribution, base"
-#error "system libraries etc. Please see the the instructions for your"
-#error "particular Operating System or Distribution"
-#endif
-
 #ifndef HAVE_STRCASESTR
 # define strcasestr(haystack, needle) strstr(haystack, needle)
 #endif
@@ -209,61 +181,24 @@
  */
 #include "etypes.h"
 
-typedef struct
-{
-   int                 min, max;
-}
-Constraints;
-
-typedef struct
-{
-   Window              xwin;
-   Win                 win;
-   Visual             *vis;
-   int                 depth;
-   Colormap            cmap;
-   int                 scr;
-   int                 w, h;
-}
-RealRoot;
-
-typedef struct
-{
-   Window              xwin;
-   Win                 win;
-   Visual             *vis;
-   int                 depth;
-   Colormap            cmap;
-   int                 scr;
-   int                 w, h;
-   Pixmap              pmap;	/* Compositing buffer */
-}
-VirtRoot;
-
 /* Configuration parameters */
-typedef struct
-{
-   struct
-   {
+typedef struct {
+   struct {
       unsigned int        step;	/* Animation time step, ms */
    } animation;
-   struct
-   {
+   struct {
       char                enable;
       int                 delay;	/* milliseconds */
    } autoraise;
-   struct
-   {
+   struct {
       char                hiquality;
       char                user;
       int                 timeout;
    } backgrounds;
-   struct
-   {
+   struct {
       int                 move_resistance;
    } buttons;
-   struct
-   {
+   struct {
       unsigned int        num;
       int                 dragdir;
       int                 dragbar_width;
@@ -278,21 +213,18 @@ typedef struct
       int                 edge_flip_mode;
       int                 edge_flip_resistance;
    } desks;
-   struct
-   {
+   struct {
       char                headers;
       char                button_image;
    } dialogs;
-   struct
-   {
+   struct {
       char                enable;	/* wmdockapp only */
       char                sticky;	/* Make dockapps sticky by default */
       int                 dirmode;
       int                 startx;
       int                 starty;
    } dock;
-   struct
-   {
+   struct {
       int                 mode;
       char                clickraises;
       char                transientsfollowleader;
@@ -304,24 +236,20 @@ typedef struct
       char                warp_on_next;
       char                warp_always;
    } focus;
-   struct
-   {
+   struct {
       char                set_xroot_info_on_root_window;
    } hints;
-   struct
-   {
+   struct {
       char                animate;
       char                onscreen;
       char                warp;
       char                show_icons;
       int                 icon_size;
-      struct
-      {
+      struct {
 	 KeySym              left, right, up, down, escape, ret;
       } key;
    } menus;
-   struct
-   {
+   struct {
       int                 mode_move;
       int                 mode_resize;
       int                 mode_info;
@@ -331,29 +259,29 @@ typedef struct
       char                update_while_moving;
       char                enable_sync_request;
    } movres;
-   struct
-   {
+   struct {
       int                 movres;
       int                 menus;
       int                 tooltips;
+      int                 focused;
+      int                 unfocused;
    } opacity;
-   struct
-   {
+   struct {
       char                manual;
       char                manual_mouse_pointer;
+      char                center_if_desk_full;
+      char                ignore_struts;
+      char                raise_fullscreen;
       char                slidein;
       char                cleanupslide;
       int                 slidemode;
       int                 slidespeedmap;
       int                 slidespeedcleanup;
-      char                ignore_struts;
-      char                raise_fullscreen;
 #ifdef HAVE_XINERAMA
       char                extra_head;	/* Not used */
 #endif
    } place;
-   struct
-   {
+   struct {
       char                enable_script;
       char               *script;
       char                enable_logout_dialog;
@@ -361,24 +289,20 @@ typedef struct
       char               *cmd_reboot;
       char               *cmd_halt;
    } session;
-   struct
-   {
+   struct {
       char                animate;
       int                 speed;
    } shading;
-   struct
-   {
+   struct {
       char                enable;
       int                 edge_snap_dist;
       int                 screen_snap_dist;
    } snap;
-   struct
-   {
+   struct {
       char                firsttime;
       char                animate;
    } startup;
-   struct
-   {
+   struct {
       char                localise;
       char                use_alt_font_cfg;
       char               *name;
@@ -387,8 +311,7 @@ typedef struct
       char               *font_cfg;
    } theme;
 #ifdef ENABLE_THEME_TRANSPARENCY
-   struct
-   {
+   struct {
       int                 alpha;
       int                 border;
       int                 widget;
@@ -402,8 +325,7 @@ typedef struct
       int                 warplist;
    } trans;
 #endif
-   struct
-   {
+   struct {
       char                enable;
       char                showsticky;
       char                showshaded;
@@ -415,8 +337,7 @@ typedef struct
       int                 icon_mode;
    } warplist;
 
-   struct
-   {
+   struct {
       char                argb_internal_objects;
       char                argb_internal_clients;
       char                argb_clients;
@@ -424,30 +345,27 @@ typedef struct
       int                 image_cache_size;
       int                 mask_alpha_threshold;
       char                enable_startup_id;
+      char                use_render_for_scaling;
+      unsigned int        no_sync_mask;
    } testing;
 
    char                autosave;
    char                memory_paranoia;
    char                save_under;
-}
-EConf;
+   char                difftime;
+} EConf;
 
-typedef struct
-{
-   struct
-   {
+typedef struct {
+   struct {
       unsigned int        startup_id;
    } apps;
-   struct
-   {
+   struct {
       char               *name;
       char               *dir;
       char               *cache_dir;
    } conf;
-   struct
-   {
-      char               *name;
-      int                 screens;
+   struct {
+      char                dummy;	/* Prevent empty struct */
 #ifdef HAVE_XINERAMA
       char                xinerama_active;
 #endif
@@ -455,11 +373,11 @@ typedef struct
       XID                 server_time;
 #endif
    } display;
-   struct
-   {
+   struct {
       Time                time;
-      int                 x, y;
-      int                 px, py;
+      int                 cx, cy;	/* Any detected pointer movement */
+      int                 mx, my;	/* Motion event */
+      int                 px, py;	/* Previous motion event */
       Time                last_btime;
       Window              last_bpress;
       Window              last_bpress2;
@@ -471,55 +389,44 @@ typedef struct
       unsigned int        damage_count;
 #endif
    } events;
-   struct
-   {
-      int                 server_grabbed;
+   struct {
       char                pointer_grab_active;
       Window              pointer_grab_window;
    } grabs;
-   struct
-   {
+   struct {
       const char         *lang;
       char                utf8_int;	/* Use UTF-8 internally */
       char                utf8_loc;	/* Locale is UTF-8 */
    } locale;
-   struct
-   {
+   struct {
       unsigned int        mod_key_mask;
       unsigned int        mod_combos[8];
    } masks;
-   struct
-   {
+   struct {
       char                check;	/* Avoid losing windows offscreen */
       char                swap;
       int                 swapcoord_x, swapcoord_y;
    } move;
-   struct
-   {
+   struct {
       signed char         enable_features;
       char                doing_manual;
       char                doing_slide;
    } place;
-   struct
-   {
+   struct {
       int                 w_old;
       int                 h_old;
    } screen;
-   struct
-   {
+   struct {
       unsigned int        extensions;
    } server;
-   struct
-   {
+   struct {
       Pixmap              ext_pmap;
       char                ext_pmap_valid;
    } root;
-   struct
-   {
+   struct {
       char               *path;
    } theme;
-   struct
-   {
+   struct {
       char               *exec_name;	/* argv[0] */
       char                master;	/* We are the master E */
       char                single;	/* No slaves */
@@ -529,11 +436,12 @@ typedef struct
       char                session_start;
       char                startup;
       char                restart;
-      char                xselect;
       char                exiting;
       char                in_signal_handler;
+      char                exit_now;
       char                save_ok;
       char                coredump;
+      int                 win_w, win_h;
       int                 exit_mode;
       char               *exit_param;
       int                 child_count;
@@ -554,8 +462,7 @@ typedef struct
    char                showing_desktop;
    Window              button_proxy_win;
    char                firsttime;
-}
-EMode;
+} EMode;
 
 /*
  * Function prototypes
@@ -588,10 +495,10 @@ int                 ConfigFileRead(FILE * fs);
 int                 ThemeConfigLoad(void);
 
 /* dialog.c */
-void                DialogOK(const char *title, const char *fmt, ...);
+void __PRINTF_2__   DialogOK(const char *title, const char *fmt, ...);
 void                DialogOKstr(const char *title, const char *txt);
-void                DialogAlert(const char *fmt, ...);
-void                DialogAlertOK(const char *fmt, ...);
+void __PRINTF__     DialogAlert(const char *fmt, ...);
+void __PRINTF__     DialogAlertOK(const char *fmt, ...);
 
 /* econfig.c */
 void                ConfigurationLoad(void);
@@ -618,8 +525,6 @@ const char         *FontLookup(const char *name);
 /* handlers.c */
 void                SignalsSetup(void);
 void                SignalsRestore(void);
-void                HandleXError(Display * d, XErrorEvent * ev);
-void                HandleXIOError(Display * d);
 
 /* main.c */
 void                EExit(int exitcode);
@@ -664,8 +569,5 @@ char              **ThemesList(int *num);
 extern const char   e_wm_name[];
 extern const char   e_wm_version[];
 extern const char   e_wm_date[];
-__EXPORT__ extern Display *disp;
-extern RealRoot     RRoot;
-__EXPORT__ extern VirtRoot VRoot;
 __EXPORT__ extern EConf Conf;
 __EXPORT__ extern EMode Mode;
