@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -32,15 +32,12 @@
 #include "grabs.h"
 #include <ctype.h>
 
-typedef struct _actiontype
-{
+typedef struct _actiontype {
    char               *params;
    struct _actiontype *next;
-}
-ActionType;
+} ActionType;
 
-struct _action
-{
+struct _action {
    char                event;
    char                anymodifier;
    int                 modifiers;
@@ -53,8 +50,7 @@ struct _action
    ActionType         *action;
 };
 
-struct _actionclass
-{
+struct _actionclass {
    char               *name;
    int                 num;
    Action            **list;
@@ -79,8 +75,7 @@ RemoveActionType(ActionType * ActionTypeToRemove)
    ptr = ActionTypeToRemove;
    while (ptr)
      {
-	if (ptr->params)
-	   Efree(ptr->params);
+	Efree(ptr->params);
 	pp = ptr;
 	ptr = ptr->next;
 	Efree(pp);
@@ -104,7 +99,7 @@ ActionCreate(char event, char anymod, int mod, int anybut, int but,
    if (!key || !key[0] || (event != EVENT_KEY_DOWN && event != EVENT_KEY_UP))
       aa->key = 0;
    else
-      aa->key = XKeysymToKeycode(disp, XStringToKeysym(key));
+      aa->key = EKeynameToKeycode(key);
    aa->key_str = (aa->key) ? Estrdup(key) : NULL;
    aa->tooltipstring =
       (tooltipstring) ? Estrdup((tooltipstring[0]) ? tooltipstring : "?!?") :
@@ -123,10 +118,8 @@ ActionDestroy(Action * aa)
       UnGrabActionKey(aa);
    if (aa->action)
       RemoveActionType(aa->action);
-   if (aa->tooltipstring)
-      Efree(aa->tooltipstring);
-   if (aa->key_str)
-      Efree(aa->key_str);
+   Efree(aa->tooltipstring);
+   Efree(aa->key_str);
    Efree(aa);
 }
 
@@ -192,7 +185,7 @@ ActionclassCreate(const char *name, int global)
    return ac;
 }
 
-void
+static void
 ActionclassDestroy(ActionClass * ac)
 {
    int                 i;
@@ -211,12 +204,10 @@ ActionclassDestroy(ActionClass * ac)
 
    for (i = 0; i < ac->num; i++)
       ActionDestroy(ac->list[i]);
-   if (ac->list)
-      Efree(ac->list);
-   if (ac->name)
-      Efree(ac->name);
-   if (ac->tooltipstring)
-      Efree(ac->tooltipstring);
+   Efree(ac->list);
+   Efree(ac->name);
+   Efree(ac->tooltipstring);
+
    Efree(ac);
    mode_action_destroy = 1;
 }
@@ -306,8 +297,6 @@ AclassConfigLoad(FILE * fs)
 		(aclass_tooltipstring) ? Estrdup((aclass_tooltipstring[0]) ?
 						 aclass_tooltipstring :
 						 "?!?") : NULL;
-	     _EFREE(aclass_tooltipstring);
-	     _EFREE(action_tooltipstring);
 	     err = 0;
 	     goto done;
 
@@ -457,7 +446,8 @@ AclassConfigLoad(FILE * fs)
 		  aa = ActionCreate(event, anymod, mod, anybut, but, anykey,
 				    key, action_tooltipstring);
 		  /* the correct place to grab an action key */
-		  _EFREE(action_tooltipstring);
+		  Efree(action_tooltipstring);
+		  action_tooltipstring = NULL;
 		  key[0] = '\0';
 		  if (global)
 		     GrabActionKey(aa);
@@ -482,8 +472,8 @@ AclassConfigLoad(FILE * fs)
       ActionclassDestroy(ac);
 
  done:
-   _EFREE(aclass_tooltipstring);
-   _EFREE(action_tooltipstring);
+   Efree(aclass_tooltipstring);
+   Efree(action_tooltipstring);
 
    return err;
 }
@@ -1252,9 +1242,9 @@ IPC_KeybindingsGet(const char *params __UNUSED__)
 	aa = ac->list[i];
 	if ((aa) && (aa->action) && (aa->event == EVENT_KEY_DOWN))
 	  {
-	     char               *key;
+	     const char         *key;
 
-	     key = XKeysymToString(XKeycodeToKeysym(disp, aa->key, 0));
+	     key = EKeycodeToString(aa->key, 0);
 	     if (!key)
 		continue;
 
@@ -1507,7 +1497,7 @@ GrabActionKey(Action * aa)
 
    mod = (aa->anymodifier) ? AnyModifier : aa->modifiers;
 
-   GrabKeySet(aa->key, mod, VRoot.win);
+   GrabKeySet(aa->key, mod, VROOT);
 }
 
 static void
@@ -1520,5 +1510,5 @@ UnGrabActionKey(Action * aa)
 
    mod = (aa->anymodifier) ? AnyModifier : aa->modifiers;
 
-   GrabKeyRelease(aa->key, mod, VRoot.win);
+   GrabKeyRelease(aa->key, mod, VROOT);
 }
