@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -29,18 +29,26 @@
 
 #define DEBUG_ARRANGE 0
 
-typedef struct
-{
+typedef struct {
    void               *data;
    int                 x, y, w, h;
    int                 p;
 } RectBox;
 
+typedef struct {
+   int                 x, y;
+   int                 p, q;
+} RectInfo;
+
 static int
 ArrangeAddToList(int *array, int current_size, int value)
 {
    int                 i, j;
 
+   if (current_size >= 2 &&
+       (value <= array[0] || value >= array[current_size - 1]))
+      return current_size;
+
    for (i = 0; i < current_size; i++)
      {
 	if (value < array[i])
@@ -57,36 +65,39 @@ ArrangeAddToList(int *array, int current_size, int value)
    return current_size + 1;
 }
 
-#define Filled(x,y) (filled[(y * (xsize - 1)) + x])
-
 static void
-ArrangeMakeFillLists(int startx, int width, int starty, int height,
-		     const RectBox * sorted, int num_sorted,
-		     int *xarray, int *nx, int *yarray, int *ny,
-		     unsigned char *filled)
+ArrangeMakeXYArrays(int tx1, int tx2, int ty1, int ty2, int fitw, int fith,
+		    const RectBox * sorted, int num_sorted,
+		    int *xarray, int *nx, int *yarray, int *ny)
 {
-   int                 j, x1, x2, y1, y2, k, y, x;
+   int                 j, x1, x2, y1, y2;
    int                 xsize, ysize;
 
    xsize = 0;
    ysize = 0;
 
    /* put all the sorted rects into the xy arrays */
-   xsize = ArrangeAddToList(xarray, xsize, startx);
-   xsize = ArrangeAddToList(xarray, xsize, width);
-   ysize = ArrangeAddToList(yarray, ysize, starty);
-   ysize = ArrangeAddToList(yarray, ysize, height);
+   xsize = ArrangeAddToList(xarray, xsize, tx1);
+   xsize = ArrangeAddToList(xarray, xsize, tx2);
+   xsize = ArrangeAddToList(xarray, xsize, tx2 - fitw);
+   ysize = ArrangeAddToList(yarray, ysize, ty1);
+   ysize = ArrangeAddToList(yarray, ysize, ty2);
+   ysize = ArrangeAddToList(yarray, ysize, ty2 - fith);
 
    for (j = 0; j < num_sorted; j++)
      {
-	if (sorted[j].x < width)
-	   xsize = ArrangeAddToList(xarray, xsize, sorted[j].x);
-	if ((sorted[j].x + sorted[j].w) < width)
-	   xsize = ArrangeAddToList(xarray, xsize, sorted[j].x + sorted[j].w);
-	if (sorted[j].y < height)
-	   ysize = ArrangeAddToList(yarray, ysize, sorted[j].y);
-	if ((sorted[j].y + sorted[j].h) < height)
-	   ysize = ArrangeAddToList(yarray, ysize, sorted[j].y + sorted[j].h);
+	x1 = sorted[j].x;
+	x2 = x1 + sorted[j].w;
+	xsize = ArrangeAddToList(xarray, xsize, x1);
+	xsize = ArrangeAddToList(xarray, xsize, x2);
+	xsize = ArrangeAddToList(xarray, xsize, x1 - fitw);
+	xsize = ArrangeAddToList(xarray, xsize, x2 - fitw);
+	y1 = sorted[j].y;
+	y2 = y1 + sorted[j].h;
+	ysize = ArrangeAddToList(yarray, ysize, y1);
+	ysize = ArrangeAddToList(yarray, ysize, y2);
+	ysize = ArrangeAddToList(yarray, ysize, y1 - fith);
+	ysize = ArrangeAddToList(yarray, ysize, y2 - fith);
      }
 #if DEBUG_ARRANGE
    for (j = 0; j < xsize; j++)
@@ -95,6 +106,19 @@ ArrangeMakeFillLists(int startx, int width, int starty, int height,
       Eprintf("yarray[%d] = %d\n", j, yarray[j]);
 #endif
 
+   *nx = xsize;
+   *ny = ysize;
+}
+
+#define Filled(x,y) (filled[(y * (xsize - 1)) + x])
+
+static void
+ArrangeMakeFillLists(const RectBox * sorted, int num_sorted,
+		     int *xarray, int xsize, int *yarray, int ysize,
+		     unsigned char *filled)
+{
+   int                 j, x1, x2, y1, y2, k, y, x;
+
    /* fill the allocation array */
    for (j = 0; j < (xsize - 1) * (ysize - 1); filled[j++] = 0)
       ;
@@ -133,8 +157,7 @@ ArrangeMakeFillLists(int startx, int width, int starty, int height,
 	       {
 		  for (x = x1; x <= x2; x++)
 		    {
-		       if (Filled(x, y) <= sorted[j].p)
-			  Filled(x, y) = sorted[j].p + 1;
+		       Filled(x, y) += sorted[j].p;
 		    }
 	       }
 	  }
@@ -149,98 +172,102 @@ ArrangeMakeFillLists(int startx, int width, int starty, int height,
 	printf("\n");
      }
 #endif
+}
 
-   *nx = xsize;
-   *ny = ysize;
+static void
+ArrangeFindSpace(const int *xarray, int xsize, const int *yarray, int ysize,
+		 unsigned char *filled, RectInfo * spaces, int *ns,
+		 int wx, int wy, int ww, int wh)
+{
+   int                 i, j, w, h, fw, fh, z1, z2;
+   int                 cost, desk;
+   int                 num_spaces = *ns;
+
+   if (wx < xarray[0] || (wx != xarray[0] && wx + ww > xarray[xsize - 1]))
+      return;
+   if (wy < yarray[0] || (wy != yarray[0] && wy + wh > yarray[ysize - 1]))
+      return;
+
+   cost = desk = 0;
+   fh = wh;
+#if DEBUG_ARRANGE > 1
+   Eprintf("Check-A %d,%d %dx%d\n", wx, wy, ww, wh);
+#endif
+   for (j = 0; j < ysize - 1; j++)
+     {
+	z2 = yarray[j + 1];
+	if (z2 <= wy)
+	   continue;
+
+	z1 = wy > yarray[j] ? wy : yarray[j];
+	z2 = wy + wh < z2 ? wy + wh : z2;
+	h = z2 - z1;
+	fw = ww;
+	for (i = 0; i < xsize - 1; i++)
+	  {
+	     z2 = xarray[i + 1];
+	     if (z2 <= wx)
+		continue;
+
+	     z1 = wx > xarray[i] ? wx : xarray[i];
+	     z2 = wx + ww < z2 ? wx + ww : z2;
+	     w = z2 - z1;
+#if DEBUG_ARRANGE > 1
+	     Eprintf("Add [%d,%d] %3dx%3d: %2d\n", i, j, w, h, Filled(i, j));
+#endif
+	     if (Filled(i, j) == 0)
+		desk += w * h;
+	     else
+		cost += w * h * Filled(i, j);
+	     fw -= w;
+	     if (fw <= 0)
+		break;
+	  }
+	fh -= h;
+	if (fh <= 0)
+	   break;
+     }
+
+#if DEBUG_ARRANGE
+   Eprintf("Check %4d,%4d %3dx%3d cost=%d desk=%d\n", wx, wy, ww, wh,
+	   cost, desk);
+#endif
+   spaces[num_spaces].x = wx;
+   spaces[num_spaces].y = wy;
+   spaces[num_spaces].p = cost;
+   spaces[num_spaces].q = desk;
+   num_spaces++;
+   *ns = num_spaces;
 }
 
 static void
 ArrangeFindSpaces(const int *xarray, int xsize, const int *yarray, int ysize,
-		  unsigned char *filled, RectBox * spaces, int *ns,
-		  RectBox * fit, int must_fit)
+		  unsigned char *filled, RectInfo * spaces, int max_spaces,
+		  int *ns, RectBox * fit)
 {
-   int                 i, j, x, y, x1, y1, xbest, ybest;
-   int                 num_spaces;
-   unsigned int        a, abest;
+   int                 ix, iy, fx, fy, fw, fh;
 
    /* create list of all "spaces" */
-   num_spaces = 0;
-   for (y = 0; y < ysize - 1; y++)
+   *ns = 0;
+   fw = fit->w;
+   fh = fit->h;
+   for (iy = 0; iy < ysize; iy++)
      {
-	for (x = 0; x < xsize - 1; x++)
+	fy = yarray[iy];
+
+	for (ix = 0; ix < xsize; ix++)
 	  {
-	     /* if the square is empty "grow" the space */
-	     if (Filled(x, y) > fit->p)
-		continue;
+	     fx = xarray[ix];
 
-	     xbest = x;
-	     ybest = y;
-	     abest = 0;
-	     x1 = xsize - 1;
-#if DEBUG_ARRANGE
-	     Eprintf("Check %d,%d: %d,%d\n", x, y, xarray[x], yarray[y]);
-#endif
-	     for (j = y; j < ysize - 1;)
-	       {
-		  for (i = x; i < x1; i++)
-		    {
-		       if (Filled(i, j) > fit->p)
-			  break;
-		    }
-		  x1 = i;
-		  j++;
-		  if (x1 <= x)
-		     break;
-		  if (must_fit && (xarray[x1] - xarray[x] < fit->w))
-		     continue;
-		  for (; j < ysize - 1; j++)
-		    {
-		       for (i = x; i < x1; i++)
-			 {
-			    if (Filled(i, j) > fit->p)
-			       goto got_one;
-			 }
-		    }
-		got_one:
-		  y1 = j;
-		  if (must_fit && (yarray[y1] - yarray[y] < fit->h))
-		     continue;
-		  a = (xarray[x1] - xarray[x]) * (yarray[y1] - yarray[y]);
-#if DEBUG_ARRANGE
-		  Eprintf("Got %4d,%4d %4dx%4d: %d\n", xarray[x],
-			  yarray[y], xarray[x1] - xarray[x],
-			  yarray[y1] - yarray[y], a);
-#endif
-		  if (a > abest)
-		    {
-		       xbest = x1;
-		       ybest = y1;
-		       abest = a;
-		    }
-	       }
-	     if (abest == 0)
-		continue;
-
-	     spaces[num_spaces].x = xarray[x];
-	     spaces[num_spaces].y = yarray[y];
-	     spaces[num_spaces].w = xarray[xbest] - xarray[x];
-	     spaces[num_spaces].h = yarray[ybest] - yarray[y];
-#if 0
-	     spaces[num_spaces].p = spaces[num_spaces].w >= fit->w &&
-		spaces[num_spaces].h >= fit->h;
-#else
-	     spaces[num_spaces].p = 1;
-#endif
-	     num_spaces++;
+	     ArrangeFindSpace(xarray, xsize, yarray, ysize, filled, spaces, ns,
+			      fx, fy, fw, fh);
+	     if (*ns >= max_spaces)
+		goto done;
 	  }
      }
-#if DEBUG_ARRANGE
-   for (j = 0; j < num_spaces; j++)
-      Eprintf("Spaces: x,y=%4d,%4d wxh=%3dx%3d p=%2d\n",
-	      spaces[j].x, spaces[j].y, spaces[j].w, spaces[j].h, spaces[j].p);
-#endif
 
-   *ns = num_spaces;
+ done:
+   ;
 }
 
 static void
@@ -271,17 +298,20 @@ ArrangeRects(const RectBox * fixed, int fixed_count, RectBox * floating,
 	     int width, int height, int policy, char initial_window)
 {
    int                 num_sorted;
+   int                 tx1, ty1, tx2, ty2;
    int                 xsize = 0, ysize = 0;
    int                *xarray, *yarray;
-   int                *leftover;
    int                 i, j, k;
    unsigned char      *filled;
-   RectBox            *spaces;
-   int                 num_spaces;
+   RectInfo           *spaces;
+   int                 num_spaces, alloc_spaces;
    int                 sort;
    int                 a1, a2;
-   int                 num_leftover;
 
+   tx1 = startx;
+   ty1 = starty;
+   tx2 = startx + width;
+   ty2 = starty + height;
    if (initial_window)
      {
 	int                 xx1, yy1, xx2, yy2;
@@ -289,17 +319,17 @@ ArrangeRects(const RectBox * fixed, int fixed_count, RectBox * floating,
 	ScreenGetAvailableAreaByPointer(&xx1, &yy1, &xx2, &yy2);
 	xx2 += xx1;
 	yy2 += yy1;
-	if (startx < xx1)
-	   startx = xx1;
-	if (width > xx2)
-	   width = xx2;
-	if (starty < yy1)
-	   starty = yy1;
-	if (height > yy2)
-	   height = yy2;
+	if (tx1 < xx1)
+	   tx1 = xx1;
+	if (tx2 > xx2)
+	   tx2 = xx2;
+	if (ty1 < yy1)
+	   ty1 = yy1;
+	if (ty2 > yy2)
+	   ty2 = yy2;
      }
 #if DEBUG_ARRANGE
-   Eprintf("Start %d,%d %dx%d\n", startx, starty, width, height);
+   Eprintf("Target area %d,%d -> %d,%d\n", tx1, ty1, tx2, ty2);
 #endif
 
    switch (policy)
@@ -346,17 +376,15 @@ ArrangeRects(const RectBox * fixed, int fixed_count, RectBox * floating,
      }
 
    /* for every floating rect in order, "fit" it into the sorted list */
-   i = ((fixed_count + floating_count) * 2) + 2;
+   i = ((fixed_count + floating_count) * 4) + 2;
    xarray = EMALLOC(int, i);
    yarray = EMALLOC(int, i);
-   filled = EMALLOC(unsigned char, i * i);
 
-   spaces = EMALLOC(RectBox, i * i);
-   leftover = NULL;
-   if (floating_count)
-      leftover = EMALLOC(int, floating_count);
+   filled = NULL;
+   spaces = NULL;
+   alloc_spaces = 0;
 
-   if (!xarray || !yarray || !filled || !spaces)
+   if (!xarray || !yarray)
       goto done;
 
    /* copy "fixed" rects into the sorted list */
@@ -364,149 +392,61 @@ ArrangeRects(const RectBox * fixed, int fixed_count, RectBox * floating,
    num_sorted = fixed_count;
 
    /* go through each floating rect in order and "fit" it in */
-   num_leftover = 0;
    for (i = 0; i < floating_count; i++)
      {
-	ArrangeMakeFillLists(startx, width, starty, height, sorted, num_sorted,
-			     xarray, &xsize, yarray, &ysize, filled);
+	ArrangeMakeXYArrays(tx1, tx2, ty1, ty2, floating[i].w, floating[i].h,
+			    sorted, num_sorted, xarray, &xsize, yarray, &ysize);
+	num_spaces = xsize * ysize;
+	if (alloc_spaces < num_spaces)
+	  {
+	     unsigned char      *ptr_f;
+	     RectInfo           *ptr_s;
+
+	     ptr_f = EREALLOC(unsigned char, filled, num_spaces);
+
+	     if (ptr_f)
+		filled = ptr_f;
+	     ptr_s = EREALLOC(RectInfo, spaces, num_spaces);
+	     if (ptr_s)
+		spaces = ptr_s;
+	     if (!ptr_f || !ptr_s)
+		goto done;
+	     alloc_spaces = num_spaces;
+	  }
+	ArrangeMakeFillLists(sorted, num_sorted,
+			     xarray, xsize, yarray, ysize, filled);
 
 	/* create list of all "spaces" */
 	ArrangeFindSpaces(xarray, xsize, yarray, ysize, filled,
-			  spaces, &num_spaces, floating + i, 1);
+			  spaces, alloc_spaces, &num_spaces, floating + i);
 
 	/* find the first space that fits */
-	k = -1;
-	sort = 0x7fffffff;
+	k = 0;
+	sort = 0x7fffffff;	/* NB! Break at 0 == free space */
 	for (j = 0; j < num_spaces; j++)
 	  {
-	     if ((spaces[j].w < floating[i].w) ||
-		 (spaces[j].h < floating[i].h) ||
-		 (spaces[j].x < startx) ||
-		 (spaces[j].x + spaces[j].w > width) ||
-		 (spaces[j].y < starty) || (spaces[j].y + spaces[j].h > height))
+	     a1 = spaces[j].p - spaces[j].q * 4;
+	     if (a1 >= sort)
 		continue;
-
-	     if (policy == ARRANGE_BY_POSITION)
-	       {
-		  a1 = (spaces[j].x + (spaces[j].w >> 1)) -
-		     (floating[i].x + (floating[i].w >> 1));
-		  a2 = (spaces[j].y + (spaces[j].h >> 1)) -
-		     (floating[i].y + (floating[i].h >> 1));
-		  if (a1 < 0)
-		     a1 = -a1;
-		  if (a2 < 0)
-		     a2 = -a2;
-		  if ((a1 + a2) < sort)
-		    {
-		       sort = a1 + a2;
-		       k = j;
-		    }
-	       }
-	     else
-	       {
-		  k = j;
-		  break;
-	       }
+	     sort = a1;
+	     k = j;
+	     if (sort == 0)
+		break;
 	  }
-
-	if (k >= 0)
+	if (spaces[k].q == 0 && Conf.place.center_if_desk_full)
 	  {
-	     if (policy == ARRANGE_BY_POSITION)
-	       {
-		  a1 = (spaces[k].x + (spaces[k].w >> 1)) - (floating[i].x +
-							     (floating[i].w >>
-							      1));
-		  a2 = (spaces[k].y + (spaces[k].h >> 1)) - (floating[i].y +
-							     (floating[i].h >>
-							      1));
-		  if (a1 >= 0)
-		    {
-		       sorted[num_sorted].x = spaces[k].x;
-		    }
-		  else
-		    {
-		       sorted[num_sorted].x =
-			  spaces[k].x + spaces[k].w - floating[i].w;
-		    }
-		  if (a2 >= 0)
-		    {
-		       sorted[num_sorted].y = spaces[k].y;
-		    }
-		  else
-		    {
-		       sorted[num_sorted].y =
-			  spaces[k].y + spaces[k].h - floating[i].h;
-		    }
-	       }
-	     else
-	       {
-		  sorted[num_sorted].x = spaces[k].x;
-		  sorted[num_sorted].y = spaces[k].y;
-	       }
-	     sorted[num_sorted].data = floating[i].data;
-	     sorted[num_sorted].w = floating[i].w;
-	     sorted[num_sorted].h = floating[i].h;
-	     sorted[num_sorted].p = floating[i].p;
-	     num_sorted++;
+	     sorted[num_sorted].x = (tx1 + tx2 - floating[i].w) / 2;
+	     sorted[num_sorted].y = (ty1 + ty2 - floating[i].h) / 2;
 	  }
 	else
-	   leftover[num_leftover++] = i;
-     }
-
-#if DEBUG_ARRANGE
-   Eprintf("Leftovers: %d\n", num_leftover);
-#endif
-   /* ok we cant fit everything in this baby.... time to fit */
-   /* the leftovers into the leftover space */
-   for (i = 0; i < num_leftover; i++)
-     {
-	ArrangeMakeFillLists(startx, width, starty, height, sorted, num_sorted,
-			     xarray, &xsize, yarray, &ysize, filled);
-
-	/* create list of all "spaces" */
-	ArrangeFindSpaces(xarray, xsize, yarray, ysize, filled,
-			  spaces, &num_spaces, floating + leftover[i], 0);
-
-	/* find the first space that fits */
-	k = -1;
-	sort = 0x7fffffff;
-	a1 = floating[leftover[i]].w * floating[leftover[i]].h;
-	k = -1;
-	for (j = 0; j < num_spaces; j++)
 	  {
-	     a2 = spaces[j].w * spaces[j].h;
-	     if ((a2 != 0) && ((a1 - a2) < sort) && (spaces[j].p))
-	       {
-		  k = j;
-		  sort = a1 - a2;
-	       }
-	  }
-
-	if (k >= 0)
-	  {
-	     /* if there's a small space ... */
 	     sorted[num_sorted].x = spaces[k].x;
 	     sorted[num_sorted].y = spaces[k].y;
 	  }
-	else
-	  {
-	     /* there is no room - put it centered */
-	     /* (but dont put top left off screen) */
-	     sorted[num_sorted].x = (width - floating[leftover[i]].w) / 2;
-	     sorted[num_sorted].y = (height - floating[leftover[i]].h) / 2;
-	  }
-	sorted[num_sorted].data = floating[leftover[i]].data;
-	sorted[num_sorted].w = floating[leftover[i]].w;
-	sorted[num_sorted].h = floating[leftover[i]].h;
-	sorted[num_sorted].p = floating[leftover[i]].p;
-	if ((sorted[num_sorted].x + sorted[num_sorted].w) > width)
-	   sorted[num_sorted].x = width - sorted[num_sorted].w;
-	if ((sorted[num_sorted].y + sorted[num_sorted].h) > height)
-	   sorted[num_sorted].y = height - sorted[num_sorted].h;
-	if (sorted[num_sorted].x < startx)
-	   sorted[num_sorted].x = startx;
-	if (sorted[num_sorted].y < starty)
-	   sorted[num_sorted].y = starty;
+	sorted[num_sorted].data = floating[i].data;
+	sorted[num_sorted].w = floating[i].w;
+	sorted[num_sorted].h = floating[i].h;
+	sorted[num_sorted].p = floating[i].p;
 	num_sorted++;
      }
 
@@ -514,21 +454,15 @@ ArrangeRects(const RectBox * fixed, int fixed_count, RectBox * floating,
    for (i = 0; i < num_sorted; i++)
       Eprintf("Sorted: x,y=%4d,%4d wxh=%3dx%3d p=%2d: %s\n",
 	      sorted[i].x, sorted[i].y, sorted[i].w, sorted[i].h, sorted[i].p,
-	      (sorted[i].data) ? ((EObj *) sorted[i].data)->name : "?");
+	      (sorted[i].data) ? EobjGetName((EObj *) sorted[i].data) : "?");
 #endif
 
  done:
    /* free up memory */
-   if (xarray)
-      Efree(xarray);
-   if (yarray)
-      Efree(yarray);
-   if (filled)
-      Efree(filled);
-   if (spaces)
-      Efree(spaces);
-   if (leftover)
-      Efree(leftover);
+   Efree(xarray);
+   Efree(yarray);
+   Efree(filled);
+   Efree(spaces);
 }
 
 void
@@ -735,8 +669,7 @@ SnapEwin(EWin * ewin, int dx, int dy, int *new_dx, int *new_dy)
 	   dy = 0;
      }
 
-   if (lst)
-      Efree(lst);
+   Efree(lst);
 
    if ((odx != dx) || (ody != dy))
      {
@@ -834,7 +767,8 @@ ArrangeGetRectList(RectBox ** pfixed, int *nfixed, RectBox ** pfloating,
 		       rb->p = EoGetLayer(ew);
 #if DEBUG_ARRANGE
 		       Eprintf("Add float: x,y=%4d,%4d wxh=%3dx%3d p=%2d: %s\n",
-			       rb->x, rb->y, rb->w, rb->h, rb->p, eo->name);
+			       rb->x, rb->y, rb->w, rb->h, rb->p,
+			       EobjGetName(eo));
 #endif
 		       continue;
 		    }
@@ -843,7 +777,7 @@ ArrangeGetRectList(RectBox ** pfixed, int *nfixed, RectBox ** pfloating,
 	     rb->data = ew;
 
 	     if (ew->props.never_use_area)
-		rb->p = 50;
+		rb->p = 100;
 	     else
 		rb->p = EoGetLayer(ew);
 	  }
@@ -870,16 +804,16 @@ ArrangeGetRectList(RectBox ** pfixed, int *nfixed, RectBox ** pfloating,
 	     w += x;
 	     x = 0;
 	  }
-	if ((x + w) > VRoot.w)
-	   w = VRoot.w - x;
+	if ((x + w) > WinGetW(VROOT))
+	   w = WinGetW(VROOT) - x;
 
 	if (y < 0)
 	  {
 	     h += y;
 	     y = 0;
 	  }
-	if ((y + h) > VRoot.h)
-	   h = VRoot.h - y;
+	if ((y + h) > WinGetH(VROOT))
+	   h = WinGetH(VROOT) - y;
 
 	if ((w <= 0) || (h <= 0))
 	   continue;
@@ -890,7 +824,7 @@ ArrangeGetRectList(RectBox ** pfixed, int *nfixed, RectBox ** pfloating,
 	rb->h = h;
 #if DEBUG_ARRANGE
 	Eprintf("Add fixed: x,y=%4d,%4d wxh=%3dx%3d p=%2d: %s\n", rb->x, rb->y,
-		rb->w, rb->h, rb->p, eo->name);
+		rb->w, rb->h, rb->p, EobjGetName(eo));
 #endif
 
 	nfix++;
@@ -934,9 +868,9 @@ ArrangeEwinXY(EWin * ewin, int *px, int *py)
    newrect.h = EoGetH(ewin);
    newrect.p = EoGetLayer(ewin);
 
-   ret = EMALLOC(RectBox, num + 1);
-   ArrangeRects(fixed, num, &newrect, 1, ret, 0, 0, VRoot.w, VRoot.h,
-		ARRANGE_BY_SIZE, 1);
+   ret = ECALLOC(RectBox, num + 1);
+   ArrangeRects(fixed, num, &newrect, 1, ret, 0, 0,
+		WinGetW(VROOT), WinGetH(VROOT), ARRANGE_BY_SIZE, 1);
 
    for (i = 0; i < num + 1; i++)
      {
@@ -947,10 +881,8 @@ ArrangeEwinXY(EWin * ewin, int *px, int *py)
 	     break;
 	  }
      }
-   if (ret)
-      Efree(ret);
-   if (fixed)
-      Efree(fixed);
+   Efree(ret);
+   Efree(fixed);
 }
 
 void
@@ -993,9 +925,9 @@ ArrangeEwins(const char *params)
 
    ArrangeGetRectList(&fixed, &nfix, &floating, &nflt, NULL);
 
-   ret = EMALLOC(RectBox, nflt + nfix);
-   ArrangeRects(fixed, nfix, floating, nflt, ret, 0, 0, VRoot.w, VRoot.h,
-		method, 1);
+   ret = ECALLOC(RectBox, nflt + nfix);
+   ArrangeRects(fixed, nfix, floating, nflt, ret, 0, 0,
+		WinGetW(VROOT), WinGetH(VROOT), method, 1);
 
    for (i = nfix; i < nflt + nfix; i++)
      {
@@ -1014,12 +946,9 @@ ArrangeEwins(const char *params)
 	   EwinMove(ewin, ret[i].x, ret[i].y);
      }
 
-   if (fixed)
-      Efree(fixed);
-   if (ret)
-      Efree(ret);
-   if (floating)
-      Efree(floating);
+   Efree(fixed);
+   Efree(ret);
+   Efree(floating);
 
  done:
    return;
