@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -36,8 +36,7 @@
 #include "timers.h"
 #include "xwin.h"
 
-typedef struct
-{
+typedef struct {
    char               *file;
    char               *real_file;
    EImage             *im;
@@ -46,8 +45,7 @@ typedef struct
    int                 xperc, yperc;
 } BgPart;
 
-struct _background
-{
+struct _background {
    char               *name;
    Pixmap              pmap;
    time_t              last_viewed;
@@ -173,7 +171,7 @@ BackgroundPixmapFree(Background * bg)
 {
    if (bg->pmap)
      {
-	EImagePixmapFree(bg->pmap);
+	EImagePixmapsFree(bg->pmap, None);
 	bg->pmap = None;
      }
 }
@@ -210,20 +208,16 @@ BackgroundImagesKeep(Background * bg, int onoff)
 static void
 BackgroundFilesRemove(Background * bg)
 {
-   if (bg->bg.file)
-      Efree(bg->bg.file);
+   Efree(bg->bg.file);
    bg->bg.file = NULL;
 
-   if (bg->bg.real_file)
-      Efree(bg->bg.real_file);
+   Efree(bg->bg.real_file);
    bg->bg.real_file = NULL;
 
-   if (bg->top.file)
-      Efree(bg->top.file);
+   Efree(bg->top.file);
    bg->top.file = NULL;
 
-   if (bg->top.real_file)
-      Efree(bg->top.real_file);
+   Efree(bg->top.real_file);
    bg->top.real_file = NULL;
 
    BackgroundImagesFree(bg);
@@ -249,8 +243,7 @@ BackgroundDestroy(Background * bg)
    BackgroundFilesRemove(bg);
    BackgroundPixmapFree(bg);
 
-   if (bg->name)
-      Efree(bg->name);
+   Efree(bg->name);
 
    Efree(bg);
 
@@ -424,8 +417,7 @@ BackgroundModify(Background * bg, EColor * solid, const char *bgn, char tile,
      }
    else
       updated = 1;
-   if (bg->bg.file)
-      Efree(bg->bg.file);
+   Efree(bg->bg.file);
    bg->bg.file = (bgn[0]) ? Estrdup(bgn) : NULL;
    if ((int)tile != bg->bg_tile)
       updated = 1;
@@ -453,8 +445,7 @@ BackgroundModify(Background * bg, EColor * solid, const char *bgn, char tile,
      }
    else
       updated = 1;
-   if (bg->top.file)
-      Efree(bg->top.file);
+   Efree(bg->top.file);
    bg->top.file = (top[0]) ? Estrdup(top) : NULL;
    if ((int)tkeep_aspect != bg->top.keep_aspect)
       updated = 1;
@@ -495,18 +486,20 @@ BgFindImageSize(BgPart * bgp, unsigned int rw, unsigned int rh,
 
    EImageGetSize(bgp->im, &iw, &ih);
 
+#if 0				/* FIXME - Remove? */
    if (bgp->keep_aspect)
       bgp->xperc = bgp->yperc;
+#endif
 
    if (bgp->xperc > 0)
       w = (rw * bgp->xperc) >> 10;
    else
-      w = (iw * rw) / VRoot.w;
+      w = (iw * rw) / WinGetW(VROOT);
 
    if (bgp->yperc > 0)
       h = (rh * bgp->yperc) >> 10;
    else
-      h = (ih * rh) / VRoot.h;
+      h = (ih * rh) / WinGetH(VROOT);
 
    if (w <= 0)
       w = 1;
@@ -541,7 +534,7 @@ BackgroundCreatePixmap(Win win, unsigned int w, unsigned int h)
     * invalid one from a previous session.
     */
    pmap = ECreatePixmap(win, w, h, 0);
-   if (win == RRoot.win && pmap == Mode.root.ext_pmap)
+   if (win == RROOT && pmap == Mode.root.ext_pmap)
      {
 	EFreePixmap(pmap);
 	pmap = ECreatePixmap(win, w, h, 0);
@@ -623,7 +616,7 @@ BackgroundRealize(Background * bg, Win win, Drawable draw, unsigned int rw,
    if (!hasbg && !hasfg)
      {
 	/* Solid color only */
-	EAllocColor(VRoot.cmap, &bg->bg_solid);
+	EAllocColor(WinGetCmap(VROOT), &bg->bg_solid);
 
 	if (!is_win)
 	  {
@@ -761,7 +754,7 @@ BackgroundApplyWin(Background * bg, Win win)
    if (pmap != None)
      {
 	ESetWindowBackgroundPixmap(win, pmap);
-	EImagePixmapFree(pmap);
+	EImagePixmapsFree(pmap, None);
      }
    else
      {
@@ -849,7 +842,7 @@ BrackgroundCreateFromImage(const char *bgid, const char *file,
    if (bg)
       return bg;
 
-   scr_asp = (VRoot.w << 16) / VRoot.h;
+   scr_asp = (WinGetW(VROOT) << 16) / WinGetH(VROOT);
    im_asp = (width << 16) / height;
    if (width == height)
      {
@@ -997,8 +990,8 @@ BackgroundCacheMini(Background * bg, int keep, int nuke)
      }
 
    /* Create new cached bg mini image */
-   pmap = ECreatePixmap(VRoot.win, 64, 48, 0);
-   BackgroundApplyPmap(bg, VRoot.win, pmap, 64, 48);
+   pmap = ECreatePixmap(VROOT, 64, 48, 0);
+   BackgroundApplyPmap(bg, VROOT, pmap, 64, 48);
    im = EImageGrabDrawable(pmap, None, 0, 0, 64, 48, 0);
    EImageSave(im, s);
    EFreePixmap(pmap);
@@ -1193,8 +1186,7 @@ BackgroundsConfigLoad(FILE * fs)
 	       }
 	     else
 	       {
-		  if (name)
-		     Efree(name);
+		  Efree(name);
 		  name = Estrdup(s2);
 	       }
 	     break;
@@ -1230,19 +1222,14 @@ BackgroundsConfigLoad(FILE * fs)
 		    &i3, &i4, &i5, &i6);
 	     if (!ignore)
 	       {
-		  if (bg1)
-		     Efree(bg1);
+		  Efree(bg1);
 		  bg1 = Estrdup(s2);
 	       }
 	     else
 	       {
-		  if (bg->bg.file)
-		     Efree(bg->bg.file);
-		  if (bg->top.file)
-		    {
-		       Efree(bg->top.file);
-		       bg->top.file = NULL;
-		    }
+		  Efree(bg->bg.file);
+		  Efree(bg->top.file);
+		  bg->top.file = NULL;
 		  bg->bg.file = Estrdup(s2);
 		  bg->bg_tile = i1;
 		  bg->bg.keep_aspect = i2;
@@ -1258,8 +1245,7 @@ BackgroundsConfigLoad(FILE * fs)
 		    &j4, &j5);
 	     if (!ignore)
 	       {
-		  if (bg2)
-		     Efree(bg2);
+		  Efree(bg2);
 		  bg2 = Estrdup(s2);
 	       }
 	     else
@@ -1280,12 +1266,9 @@ BackgroundsConfigLoad(FILE * fs)
    err = -1;
 
  done:
-   if (name)
-      Efree(name);
-   if (bg1)
-      Efree(bg1);
-   if (bg2)
-      Efree(bg2);
+   Efree(name);
+   Efree(bg1);
+   Efree(bg2);
 
    return err;
 }
@@ -1614,8 +1597,7 @@ BG_DialogSetFileName(DItem * di)
    Esnprintf(s, sizeof(s),
 	     _("Background definition information:\nName: %s\nFile: %s\n"),
 	     BackgroundGetName(tmp_bg), (stmp) ? stmp : _("-NONE-"));
-   if (stmp)
-      Efree(stmp);
+   Efree(stmp);
    DialogItemSetText(di, s);
 }
 
@@ -1766,7 +1748,7 @@ BG_RedrawView(void)
 
    ic_button = ImageclassFind("DIALOG_BUTTON", 0);
 
-   XSetForeground(disp, gc, BlackPixel(disp, VRoot.scr));
+   XSetForeground(disp, gc, Dpy.pixel_black);
    XFillRectangle(disp, pmap, gc, 0, 0, w, h);
 
    x = -(num * (64 + 8) - w) * tmp_bg_sel_sliderval / (4 * num);
@@ -2389,8 +2371,7 @@ BackgroundSet1(const char *name, const char *params)
      }
    else if (!strcmp(type, "bg.file"))
      {
-	if (bg->bg.file)
-	   Efree(bg->bg.file);
+	Efree(bg->bg.file);
 	bg->bg.file = Estrdup(p);
      }
    else if (!strcmp(type, "bg.tile"))
@@ -2419,8 +2400,7 @@ BackgroundSet1(const char *name, const char *params)
      }
    else if (!strcmp(type, "top.file"))
      {
-	if (bg->top.file)
-	   Efree(bg->top.file);
+	Efree(bg->top.file);
 	bg->top.file = Estrdup(p);
      }
    else if (!strcmp(type, "top.keep_aspect"))
