@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -112,7 +112,7 @@ BorderWinpartITclassApply(EWin * ewin, int i, int force)
 
    ITApply(ewb->win, ewin->border->part[i].iclass, is,
 	   ewb->state, ewin->state.active, EoIsSticky(ewin),
-	   ST_BORDER, ewin->border->part[i].tclass, ts, txt);
+	   ST_BORDER, ewin->border->part[i].tclass, ts, txt, 1);
 }
 
 static int
@@ -200,13 +200,14 @@ BorderWinpartCalc(EWin * ewin, int i, int ww, int hh)
       BorderWinpartCalc(ewin, topleft, ww, hh);
    if (bottomright >= 0)
       BorderWinpartCalc(ewin, bottomright, ww, hh);
+
    x = y = 0;
    if (topleft == -1)
      {
-	x = ((ewin->border->part[i].geom.topleft.x.percent *
-	      ww) >> 10) + ewin->border->part[i].geom.topleft.x.absolute;
-	y = ((ewin->border->part[i].geom.topleft.y.percent *
-	      hh) >> 10) + ewin->border->part[i].geom.topleft.y.absolute;
+	x = ((ewin->border->part[i].geom.topleft.x.percent * ww) >> 10) +
+	   ewin->border->part[i].geom.topleft.x.absolute;
+	y = ((ewin->border->part[i].geom.topleft.y.percent * hh) >> 10) +
+	   ewin->border->part[i].geom.topleft.y.absolute;
      }
    else if (topleft >= 0)
      {
@@ -219,14 +220,13 @@ BorderWinpartCalc(EWin * ewin, int i, int ww, int hh)
 	   ewin->border->part[i].geom.topleft.y.absolute +
 	   ewin->bits[topleft].y;
      }
+
    ox = oy = 0;
    if (bottomright == -1)
      {
-	ox = ((ewin->border->
-	       part[i].geom.bottomright.x.percent * ww) >> 10) +
+	ox = ((ewin->border->part[i].geom.bottomright.x.percent * ww) >> 10) +
 	   ewin->border->part[i].geom.bottomright.x.absolute;
-	oy = ((ewin->border->
-	       part[i].geom.bottomright.y.percent * hh) >> 10) +
+	oy = ((ewin->border->part[i].geom.bottomright.y.percent * hh) >> 10) +
 	   ewin->border->part[i].geom.bottomright.y.absolute;
      }
    else if (bottomright >= 0)
@@ -240,6 +240,7 @@ BorderWinpartCalc(EWin * ewin, int i, int ww, int hh)
 	   ewin->border->part[i].geom.bottomright.y.absolute +
 	   ewin->bits[bottomright].y;
      }
+
    /*
     * calculate height before width, because we may need it in order to
     * determine the font size. But we might do it the other way around for
@@ -476,8 +477,7 @@ EwinBorderDetach(EWin * ewin)
 	if (ewin->bits[i].win)
 	   EDestroyWindow(ewin->bits[i].win);
      }
-   if (ewin->bits)
-      Efree(ewin->bits);
+   Efree(ewin->bits);
    ewin->bits = NULL;
    BorderDecRefcount(b);
 
@@ -524,20 +524,6 @@ EwinBorderSetTo(EWin * ewin, const Border * b)
 	EMapWindow(ewin->bits[i].win);
 	EventCallbackRegister(ewin->bits[i].win, 0,
 			      BorderWinpartHandleEvents, &ewin->bits[i]);
-	/*
-	 * KeyPressMask KeyReleaseMask ButtonPressMask 
-	 * ButtonReleaseMask
-	 * EnterWindowMask LeaveWindowMask PointerMotionMask 
-	 * PointerMotionHintMask Button1MotionMask 
-	 * Button2MotionMask
-	 * Button3MotionMask Button4MotionMask Button5MotionMask
-	 * ButtonMotionMask KeymapStateMask ExposureMask 
-	 * VisibilityChangeMask StructureNotifyMask 
-	 * ResizeRedirectMask 
-	 * SubstructureNotifyMask SubstructureRedirectMask 
-	 * FocusChangeMask PropertyChangeMas ColormapChangeMask 
-	 * OwnerGrabButtonMask
-	 */
 	if (b->part[i].flags & FLAG_TITLE)
 	   ESelectInput(ewin->bits[i].win, EWIN_BORDER_TITLE_EVENT_MASK);
 	else
@@ -572,7 +558,7 @@ EwinBorderSetTo(EWin * ewin, const Border * b)
 	   if (!b->part[i].ontop)
 	      wl[j++] = WinGetXwin(ewin->bits[i].win);
 	}
-      XRestackWindows(disp, wl, j);
+      EXRestackWindows(wl, j);
       Efree(wl);
    }
 
@@ -660,13 +646,9 @@ BorderDestroy(Border * b)
 	ECursorFree(b->part[i].ec);
      }
 
-   if (b->num_winparts > 0)
-      Efree(b->part);
-
-   if (b->name)
-      Efree(b->name);
-   if (b->group_border_name)
-      Efree(b->group_border_name);
+   Efree(b->part);
+   Efree(b->name);
+   Efree(b->group_border_name);
    ActionclassFree(b->aclass);
 }
 
@@ -850,24 +832,6 @@ BorderWinpartIndex(EWin * ewin, Win win)
    return -1;			/* Not found */
 }
 
-void
-EwinBorderEventsConfigure(EWin * ewin, int mode)
-{
-   int                 i;
-   long                emask;
-
-   emask = (mode) ? ~((long)0) : ~(EnterWindowMask | LeaveWindowMask);
-
-   for (i = 0; i < ewin->border->num_winparts; i++)
-     {
-	if (ewin->border->part[i].flags & FLAG_TITLE)
-	   ESelectInput(ewin->bits[i].win,
-			EWIN_BORDER_TITLE_EVENT_MASK & emask);
-	else
-	   ESelectInput(ewin->bits[i].win, EWIN_BORDER_PART_EVENT_MASK & emask);
-     }
-}
-
 /*
  * Border event handlers
  */
@@ -1195,56 +1159,59 @@ BorderConfigLoad(FILE * fs)
 	i1 = ConfigParseline1(s, s2, NULL, NULL);
 	i2 = atoi(s2);
 
-	if (i2 == CONFIG_OPEN)
+	switch (i1)
 	  {
+	  case CONFIG_CLOSE:
+	     goto done;
+	  case CONFIG_CLASSNAME:
+	  case BORDER_NAME:
+	     if (BorderFind(s2))
+	       {
+		  SkipTillEnd(fs);
+		  goto done;
+	       }
+	     b = BorderCreate(s2);
+	     continue;
+	  }
+
+	if (!b)
+	   break;
+
+	switch (i1)
+	  {
+	  default:
+	     break;
+	  case BORDER_INIT:
+	     if (i2 != CONFIG_OPEN)
+		break;
 	     err = BorderPartLoad(fs, i1, b);
 	     if (err)
 		break;
-	  }
-	else
-	  {
-	     switch (i1)
-	       {
-	       case CONFIG_CLOSE:
-		  goto done;
-	       case BORDER_INIT:
-		  break;
-	       case CONFIG_CLASSNAME:
-	       case BORDER_NAME:
-		  if (BorderFind(s2))
-		    {
-		       SkipTillEnd(fs);
-		       goto done;
-		    }
-		  b = BorderCreate(s2);
-		  break;
-	       case BORDER_GROUP_NAME:
-		  b->group_border_name = Estrdup(s2);
-		  break;
-	       case BORDER_LEFT:
-		  b->border.left = i2;
-		  break;
-	       case BORDER_RIGHT:
-		  b->border.right = i2;
-		  break;
-	       case BORDER_TOP:
-		  b->border.top = i2;
-		  break;
-	       case BORDER_BOTTOM:
-		  b->border.bottom = i2;
-		  break;
-	       case SHADEDIR:
-		  b->shadedir = i2;
-		  break;
-	       case BORDER_CHANGES_SHAPE:
-		  b->changes_shape = i2;
-		  break;
-	       case CONFIG_ACTIONCLASS:
-		  b->aclass = ActionclassFind(s2);
-		  break;
-	       default:
-		  break;
-	       }
+	     break;
+	  case BORDER_GROUP_NAME:
+	     b->group_border_name = Estrdup(s2);
+	     break;
+	  case BORDER_LEFT:
+	     b->border.left = i2;
+	     break;
+	  case BORDER_RIGHT:
+	     b->border.right = i2;
+	     break;
+	  case BORDER_TOP:
+	     b->border.top = i2;
+	     break;
+	  case BORDER_BOTTOM:
+	     b->border.bottom = i2;
+	     break;
+	  case SHADEDIR:
+	     b->shadedir = i2;
+	     break;
+	  case BORDER_CHANGES_SHAPE:
+	     b->changes_shape = i2;
+	     break;
+	  case CONFIG_ACTIONCLASS:
+	     b->aclass = ActionclassFind(s2);
+	     break;
 	  }
      }
    err = -1;
