@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -41,20 +41,18 @@
    ButtonPressMask | ButtonReleaseMask | EnterWindowMask | LeaveWindowMask | \
    PointerMotionMask)
 
-typedef struct
-{
-   Constraints         width, height;
+typedef struct {
+   int                 width_min, width_max;
+   int                 height_min, height_max;
    int                 xorigin, yorigin;
    int                 xabs, xrel;
    int                 yabs, yrel;
    int                 xsizerel, xsizeabs;
    int                 ysizerel, ysizeabs;
    char                size_from_image;
-}
-BGeometry;
+} BGeometry;
 
-struct _button
-{
+struct _button {
    EObj                o;
    BGeometry           geom;
    ImageClass         *iclass;
@@ -77,8 +75,7 @@ struct _button
 
 static Ecore_List  *button_list = NULL;
 
-static struct
-{
+static struct {
    Button             *button;
    char                loading_user;
    char                move_pending;
@@ -144,10 +141,10 @@ ButtonCreate(const char *name, int id, const char *iclass,
       b->tclass = TextclassAlloc(tclass, 1);
 
    b->flags = flags;
-   b->geom.width.min = minw;
-   b->geom.width.max = maxw;
-   b->geom.height.min = minh;
-   b->geom.height.max = maxh;
+   b->geom.width_min = minw;
+   b->geom.width_max = maxw;
+   b->geom.height_min = minh;
+   b->geom.height_max = maxh;
    b->geom.xorigin = xo;
    b->geom.yorigin = yo;
    b->geom.xabs = xa;
@@ -192,9 +189,7 @@ ButtonDestroy(Button * b)
    ImageclassFree(b->iclass);
    ActionclassFree(b->aclass);
    TextclassFree(b->tclass);
-
-   if (b->label)
-      Efree(b->label);
+   Efree(b->label);
 
    Efree(b);
 }
@@ -239,21 +234,21 @@ ButtonCalc(Button * b)
      }
    else
      {
-	w = ((b->geom.xsizerel * VRoot.w) >> 10) + b->geom.xsizeabs;
-	h = ((b->geom.ysizerel * VRoot.h) >> 10) + b->geom.ysizeabs;
+	w = ((b->geom.xsizerel * WinGetW(VROOT)) >> 10) + b->geom.xsizeabs;
+	h = ((b->geom.ysizerel * WinGetH(VROOT)) >> 10) + b->geom.ysizeabs;
      }
-   if (w > b->geom.width.max)
-      w = b->geom.width.max;
-   else if (w < b->geom.width.min)
-      w = b->geom.width.min;
-   if (h > b->geom.height.max)
-      h = b->geom.height.max;
-   else if (h < b->geom.height.min)
-      h = b->geom.height.min;
+   if (w > b->geom.width_max)
+      w = b->geom.width_max;
+   else if (w < b->geom.width_min)
+      w = b->geom.width_min;
+   if (h > b->geom.height_max)
+      h = b->geom.height_max;
+   else if (h < b->geom.height_min)
+      h = b->geom.height_min;
    xo = (w * b->geom.xorigin) >> 10;
    yo = (h * b->geom.yorigin) >> 10;
-   x = ((b->geom.xrel * VRoot.w) >> 10) + b->geom.xabs - xo;
-   y = ((b->geom.yrel * VRoot.h) >> 10) + b->geom.yabs - yo;
+   x = ((b->geom.xrel * WinGetW(VROOT)) >> 10) + b->geom.xabs - xo;
+   y = ((b->geom.yrel * WinGetH(VROOT)) >> 10) + b->geom.yabs - yo;
 
    EoMoveResize(b, x, y, w, h);
 }
@@ -262,7 +257,7 @@ static void
 ButtonDraw(Button * b)
 {
    ITApply(EoGetWin(b), b->iclass, NULL,
-	   b->state, 0, 0, ST_BUTTON, b->tclass, NULL, b->label);
+	   b->state, 0, 0, ST_BUTTON, b->tclass, NULL, b->label, 0);
    EoShapeUpdate(b, 0);
 }
 
@@ -343,21 +338,21 @@ ButtonMoveToCoord(Button * b, int x, int y)
    if (ButtonIsFixed(b))
       return;
 
-   if ((x + (EoGetW(b) >> 1)) < (VRoot.w / 3))
+   if ((x + (EoGetW(b) >> 1)) < (WinGetW(VROOT) / 3))
       relx = 0;
-   else if ((x + (EoGetW(b) >> 1)) > ((VRoot.w * 2) / 3))
+   else if ((x + (EoGetW(b) >> 1)) > ((WinGetW(VROOT) * 2) / 3))
       relx = 1024;
    else
       relx = 512;
-   rx = (relx * VRoot.w) >> 10;
+   rx = (relx * WinGetW(VROOT)) >> 10;
    absx = x - rx;
-   if ((y + (EoGetH(b) >> 1)) < (VRoot.h / 3))
+   if ((y + (EoGetH(b) >> 1)) < (WinGetH(VROOT) / 3))
       rely = 0;
-   else if ((y + (EoGetH(b) >> 1)) > ((VRoot.h * 2) / 3))
+   else if ((y + (EoGetH(b) >> 1)) > ((WinGetH(VROOT) * 2) / 3))
       rely = 1024;
    else
       rely = 512;
-   ry = (rely * VRoot.h) >> 10;
+   ry = (rely * WinGetH(VROOT)) >> 10;
    absy = y - ry;
    if (!(b->flags & FLAG_FIXED_HORIZ))
      {
@@ -421,8 +416,8 @@ ButtonDragStart(Button * b)
    GrabPointerSet(EoGetWin(b), ECSR_GRAB, 0);
    Mode.mode = MODE_BUTTONDRAG;
    Mode_buttons.move_pending = 1;
-   Mode_buttons.start_x = Mode.events.x;
-   Mode_buttons.start_y = Mode.events.y;
+   Mode_buttons.start_x = Mode.events.cx;
+   Mode_buttons.start_y = Mode.events.cy;
 }
 
 static void
@@ -434,7 +429,7 @@ ButtonDragEnd(Button * b)
 
    if (!Mode_buttons.move_pending)
      {
-	dsk = DesktopAt(Mode.events.x, Mode.events.y);
+	dsk = DesktopAt(Mode.events.mx, Mode.events.my);
 	ButtonMoveToDesktop(b, dsk);
 	dsk = EoGetDesk(b);
 	ButtonMoveRelative(b, -EoGetX(dsk), -EoGetY(dsk));
@@ -499,7 +494,7 @@ ButtonEventMouseDown(Button * b, XEvent * ev)
 	Window              win = ev->xbutton.window;
 
 	ev->xbutton.window = b->inside_win;
-	XSendEvent(disp, b->inside_win, False, ButtonPressMask, ev);
+	EXSendEvent(b->inside_win, ButtonPressMask, ev);
 	ev->xbutton.window = win;
      }
 
@@ -527,7 +522,7 @@ ButtonEventMouseUp(Button * b, XEvent * ev)
 	Window              win = ev->xbutton.window;
 
 	ev->xbutton.window = b->inside_win;
-	XSendEvent(disp, b->inside_win, False, ButtonReleaseMask, ev);
+	EXSendEvent(b->inside_win, ButtonReleaseMask, ev);
 	ev->xbutton.window = win;
      }
 
@@ -558,15 +553,15 @@ ButtonEventMotion(Button * b, XEvent * ev __UNUSED__)
    if (Mode.mode != MODE_BUTTONDRAG)
       return;
 
-   dx = Mode.events.x - Mode.events.px;
-   dy = Mode.events.y - Mode.events.py;
+   dx = Mode.events.mx - Mode.events.px;
+   dy = Mode.events.my - Mode.events.py;
 
    if (Mode_buttons.move_pending)
      {
 	int                 x, y;
 
-	x = Mode.events.x - Mode_buttons.start_x;
-	y = Mode.events.y - Mode_buttons.start_y;
+	x = Mode.events.mx - Mode_buttons.start_x;
+	y = Mode.events.my - Mode_buttons.start_y;
 	if (x < 0)
 	   x = -x;
 	if (y < 0)
@@ -709,10 +704,10 @@ ButtonsConfigLoad(FILE * fs)
 		  pbt->flags = flags;
 		  pbt->internal = internal;
 		  pbt->default_show = show;
-		  pbt->geom.width.min = minw;
-		  pbt->geom.width.max = maxw;
-		  pbt->geom.height.min = minh;
-		  pbt->geom.height.max = maxh;
+		  pbt->geom.width_min = minw;
+		  pbt->geom.width_max = maxw;
+		  pbt->geom.height_min = minh;
+		  pbt->geom.height_max = maxh;
 		  pbt->geom.xorigin = xo;
 		  pbt->geom.yorigin = yo;
 		  pbt->geom.xabs = xa;
@@ -859,10 +854,10 @@ ButtonsConfigSave(void)
 	 fprintf(fs, "11 %s\n", ActionclassGetName(b->aclass));
       if (EoGetLayer(b) >= 0)
 	 fprintf(fs, "453 %i\n", EoGetLayer(b));
-      fprintf(fs, "456 %i\n", b->geom.width.min);
-      fprintf(fs, "457 %i\n", b->geom.width.max);
-      fprintf(fs, "468 %i\n", b->geom.height.min);
-      fprintf(fs, "469 %i\n", b->geom.height.max);
+      fprintf(fs, "456 %i\n", b->geom.width_min);
+      fprintf(fs, "457 %i\n", b->geom.width_max);
+      fprintf(fs, "468 %i\n", b->geom.height_min);
+      fprintf(fs, "469 %i\n", b->geom.height_max);
       fprintf(fs, "528 %i\n", b->geom.xorigin);
       fprintf(fs, "529 %i\n", b->geom.yorigin);
       fprintf(fs, "530 %i\n", b->geom.xabs);
@@ -927,8 +922,7 @@ ButtonsSighan(int sig, void *prm __UNUSED__)
      }
 }
 
-typedef struct
-{
+typedef struct {
    int                 id;
    int                 match;
    const char         *regex;
@@ -949,8 +943,10 @@ _ButtonHideShow(void *data, void *prm)
 	match = matchregexp(bmd->regex, EoGetName(b));
 	if ((match && !bmd->match) || (!match && bmd->match))
 	   return;
-	if (!strcmp(EoGetName(b), "_DESKTOP_DESKRAY_DRAG_CONTROL"))	/* FIXME - ??? */
+#if ENABLE_DESKRAY
+	if (!strcmp(EoGetName(b), "_DESKTOP_DESKRAY_DRAG_CONTROL"))
 	   return;
+#endif
      }
 
    ButtonToggle(b);
