@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -115,7 +115,7 @@ ContainerCreate(const char *name)
    ct->scrollbar_clicked = 0;
    ct->scrollbox_clicked = 0;
 
-   ct->win = ECreateClientWindow(VRoot.win, 0, 0, 1, 1);
+   ct->win = ECreateClientWindow(VROOT, 0, 0, 1, 1);
    ct->icon_win = ECreateWindow(ct->win, 0, 0, 128, 26, 0);
    EventCallbackRegister(ct->icon_win, 0, ContainerEventIconWin, ct);
    ct->cover_win = ECreateWindow(ct->win, 0, 0, 128, 26, 0);
@@ -173,13 +173,11 @@ ContainerDestroy(Container * ct, int exiting)
 
    ct->ops->Exit(ct, exiting);
 
-   if (ct->name)
-      Efree(ct->name);
+   Efree(ct->name);
 
    EDestroyWindow(ct->win);
 
-   if (ct->objs)
-      Efree(ct->objs);
+   Efree(ct->objs);
 
    Efree(ct);
 
@@ -326,7 +324,8 @@ ContainerShow(Container * ct)
 	/* The first one */
 	EwinMoveToDesktop(ewin, EoGetDesk(ewin));
 	EwinResize(ewin, 128, 32);
-	EwinMove(ewin, VRoot.w - EoGetW(ewin), VRoot.h - EoGetH(ewin));
+	EwinMove(ewin, WinGetW(VROOT) - EoGetW(ewin),
+		 WinGetH(VROOT) - EoGetH(ewin));
      }
 
    EwinShow(ewin);
@@ -1015,15 +1014,15 @@ ContainerLayout(Container * ct, int *px, int *py, int *pw, int *ph)
 	     add = ct->max;
 	     if (ct->ewin->border)
 	       {
-		  if ((bt + bb + add) > VRoot.h)
-		     add = VRoot.h - (bt + bb);
+		  if ((bt + bb + add) > WinGetH(VROOT))
+		     add = WinGetH(VROOT) - (bt + bb);
 	       }
 	     y += (((ct->ewin->client.h - add) * ct->auto_resize_anchor) >> 10);
 	     h = add;
 	     if (ct->ewin->border)
 	       {
-		  if ((EoGetY(ct->ewin) + bt + bb + add) > VRoot.h)
-		     y = VRoot.h - (bt + bb + add);
+		  if ((EoGetY(ct->ewin) + bt + bb + add) > WinGetH(VROOT))
+		     y = WinGetH(VROOT) - (bt + bb + add);
 	       }
 	  }
 	else
@@ -1031,15 +1030,15 @@ ContainerLayout(Container * ct, int *px, int *py, int *pw, int *ph)
 	     add = ct->max;
 	     if (ct->ewin->border)
 	       {
-		  if ((bl + br + add) > VRoot.w)
-		     add = VRoot.w - (bl + br);
+		  if ((bl + br + add) > WinGetW(VROOT))
+		     add = WinGetW(VROOT) - (bl + br);
 	       }
 	     x += (((ct->ewin->client.w - add) * ct->auto_resize_anchor) >> 10);
 	     w = add;
 	     if (ct->ewin->border)
 	       {
-		  if ((EoGetX(ct->ewin) + bl + br + add) > VRoot.w)
-		     x = VRoot.w - (bl + br + add);
+		  if ((EoGetX(ct->ewin) + bl + br + add) > WinGetW(VROOT))
+		     x = WinGetW(VROOT) - (bl + br + add);
 	       }
 	  }
      }
@@ -1061,7 +1060,6 @@ ContainerDraw(Container * ct)
    int                 ib_x0, ib_y0, ib_w0, ib_h0;
    EImage             *im;
    int                 ww, hh;
-   Pixmap              pmap, mask;
 
    if (!ct->ic_box)
       ContainerLayoutImageWin(ct);
@@ -1160,13 +1158,8 @@ ContainerDraw(Container * ct)
    if (im)
      {
 	EMapWindow(ct->icon_win);
-	EImageRenderPixmaps(im, ct->icon_win, EIMAGE_HIGH_MASK_THR,
-			    &pmap, &mask, 0, 0);
-	ESetWindowBackgroundPixmap(ct->icon_win, pmap);
-	EShapeCombineMask(ct->icon_win, ShapeBounding, 0, 0, mask, ShapeSet);
-	EImagePixmapFree(pmap);
+	EImageApplyToWin(im, ct->icon_win, EIMAGE_HIGH_MASK_THR, 0, 0);
 	EImageFree(im);
-	EClearWindow(ct->icon_win);
 
 	if (ct->type == IB_TYPE_SYSTRAY && ct->nobg && !ct->draw_icon_base)
 	   EShapePropagate(ct->icon_win);
@@ -1553,8 +1546,7 @@ _DlgFillContainer(Dialog * d, DItem * table, void *data)
    tmp_ib_cover_hide = ct->cover_hide;
    tmp_ib_autoresize_anchor = ct->auto_resize_anchor;
    tmp_ib_anim_mode = ct->anim_mode;
-   if (tmp_ib_name)
-      Efree(tmp_ib_name);
+   Efree(tmp_ib_name);
    tmp_ib_name = Estrdup(ct->name);
 
    DialogSetTitle(d, ct->dlg_title);
