@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -36,6 +36,7 @@
 #include "grabs.h"
 #include "hints.h"
 #include "iclass.h"
+#include "screen.h"
 #include "settings.h"
 #include "timers.h"
 #include "tooltips.h"
@@ -52,15 +53,13 @@
 
 #define ENLIGHTENMENT_CONF_NUM_DESKTOPS 32
 
-typedef struct
-{
+typedef struct {
    Desk               *current;
    Desk               *previous;
    Desk               *desk[ENLIGHTENMENT_CONF_NUM_DESKTOPS];
    unsigned int        order[ENLIGHTENMENT_CONF_NUM_DESKTOPS];
    int                 drag_x0, drag_y0;
-}
-Desktops;
+} Desktops;
 
 static void         DeskRaise(unsigned int num);
 static void         DeskLower(unsigned int num);
@@ -78,6 +77,10 @@ DeskControlsCreate(Desk * dsk)
 {
    char                s[512];
    const char         *ic1, *ic2, *ic3;
+
+#if ENABLE_DESKRAY
+   const char         *ic4;
+#endif
    char                ac1[64], ac2[64], ac3[64];
    Button             *b;
    ActionClass        *ac;
@@ -95,8 +98,8 @@ DeskControlsCreate(Desk * dsk)
       Conf.desks.dragbar_width = 64;
    if (Conf.desks.dragbar_length < 0)
       Conf.desks.dragbar_length = 0;
-   else if (Conf.desks.dragbar_length > VRoot.w)
-      Conf.desks.dragbar_length = VRoot.w;
+   else if (Conf.desks.dragbar_length > WinGetW(VROOT))
+      Conf.desks.dragbar_length = WinGetW(VROOT);
 
    Esnprintf(ac1, sizeof(ac1), "DRAGBAR_DESKTOP_%i", dsk->num);
    if (!ActionclassFind(ac1))
@@ -170,7 +173,7 @@ DeskControlsCreate(Desk * dsk)
 	ic1 = "DESKTOP_DRAGBUTTON_VERT";
 	ic2 = "DESKTOP_RAISEBUTTON_VERT";
 	ic3 = "DESKTOP_LOWERBUTTON_VERT";
-#if 0
+#if ENABLE_DESKRAY
 	ic4 = "DESKTOP_DESKRAY_VERT";
 #endif
      }
@@ -179,7 +182,7 @@ DeskControlsCreate(Desk * dsk)
 	ic1 = "DESKTOP_DRAGBUTTON_HORIZ";
 	ic2 = "DESKTOP_RAISEBUTTON_HORIZ";
 	ic3 = "DESKTOP_LOWERBUTTON_HORIZ";
-#if 0
+#if ENABLE_DESKRAY
 	ic4 = "DESKTOP_DESKRAY_HORIZ";
 #endif
      }
@@ -229,7 +232,7 @@ DeskControlsCreate(Desk * dsk)
      case 0:
 	w[0] = w[1] = w[2] = h[0] = h[1] = Conf.desks.dragbar_width;
 	if (Conf.desks.dragbar_length == 0)
-	   h[2] = VRoot.h - (Conf.desks.dragbar_width * 2);
+	   h[2] = WinGetH(VROOT) - (Conf.desks.dragbar_width * 2);
 	else
 	   h[2] = Conf.desks.dragbar_length;
 	x[0] = x[1] = x[2] = 0;
@@ -240,10 +243,10 @@ DeskControlsCreate(Desk * dsk)
      case 1:
 	w[0] = w[1] = w[2] = h[0] = h[1] = Conf.desks.dragbar_width;
 	if (Conf.desks.dragbar_length == 0)
-	   h[2] = VRoot.h - (Conf.desks.dragbar_width * 2);
+	   h[2] = WinGetH(VROOT) - (Conf.desks.dragbar_width * 2);
 	else
 	   h[2] = Conf.desks.dragbar_length;
-	x[0] = x[1] = x[2] = VRoot.w - Conf.desks.dragbar_width;
+	x[0] = x[1] = x[2] = WinGetW(VROOT) - Conf.desks.dragbar_width;
 	y[m] = 0;
 	y[n] = y[m] + h[m];
 	y[o] = y[n] + h[n];
@@ -251,7 +254,7 @@ DeskControlsCreate(Desk * dsk)
      case 2:
 	h[0] = h[1] = h[2] = w[0] = w[1] = Conf.desks.dragbar_width;
 	if (Conf.desks.dragbar_length == 0)
-	   w[2] = VRoot.w - (Conf.desks.dragbar_width * 2);
+	   w[2] = WinGetW(VROOT) - (Conf.desks.dragbar_width * 2);
 	else
 	   w[2] = Conf.desks.dragbar_length;
 	y[0] = y[1] = y[2] = 0;
@@ -262,10 +265,10 @@ DeskControlsCreate(Desk * dsk)
      case 3:
 	h[0] = h[1] = h[2] = w[0] = w[1] = Conf.desks.dragbar_width;
 	if (Conf.desks.dragbar_length == 0)
-	   w[2] = VRoot.w - (Conf.desks.dragbar_width * 2);
+	   w[2] = WinGetW(VROOT) - (Conf.desks.dragbar_width * 2);
 	else
 	   w[2] = Conf.desks.dragbar_length;
-	y[0] = y[1] = y[2] = VRoot.h - Conf.desks.dragbar_width;
+	y[0] = y[1] = y[2] = WinGetH(VROOT) - Conf.desks.dragbar_width;
 	x[m] = 0;
 	x[n] = x[m] + w[m];
 	x[o] = x[n] + w[n];
@@ -288,7 +291,7 @@ DeskControlsCreate(Desk * dsk)
 	ButtonSetCallback(b, DeskButtonCallback, EoObj(dsk));
      }
 
-#if 0				/* What is this anyway? */
+#if ENABLE_DESKRAY
    if (dsk->num > 0)
      {
 	if (Conf.desks.dragdir == 0)
@@ -303,7 +306,7 @@ DeskControlsCreate(Desk * dsk)
 	     b = ButtonCreate("_DESKTOP_DESKRAY_DRAG_CONTROL", 2, ic4, ac1,
 			      NULL, NULL, 1, FLAG_FIXED_VERT, 1, 99999, 1,
 			      99999, 0, 0,
-			      EoGetX(dsk) + VRoot.w -
+			      EoGetX(dsk) + WinGetW(VROOT) -
 			      Conf.desks.dragbar_width, 0, EoGetY(dsk),
 			      0, 0, 0, 0, 0, 1, 0, 1);
 	  }
@@ -319,8 +322,8 @@ DeskControlsCreate(Desk * dsk)
 	     b = ButtonCreate("_DESKTOP_DESKRAY_DRAG_CONTROL", 2, ic4, ac1,
 			      NULL, NULL, 1, FLAG_FIXED_HORIZ, 1, 99999, 1,
 			      99999, 0, 0, EoGetX(dsk), 0,
-			      EoGetY(dsk) + VRoot.h - Conf.desks.dragbar_width,
-			      0, 0, 0, 0, 0, 1, 0, 1);
+			      EoGetY(dsk) + WinGetH(VROOT) -
+			      Conf.desks.dragbar_width, 0, 0, 0, 0, 0, 1, 0, 1);
 	  }
      }
 #endif
@@ -353,7 +356,7 @@ DeskConfigure(Desk * dsk)
 
    if (dsk->num > 0)
      {
-	EoMove(dsk, VRoot.w, 0);
+	EoMove(dsk, WinGetW(VROOT), 0);
 	EoMap(dsk, 0);
      }
 
@@ -379,11 +382,12 @@ DeskCreate(int desk, int configure)
    dsk->num = desk;
    desks.order[desk] = desk;
 
-   win = (desk == 0) ? VRoot.win : NoWin;
+   win = (desk == 0) ? VROOT : NULL;
 
    Esnprintf(buf, sizeof(buf), "Desk-%d", desk);
    EoSetNoRedirect(dsk, 1);
-   EoInit(dsk, EOBJ_TYPE_DESK, win, 0, 0, VRoot.w, VRoot.h, 0, buf);
+   EoInit(dsk, EOBJ_TYPE_DESK, win,
+	  0, 0, WinGetW(VROOT), WinGetH(VROOT), 0, buf);
    EventCallbackRegister(EoGetWin(dsk), 0, DeskHandleEvents, dsk);
    EoSetFade(dsk, 0);
    EoSetShadow(dsk, 0);
@@ -394,13 +398,15 @@ DeskCreate(int desk, int configure)
 #if !USE_BG_WIN_ON_ALL_DESKS	/* TBD - Use per virtual root bg window? */
 #if USE_COMPOSITE
 	/* Add background window */
-	eo = EobjWindowCreate(EOBJ_TYPE_ROOT_BG, 0, 0, VRoot.w, VRoot.h,
+	eo = EobjWindowCreate(EOBJ_TYPE_ROOT_BG,
+			      0, 0, WinGetW(VROOT), WinGetH(VROOT),
 			      0, "Root-bg");
 	eo->floating = 0;
 	eo->fade = eo->shadow = 0;
 	EobjSetLayer(eo, 0);
 	EventCallbackRegister(EobjGetWin(eo), 0, DeskHandleEvents, dsk);
 	dsk->bg.o_bg = eo;
+	ESelectInput(EobjGetWin(eo), EnterWindowMask);
 #endif
 #endif
      }
@@ -408,10 +414,6 @@ DeskCreate(int desk, int configure)
      {
 	EoSetFloating(dsk, 1);
 	EoSetLayer(dsk, 0);
-#if 0				/* TBD */
-	dsk->event_mask = EDESK_EVENT_MASK;
-	DeskEventsConfigure(dsk, 1);
-#endif
 	/* Set the _XROOT... atoms so apps will find them even before the bg is set */
 	HintsSetRootInfo(EoGetWin(dsk), None, 0);
      }
@@ -420,7 +422,8 @@ DeskCreate(int desk, int configure)
 #if USE_COMPOSITE
    /* Add background window */
    Esnprintf(buf, sizeof(buf), "Desk-bg-%d", desk);
-   eo = EobjWindowCreate(EOBJ_TYPE_MISC, 0, 0, VRoot.w, VRoot.h, 0, buf);
+   eo = EobjWindowCreate(EOBJ_TYPE_MISC,
+			 0, 0, WinGetW(VROOT), WinGetH(VROOT), 0, buf);
    eo->floating = 0;
    eo->fade = eo->shadow = 0;
    EobjReparent(eo, EoObj(dsk), 0, 0);
@@ -474,7 +477,7 @@ Win
 DeskGetBackgroundWin(const Desk * dsk)
 {
    if (!dsk)
-      return VRoot.win;
+      return VROOT;
    return EobjGetWin(dsk->bg.o);
 }
 
@@ -537,14 +540,14 @@ DeskBackgroundConfigure(Desk * dsk)
 	     if (pmap != None)
 	       {
 		  ESetWindowBackgroundPixmap(win, pmap);
-		  if (dsk->num == 0 && win != VRoot.win)
-		     ESetWindowBackgroundPixmap(VRoot.win, pmap);
+		  if (dsk->num == 0 && win != VROOT)
+		     ESetWindowBackgroundPixmap(VROOT, pmap);
 	       }
 	     else
 	       {
 		  ESetWindowBackground(win, pixel);
-		  if (dsk->num == 0 && win != VRoot.win)
-		     ESetWindowBackground(VRoot.win, pixel);
+		  if (dsk->num == 0 && win != VROOT)
+		     ESetWindowBackground(VROOT, pixel);
 	       }
 	     EClearWindow(win);
 	  }
@@ -557,8 +560,8 @@ DeskBackgroundConfigure(Desk * dsk)
 	   HintsSetRootInfo(EoGetWin(dsk), None, 0);
 
 	ESetWindowBackgroundPixmap(win, None);
-	if (dsk->num == 0 && win != VRoot.win)
-	   ESetWindowBackgroundPixmap(VRoot.win, None);
+	if (dsk->num == 0 && win != VROOT)
+	   ESetWindowBackgroundPixmap(VROOT, None);
      }
 }
 
@@ -644,7 +647,7 @@ DeskBackgroundRefresh(Desk * dsk, int why)
 
    if (bg && dsk->viewable)
       if (Conf.hints.set_xroot_info_on_root_window && dsk->num > 0)
-	 HintsSetRootInfo(VRoot.win, pmap, pixel);
+	 HintsSetRootInfo(VROOT, pmap, pixel);
 
    if (changed)
       ModulesSignal(ESIGNAL_BACKGROUND_CHANGE, dsk);
@@ -700,7 +703,7 @@ DeskResize(int desk, int w, int h)
 
    if (dsk->num != 0)
      {
-	x = (dsk->viewable) ? EoGetX(dsk) : VRoot.w;
+	x = (dsk->viewable) ? EoGetX(dsk) : WinGetW(VROOT);
 	EoMoveResize(dsk, x, 0, w, h);
      }
 #if USE_COMPOSITE
@@ -832,6 +835,8 @@ DesksResize(int w, int h)
    for (i = 0; i < Conf.desks.num; i++)
       DeskResize(i, w, h);
 
+   EdgeWindowsShow();
+
    ModulesSignal(ESIGNAL_DESK_RESIZE, NULL);
 }
 
@@ -1007,8 +1012,8 @@ DesktopAt(int x, int y)
    for (i = 0; i < Conf.desks.num; i++)
      {
 	dsk = _DeskGet(desks.order[i]);
-	if (x >= EoGetX(dsk) && x < (EoGetX(dsk) + VRoot.w) &&
-	    y >= EoGetY(dsk) && y < (EoGetY(dsk) + VRoot.h))
+	if (x >= EoGetX(dsk) && x < (EoGetX(dsk) + WinGetW(VROOT)) &&
+	    y >= EoGetY(dsk) && y < (EoGetY(dsk) + WinGetH(VROOT)))
 	   return _DeskGet(desks.order[i]);
      }
    return _DeskGet(0);
@@ -1124,7 +1129,7 @@ DeskHide(unsigned int desk)
    if (dsk->viewable)
       BackgroundTouch(dsk->bg.bg);
    dsk->viewable = 0;
-   EoMove(dsk, VRoot.w, 0);
+   EoMove(dsk, WinGetW(VROOT), 0);
 }
 
 static void
@@ -1207,20 +1212,20 @@ DeskGoto(Desk * dsk)
 		    {
 		    default:
 		    case 0:
-		       x = VRoot.w;
+		       x = WinGetW(VROOT);
 		       y = 0;
 		       break;
 		    case 1:
-		       x = -VRoot.w;
+		       x = -WinGetW(VROOT);
 		       y = 0;
 		       break;
 		    case 2:
 		       x = 0;
-		       y = VRoot.h;
+		       y = WinGetH(VROOT);
 		       break;
 		    case 3:
 		       x = 0;
-		       y = -VRoot.h;
+		       y = -WinGetH(VROOT);
 		       break;
 		    }
 		  DeskMove(dsk, x, y);
@@ -1287,7 +1292,7 @@ DeskRaise(unsigned int desk)
 
    ModulesSignal(ESIGNAL_DESK_SWITCH_DONE, NULL);
 
-   ESync();
+   ESync(ESYNC_DESKS);
 }
 
 static void
@@ -1323,7 +1328,7 @@ DeskLower(unsigned int desk)
    HintsSetCurrentDesktop();
 
    EUngrabServer();
-   ESync();
+   ESync(ESYNC_DESKS);
 }
 
 #if 0				/* Unused */
@@ -1434,10 +1439,9 @@ DeskRestack(Desk * dsk)
 		   EXWindowGetParent(wl[i]));
      }
 
-   XRestackWindows(disp, wl, tot);
+   EXRestackWindows(wl, tot);
 
-   if (wl)
-      Efree(wl);
+   Efree(wl);
 
  done:
    if (dsk->stack.update_client_list)
@@ -1625,8 +1629,8 @@ DeskCurrentGotoArea(int ax, int ay)
 
    ModulesSignal(ESIGNAL_AREA_SWITCH_START, NULL);
 
-   dx = VRoot.w * (ax - pax);
-   dy = VRoot.h * (ay - pay);
+   dx = WinGetW(VROOT) * (ax - pax);
+   dy = WinGetH(VROOT) * (ay - pay);
 
    if (dx < 0)
       SoundPlay("SOUND_MOVE_AREA_LEFT");
@@ -1733,8 +1737,7 @@ static void
 ButtonProxySendEvent(XEvent * ev)
 {
    if (Mode.button_proxy_win)
-      XSendEvent(disp, Mode.button_proxy_win, False, SubstructureNotifyMask,
-		 ev);
+      EXSendEvent(Mode.button_proxy_win, SubstructureNotifyMask, ev);
 }
 
 static void
@@ -1744,8 +1747,8 @@ DeskDragStart(int desk)
 
    dsk = _DeskGet(desk);
 
-   desks.drag_x0 = Mode.events.x - EoGetX(dsk);
-   desks.drag_y0 = Mode.events.y - EoGetY(dsk);
+   desks.drag_x0 = Mode.events.cx - EoGetX(dsk);
+   desks.drag_y0 = Mode.events.cy - EoGetY(dsk);
 
    Mode.mode = MODE_DESKDRAG;
 }
@@ -1761,8 +1764,8 @@ DeskDragMotion(Desk * dsk)
 {
    int                 x, y;
 
-   x = Mode.events.x - desks.drag_x0;
-   y = Mode.events.y - desks.drag_y0;
+   x = Mode.events.mx - desks.drag_x0;
+   y = Mode.events.my - desks.drag_y0;
 
    switch (Conf.desks.dragdir)
      {
@@ -1861,41 +1864,28 @@ DeskEventButtonRelease(Desk * dsk, XEvent * ev)
 static void
 DeskRootResize(int root, int w, int h)
 {
-   int                 ww, hh;
-
    if (EDebug(EDBUG_TYPE_DESKS))
       Eprintf("DeskRootResize %d %dx%d\n", root, w, h);
 
    if (root)
      {
-#if 0
-	RRoot.w = DisplayWidth(disp, RRoot.scr);
-	RRoot.h = DisplayHeight(disp, RRoot.scr);
-
-	if (w != RRoot.w || h != RRoot.h)
-	   Eprintf
-	      ("DeskRootResize (root): Screen size mismatch: root=%dx%d event=%dx%d\n",
-	       RRoot.w, RRoot.h, w, h);
-#endif
-	RRoot.w = w;
-	RRoot.h = h;
+	WinGetW(RROOT) = w;
+	WinGetH(RROOT) = h;
      }
 
-   if (w == VRoot.w && h == VRoot.h)
+   if (w == WinGetW(VROOT) && h == WinGetH(VROOT))
       return;
 
-   EWindowSync(VRoot.win);
+   EWindowSync(VROOT);
 
    /* Quit if size is not final */
-   EGetGeometry(VRoot.win, NULL, NULL, NULL, &ww, &hh, NULL, NULL);
-   if (w != ww || h != hh)
+   if (w != WinGetW(VROOT) || h != WinGetH(VROOT))
       return;
 
-   Mode.screen.w_old = VRoot.w;
-   Mode.screen.h_old = VRoot.h;
-   VRoot.w = w;
-   VRoot.h = h;
+   Mode.screen.w_old = WinGetW(VROOT);
+   Mode.screen.h_old = WinGetH(VROOT);
 
+   ScreenInit();
    DesksResize(w, h);
 }
 
@@ -1917,11 +1907,8 @@ DeskPropertyChange(Desk * dsk, XEvent * ev)
 	if (EDebug(EDBUG_TYPE_DESKS))
 	   Eprintf("DeskPropertyChange win=%#lx _XROOTPMAP_ID=%#lx\n",
 		   ev->xany.window, pmap);
-	if (ev->xany.window != VRoot.xwin)
+	if (ev->xany.window != WinGetXwin(VROOT))
 	   return;
-#if 0				/* FIXME - Remove? Or was this here for a reason? */
-	dsk->bg.pmap_set = pmap;
-#endif
 	if (pmap == dsk->bg.pmap)
 	   return;
 	if (pmap == Mode.root.ext_pmap)
@@ -1935,7 +1922,7 @@ DeskPropertyChange(Desk * dsk, XEvent * ev)
 	if (EDebug(EDBUG_TYPE_DESKS))
 	   Eprintf("DeskPropertyChange win=%#lx _XROOTCOLOR_PIXEL\n",
 		   ev->xany.window);
-	if (ev->xany.window != VRoot.xwin)
+	if (ev->xany.window != WinGetXwin(VROOT))
 	   return;
      }
 }
@@ -1963,17 +1950,17 @@ DeskHandleEvents(Win win __UNUSED__, XEvent * ev, void *prm)
 
      case MotionNotify:
 	/* Motion over desk buttons doesn't go here - We probably don't care much. */
-	DesksSetCurrent(DesktopAt(Mode.events.x, Mode.events.y));
+	DesksSetCurrent(DesktopAt(Mode.events.mx, Mode.events.my));
 	TooltipsSetPending(1, DeskGetAclass, dsk);
 	break;
 
      case ConfigureNotify:
-	if (ev->xconfigure.window == VRoot.xwin)
+	if (ev->xconfigure.window == WinGetXwin(VROOT))
 	   DeskRootResize(0, ev->xconfigure.width, ev->xconfigure.height);
 	break;
 
      case PropertyNotify:
-	if (ev->xany.window == VRoot.xwin)
+	if (ev->xany.window == WinGetXwin(VROOT))
 	   DeskPropertyChange(dsk, ev);
 	break;
 
@@ -2015,7 +2002,7 @@ DeskDragdirSet(const char *params)
    for (i = 1; i < Conf.desks.num; i++)
      {
 	dsk = _DeskGet(i);
-	EoMove(dsk, (dsk->viewable) ? 0 : VRoot.w, 0);
+	EoMove(dsk, (dsk->viewable) ? 0 : WinGetW(VROOT), 0);
      }
    DesksControlsRefresh();
 }
@@ -2077,7 +2064,9 @@ doDragbarLengthSet(EWin * edummy, const char *params)
      }
    return 0;
 }
+#endif
 
+#if ENABLE_DESKRAY
 static int
 doDeskray(EWin * edummy, const char *params)
 {
@@ -2109,7 +2098,7 @@ doDeskray(EWin * edummy, const char *params)
      }
    return 0;
 }
-#endif
+#endif /* ENABLE_DESKRAY */
 
 static void
 DesksInit(void)
@@ -2442,7 +2431,6 @@ CB_ConfigureAreas(Dialog * d __UNUSED__, int val, void *data __UNUSED__)
 	if (tmp_edge_resist < 1)
 	   tmp_edge_resist = 1;
 	Conf.desks.edge_flip_resistance = tmp_edge_resist;
-	EdgeWindowsShow();
      }
    autosave();
 }
@@ -2476,20 +2464,12 @@ CB_AreaDisplayRedraw(Dialog * d __UNUSED__, int val, void *data)
 	ImageClass         *ic;
 
 	ic = ImageclassFind("SETTINGS_AREA_AREA", 0);
-	if (ic)
-	   ImageclassApply(ic, win, 0, 0, STATE_NORMAL, ST_SOLID);
+	ImageclassApply(ic, win, 0, 0, STATE_NORMAL, ST_SOLID);
+
 	awin = ECreateWindow(win, 0, 0, 18, 14, 0);
 	ic = ImageclassFind("SETTINGS_AREADESK_AREA", 0);
-	if (ic)
-	  {
-	     Pixmap              pmap;
+	ImageclassApply(ic, awin, 0, 0, STATE_NORMAL, ST_SOLID);
 
-	     pmap = ImageclassApplySimple(ic, awin, None, STATE_NORMAL,
-					  0, 0, 18, 14);
-	     ESetWindowBackgroundPixmap(awin, pmap);
-	     EFreePixmap(pmap);
-	  }
-	EClearWindow(awin);
 	called = 1;
      }
    EMoveResizeWindow(awin, ((w / 2) - (9 * tmp_area_x)),
