@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -34,8 +34,7 @@
 
 #define DEBUG_DIALOGS 0
 
-typedef struct
-{
+typedef struct {
    char                horizontal;
 
    char                numeric;
@@ -71,34 +70,29 @@ typedef struct
    Win                 border_win;
 } DItemSlider;
 
-typedef struct
-{
+typedef struct {
    Win                 area_win;
    int                 w, h;
    DialogItemCallbackFunc *init_func;
    DialogItemCallbackFunc *event_func;
 } DItemArea;
 
-typedef struct
-{
+typedef struct {
    Win                 check_win;
    int                 check_orig_w, check_orig_h;
    char                onoff;
    char               *onoff_ptr;
 } DItemCheckButton;
 
-typedef struct
-{
+typedef struct {
    char               *image;
 } DItemImage;
 
-typedef struct
-{
+typedef struct {
    char                horizontal;
 } DItemSeparator;
 
-typedef struct
-{
+typedef struct {
    int                 num_columns;
    char                border;
    char                homogenous_h;
@@ -107,8 +101,7 @@ typedef struct
    DItem             **items;
 } DItemTable;
 
-typedef struct
-{
+typedef struct {
    Win                 radio_win;
    int                 radio_orig_w, radio_orig_h;
    char                onoff;
@@ -119,8 +112,7 @@ typedef struct
    DialogItemCallbackFunc *event_func;
 } DItemRadioButton;
 
-struct _ditem
-{
+struct _ditem {
    int                 type;
    Dialog             *dlg;
    DialogCallbackFunc *func;
@@ -140,8 +132,7 @@ struct _ditem
    int                 x, y, w, h;
    Win                 win;
    char               *text;
-   union
-   {
+   union {
       DItemCheckButton    check_button;
       DItemTable          table;
       DItemImage          image;
@@ -149,8 +140,7 @@ struct _ditem
       DItemRadioButton    radio_button;
       DItemSlider         slider;
       DItemArea           area;
-   }
-   item;
+   } item;
 
    char                realized;
    char                update;
@@ -160,16 +150,14 @@ struct _ditem
    char                clicked;
 };
 
-typedef struct
-{
+typedef struct {
    KeyCode             key;
    DialogCallbackFunc *func;
    int                 val;
    void               *data;
 } DKeyBind;
 
-struct _dialog
-{
+struct _dialog {
    EWin               *ewin;
    Win                 win;
    int                 w, h;
@@ -225,14 +213,14 @@ DialogBindKey(Dialog * d, const char *key, DialogCallbackFunc * func, int val,
    d->keybindings[d->num_bindings - 1].val = val;
    d->keybindings[d->num_bindings - 1].func = func;
    d->keybindings[d->num_bindings - 1].data = data;
-   d->keybindings[d->num_bindings - 1].key =
-      XKeysymToKeycode(disp, XStringToKeysym(key));
+   d->keybindings[d->num_bindings - 1].key = EKeynameToKeycode(key);
 }
 
 void
 DialogKeybindingsDestroy(Dialog * d)
 {
-   _EFREE(d->keybindings);
+   Efree(d->keybindings);
+   d->keybindings = NULL;
    d->num_bindings = 0;
 }
 
@@ -250,7 +238,7 @@ DialogCreate(const char *name)
    ecore_list_append(dialog_list, d);
 
    d->name = Estrdup(name);
-   d->win = ECreateClientWindow(VRoot.win, -20, -20, 2, 2);
+   d->win = ECreateClientWindow(VROOT, -20, -20, 2, 2);
    EventCallbackRegister(d->win, 0, DialogHandleEvents, d);
 
    d->iclass = ImageclassAlloc("DIALOG", 1);
@@ -267,10 +255,8 @@ DialogDestroy(Dialog * d)
 {
    ecore_list_node_remove(dialog_list, d);
 
-   if (d->name)
-      Efree(d->name);
-   if (d->title)
-      Efree(d->title);
+   Efree(d->name);
+   Efree(d->title);
    DialogKeybindingsDestroy(d);
    if (d->item)
       DialogItemDestroy(d->item, 0);
@@ -298,8 +284,7 @@ DialogFind(const char *name)
 void
 DialogSetTitle(Dialog * d, const char *title)
 {
-   if (d->title)
-      Efree(d->title);
+   Efree(d->title);
    d->title = Estrdup(title);
    d->set_title = 1;
 }
@@ -440,7 +425,7 @@ DialogDrawButton(Dialog * d __UNUSED__, DButton * db)
    else
      {
 	ITApply(db->win, db->iclass, NULL, state, 0, 0,
-		ST_WIDGET, db->tclass, NULL, db->text);
+		ST_WIDGET, db->tclass, NULL, db->text, 0);
      }
 }
 #endif
@@ -473,12 +458,13 @@ DialogRedraw(Dialog * d)
 
    FreePmapMask(&(d->pmm_bg));
    ImageclassApplyCopy(d->iclass, d->win, d->w, d->h, 0, 0, STATE_NORMAL,
-		       &(d->pmm_bg), 0, ST_DIALOG);
+		       &(d->pmm_bg), IC_FLAG_NONE, ST_DIALOG);
    if (d->pmm_bg.pmap == None)
       return;
 
    EGetWindowBackgroundPixmap(d->win);
-   EXCopyArea(d->pmm_bg.pmap, WinGetPmap(d->win), 0, 0, d->w, d->h, 0, 0);
+   EXCopyAreaTiled(d->pmm_bg.pmap, None, WinGetPmap(d->win), 0, 0, d->w, d->h,
+		   0, 0);
 
    d->redraw = 1;
 
@@ -1167,16 +1153,10 @@ DialogRealizeItem(Dialog * d, DItem * di)
 	im = ThemeImageLoad(di->item.image.image);
 	if (im)
 	  {
-	     Pixmap              pmap = 0, mask = 0;
-
 	     EImageGetSize(im, &iw, &ih);
 	     di->win = ECreateWindow(d->win, 0, 0, iw, ih, 0);
 	     EMapWindow(di->win);
-	     EImageRenderPixmaps(im, di->win, 0, &pmap, &mask, 0, 0);
-	     ESetWindowBackgroundPixmap(di->win, pmap);
-	     EShapeCombineMask(di->win, ShapeBounding, 0, 0, mask, ShapeSet);
-	     EClearWindow(di->win);
-	     EImagePixmapFree(pmap);
+	     EImageApplyToWin(im, di->win, 0, 0, 0);
 	     EImageFree(im);
 	  }
 	di->w = iw;
@@ -1444,10 +1424,8 @@ DialogRealizeItem(Dialog * d, DItem * di)
 			  r++;
 		       }
 		  }
-		if (col_size)
-		   Efree(col_size);
-		if (row_size)
-		   Efree(row_size);
+		Efree(col_size);
+		Efree(row_size);
 	     }
 	}
 	break;
@@ -1597,7 +1575,7 @@ DialogDrawItem(Dialog * d, DItem * di)
 	      pad = ImageclassGetPadding(d->iclass);
 	      gcv.subwindow_mode = IncludeInferiors;
 	      gc = EXCreateGC(WinGetPmap(d->win), GCSubwindowMode, &gcv);
-	      XSetForeground(disp, gc, BlackPixel(disp, VRoot.scr));
+	      XSetForeground(disp, gc, Dpy.pixel_black);
 	      XDrawRectangle(disp, WinGetPmap(d->win), gc,
 			     pad->left + di->x, pad->top + di->y, di->w, di->h);
 	      EXFreeGC(gc);
@@ -1666,7 +1644,7 @@ DialogDrawItem(Dialog * d, DItem * di)
 	else if (!(di->hilited) && (di->clicked))
 	   state = STATE_CLICKED;
 	ITApply(di->win, di->iclass, NULL, state, 0, 0,
-		ST_WIDGET, di->tclass, NULL, di->text);
+		ST_WIDGET, di->tclass, NULL, di->text, 0);
 	break;
 
      case DITEM_AREA:
@@ -1738,8 +1716,8 @@ DialogDrawItem(Dialog * d, DItem * di)
 	if (!di->text)
 	   break;
 	if (!d->redraw)
-	   EXCopyArea(d->pmm_bg.pmap, WinGetPmap(d->win), di->x, di->y,
-		      di->w, di->h, di->x, di->y);
+	   EXCopyAreaTiled(d->pmm_bg.pmap, None, WinGetPmap(d->win),
+			   di->x, di->y, di->w, di->h, di->x, di->y);
 	TextDraw(di->tclass, d->win, WinGetPmap(d->win), 0, 0, state, di->text,
 		 x, di->y, w, 99999, 17, TextclassGetJustification(di->tclass));
 	break;
@@ -1755,8 +1733,7 @@ DialogUpdate(Dialog * d)
    if (d->item)
       DialogDrawItem(d, d->item);
    if (d->xu1 < d->xu2 && d->yu1 < d->yu2)
-      EClearArea(d->win, d->xu1, d->yu1, d->xu2 - d->xu1, d->yu2 - d->yu1,
-		 False);
+      EClearArea(d->win, d->xu1, d->yu1, d->xu2 - d->xu1, d->yu2 - d->yu1);
    d->update = 0;
    d->xu1 = d->yu1 = 99999;
    d->xu2 = d->yu2 = 0;
@@ -1804,8 +1781,7 @@ DialogItemsRealize(Dialog * d)
 void
 DialogItemSetText(DItem * di, const char *text)
 {
-   if (di->text)
-      Efree(di->text);
+   Efree(di->text);
    di->text = Estrdup(text);
 }
 
@@ -1881,8 +1857,7 @@ DialogItemSeparatorSetOrientation(DItem * di, char horizontal)
 void
 DialogItemImageSetFile(DItem * di, const char *image)
 {
-   if (di->item.image.image)
-      Efree(di->item.image.image);
+   Efree(di->item.image.image);
    di->item.image.image = Estrdup(image);
    di->fill_h = 0;
    di->fill_v = 0;
@@ -2006,9 +1981,7 @@ DialogItemTableEmpty(DItem * di)
    for (i = 0; i < di->item.table.num_items; i++)
       DialogItemDestroy(di->item.table.items[i], 1);
 
-   if (di->item.table.items)
-      Efree(di->item.table.items);
-
+   Efree(di->item.table.items);
    di->item.table.items = NULL;
    di->item.table.num_items = 0;
 }
@@ -2019,8 +1992,7 @@ DialogItemDestroy(DItem * di, int clean)
    if (di->type == DITEM_TABLE)
       DialogItemTableEmpty(di);
 
-   if (di->text)
-      Efree(di->text);
+   Efree(di->text);
 
    switch (di->type)
      {
@@ -2032,8 +2004,7 @@ DialogItemDestroy(DItem * di, int clean)
 	EDestroyWindow(di->item.check_button.check_win);
 	break;
      case DITEM_IMAGE:
-	if (di->item.image.image)
-	   Efree(di->item.image.image);
+	Efree(di->item.image.image);
 	break;
      case DITEM_RADIOBUTTON:
 	if (!clean)
@@ -2189,8 +2160,8 @@ DItemEventMotion(Win win __UNUSED__, DItem * di, XEvent * ev)
 	   break;
 	if (ev->xmotion.window == WinGetXwin(di->item.slider.knob_win))
 	  {
-	     dx = Mode.events.x - Mode.events.px;
-	     dy = Mode.events.y - Mode.events.py;
+	     dx = Mode.events.mx - Mode.events.px;
+	     dy = Mode.events.my - Mode.events.py;
 	     if (di->item.slider.horizontal)
 	       {
 		  di->item.slider.wanted_val += dx;
