@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -33,20 +33,15 @@
 #include "config.h"
 #include "e16-ecore_hints.h"
 
-#define _ATOM_GET(name) \
-   XInternAtom(_ecore_x_disp, name, False)
-
 /* Window property change actions (must match _NET_WM_STATE_... ones) */
 #define ECORE_X_PROP_LIST_REMOVE    0
 #define ECORE_X_PROP_LIST_ADD       1
 #define ECORE_X_PROP_LIST_TOGGLE    2
 
-extern Display     *disp;
-
-#define _ecore_x_disp disp
-
 #ifdef USE_ECORE_X
 
+extern Display     *_ecore_x_disp;
+
 void
 ecore_x_icccm_state_set_iconic(Ecore_X_Window win)
 {
@@ -67,6 +62,17 @@ ecore_x_icccm_state_set_withdrawn(Ecore_X_Window win)
 
 #else /* USE_ECORE_X */
 
+#define DEBUG_CHECK 0
+#if DEBUG_CHECK
+#include <assert.h>
+#endif
+#include "hints.h"
+#include "xwin.h"
+
+#define _ecore_x_disp disp
+
+#define N_ITEMS(x) (sizeof(x)/sizeof(x[0]))
+
 /*
  * General stuff
  */
@@ -640,55 +646,37 @@ ecore_x_window_prop_window_list_get(Ecore_X_Window win, Ecore_X_Atom atom,
 /*
  * ICCCM stuff
  */
-Ecore_X_Atom        ECORE_X_ATOM_WM_STATE;
-Ecore_X_Atom        ECORE_X_ATOM_WM_NAME;
-Ecore_X_Atom        ECORE_X_ATOM_WM_ICON_NAME;
-Ecore_X_Atom        ECORE_X_ATOM_WM_CLASS;
-Ecore_X_Atom        ECORE_X_ATOM_WM_WINDOW_ROLE;
-Ecore_X_Atom        ECORE_X_ATOM_WM_NORMAL_HINTS;
-Ecore_X_Atom        ECORE_X_ATOM_WM_HINTS;
-Ecore_X_Atom        ECORE_X_ATOM_WM_COMMAND;
-Ecore_X_Atom        ECORE_X_ATOM_WM_CLIENT_MACHINE;
-Ecore_X_Atom        ECORE_X_ATOM_WM_CLIENT_LEADER;
-Ecore_X_Atom        ECORE_X_ATOM_WM_TRANSIENT_FOR;
 
-Ecore_X_Atom        ECORE_X_ATOM_WM_COLORMAP_WINDOWS;
-
-Ecore_X_Atom        ECORE_X_ATOM_WM_CHANGE_STATE;
-
-Ecore_X_Atom        ECORE_X_ATOM_WM_PROTOCOLS;
-Ecore_X_Atom        ECORE_X_ATOM_WM_DELETE_WINDOW;
-Ecore_X_Atom        ECORE_X_ATOM_WM_TAKE_FOCUS;
+static const char  *const atoms_icccm_names[] = {
+/* *INDENT-OFF* */
 
+   /* ICCCM */
+   "WM_STATE",
+   "WM_WINDOW_ROLE",
+   "WM_CLIENT_LEADER",
+   "WM_COLORMAP_WINDOWS",
+   "WM_CHANGE_STATE",
+   "WM_PROTOCOLS",
+   "WM_DELETE_WINDOW",
+   "WM_TAKE_FOCUS",
 #if 0
-Ecore_X_Atom        ECORE_X_ATOM_WM_SAVE_YOURSELF;
+   "WM_SAVE_YOURSELF",
 #endif
 
+   /* Misc. */
+   "UTF8_STRING",
+
+/* *INDENT-ON* */
+};
+unsigned int        atoms_icccm[CHECK_COUNT_ICCCM];
+
 void
 ecore_x_icccm_init(void)
 {
-   ECORE_X_ATOM_WM_STATE = _ATOM_GET("WM_STATE");
-   ECORE_X_ATOM_WM_NAME = XA_WM_NAME;
-   ECORE_X_ATOM_WM_ICON_NAME = XA_WM_ICON_NAME;
-   ECORE_X_ATOM_WM_CLASS = XA_WM_CLASS;
-   ECORE_X_ATOM_WM_WINDOW_ROLE = _ATOM_GET("WM_WINDOW_ROLE");
-   ECORE_X_ATOM_WM_NORMAL_HINTS = XA_WM_NORMAL_HINTS;
-   ECORE_X_ATOM_WM_HINTS = XA_WM_HINTS;
-   ECORE_X_ATOM_WM_COMMAND = XA_WM_COMMAND;
-   ECORE_X_ATOM_WM_CLIENT_MACHINE = XA_WM_CLIENT_MACHINE;
-   ECORE_X_ATOM_WM_CLIENT_LEADER = _ATOM_GET("WM_CLIENT_LEADER");
-   ECORE_X_ATOM_WM_TRANSIENT_FOR = XA_WM_TRANSIENT_FOR;
-
-   ECORE_X_ATOM_WM_COLORMAP_WINDOWS = _ATOM_GET("WM_COLORMAP_WINDOWS");
-
-   ECORE_X_ATOM_WM_CHANGE_STATE = _ATOM_GET("WM_CHANGE_STATE");
-
-   ECORE_X_ATOM_WM_PROTOCOLS = _ATOM_GET("WM_PROTOCOLS");
-   ECORE_X_ATOM_WM_DELETE_WINDOW = _ATOM_GET("WM_DELETE_WINDOW");
-   ECORE_X_ATOM_WM_TAKE_FOCUS = _ATOM_GET("WM_TAKE_FOCUS");
-#if 0
-   ECORE_X_ATOM_WM_SAVE_YOURSELF = _ATOM_GET("WM_SAVE_YOURSELF");
+#if DEBUG_CHECK
+   assert(CHECK_COUNT_ICCCM == N_ITEMS(atoms_icccm));
 #endif
+   AtomListIntern(atoms_icccm_names, N_ITEMS(atoms_icccm), atoms_icccm);
 }
 
 static void
@@ -801,229 +789,121 @@ ecore_x_icccm_name_class_get(Ecore_X_Window win, char **name, char **clss)
 /*
  * _NET_WM hints (EWMH)
  */
-Ecore_X_Atom        ECORE_X_ATOM_UTF8_STRING;
 
-/* Window manager info */
-Ecore_X_Atom        ECORE_X_ATOM_NET_SUPPORTED;
-Ecore_X_Atom        ECORE_X_ATOM_NET_SUPPORTING_WM_CHECK;
+static const char  *const atoms_netwm_names[] = {
+/* *INDENT-OFF* */
 
-/* Desktop status/requests */
-Ecore_X_Atom        ECORE_X_ATOM_NET_NUMBER_OF_DESKTOPS;
-Ecore_X_Atom        ECORE_X_ATOM_NET_VIRTUAL_ROOTS;
-Ecore_X_Atom        ECORE_X_ATOM_NET_DESKTOP_NAMES;
-Ecore_X_Atom        ECORE_X_ATOM_NET_DESKTOP_GEOMETRY;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WORKAREA;
-Ecore_X_Atom        ECORE_X_ATOM_NET_CURRENT_DESKTOP;
-Ecore_X_Atom        ECORE_X_ATOM_NET_DESKTOP_VIEWPORT;
-Ecore_X_Atom        ECORE_X_ATOM_NET_SHOWING_DESKTOP;
+   /* Window manager info */
+   "_NET_SUPPORTED",
+   "_NET_SUPPORTING_WM_CHECK",
 
-Ecore_X_Atom        ECORE_X_ATOM_NET_ACTIVE_WINDOW;
-Ecore_X_Atom        ECORE_X_ATOM_NET_CLIENT_LIST;
-Ecore_X_Atom        ECORE_X_ATOM_NET_CLIENT_LIST_STACKING;
+   /* Desktop status/requests */
+   "_NET_NUMBER_OF_DESKTOPS",
+   "_NET_VIRTUAL_ROOTS",
+   "_NET_DESKTOP_GEOMETRY",
+   "_NET_DESKTOP_NAMES",
+   "_NET_DESKTOP_VIEWPORT",
+   "_NET_WORKAREA",
+   "_NET_CURRENT_DESKTOP",
+   "_NET_SHOWING_DESKTOP",
 
-/* Client window props/client messages */
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_NAME;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_VISIBLE_NAME;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ICON_NAME;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_VISIBLE_ICON_NAME;
+   "_NET_ACTIVE_WINDOW",
+   "_NET_CLIENT_LIST",
+   "_NET_CLIENT_LIST_STACKING",
 
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_DESKTOP;
+   /* Client window props/client messages */
+   "_NET_WM_NAME",
+   "_NET_WM_VISIBLE_NAME",
+   "_NET_WM_ICON_NAME",
+   "_NET_WM_VISIBLE_ICON_NAME",
 
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_WINDOW_TYPE;
+   "_NET_WM_DESKTOP",
 
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_WINDOW_TYPE_DESKTOP;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_WINDOW_TYPE_DOCK;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_WINDOW_TYPE_TOOLBAR;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_WINDOW_TYPE_MENU;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_WINDOW_TYPE_UTILITY;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_WINDOW_TYPE_SPLASH;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_WINDOW_TYPE_DIALOG;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_WINDOW_TYPE_NORMAL;
+   "_NET_WM_WINDOW_TYPE",
+   "_NET_WM_WINDOW_TYPE_DESKTOP",
+   "_NET_WM_WINDOW_TYPE_DOCK",
+   "_NET_WM_WINDOW_TYPE_TOOLBAR",
+   "_NET_WM_WINDOW_TYPE_MENU",
+   "_NET_WM_WINDOW_TYPE_UTILITY",
+   "_NET_WM_WINDOW_TYPE_SPLASH",
+   "_NET_WM_WINDOW_TYPE_DIALOG",
+   "_NET_WM_WINDOW_TYPE_NORMAL",
 
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STATE;
+   "_NET_WM_STATE",
+   "_NET_WM_STATE_MODAL",
+   "_NET_WM_STATE_STICKY",
+   "_NET_WM_STATE_MAXIMIZED_VERT",
+   "_NET_WM_STATE_MAXIMIZED_HORZ",
+   "_NET_WM_STATE_SHADED",
+   "_NET_WM_STATE_SKIP_TASKBAR",
+   "_NET_WM_STATE_SKIP_PAGER",
+   "_NET_WM_STATE_HIDDEN",
+   "_NET_WM_STATE_FULLSCREEN",
+   "_NET_WM_STATE_ABOVE",
+   "_NET_WM_STATE_BELOW",
+   "_NET_WM_STATE_DEMANDS_ATTENTION",
 
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STATE_MODAL;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STATE_STICKY;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STATE_MAXIMIZED_VERT;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STATE_MAXIMIZED_HORZ;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STATE_SHADED;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STATE_SKIP_TASKBAR;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STATE_SKIP_PAGER;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STATE_HIDDEN;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STATE_FULLSCREEN;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STATE_ABOVE;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STATE_BELOW;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STATE_DEMANDS_ATTENTION;
+   "_NET_WM_ALLOWED_ACTIONS",
+   "_NET_WM_ACTION_MOVE",
+   "_NET_WM_ACTION_RESIZE",
+   "_NET_WM_ACTION_MINIMIZE",
+   "_NET_WM_ACTION_SHADE",
+   "_NET_WM_ACTION_STICK",
+   "_NET_WM_ACTION_MAXIMIZE_HORZ",
+   "_NET_WM_ACTION_MAXIMIZE_VERT",
+   "_NET_WM_ACTION_FULLSCREEN",
+   "_NET_WM_ACTION_CHANGE_DESKTOP",
+   "_NET_WM_ACTION_CLOSE",
+   "_NET_WM_ACTION_ABOVE",
+   "_NET_WM_ACTION_BELOW",
 
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ALLOWED_ACTIONS;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ACTION_MOVE;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ACTION_RESIZE;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ACTION_MINIMIZE;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ACTION_SHADE;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ACTION_STICK;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ACTION_MAXIMIZE_HORZ;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ACTION_MAXIMIZE_VERT;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ACTION_FULLSCREEN;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ACTION_CHANGE_DESKTOP;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ACTION_CLOSE;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ACTION_ABOVE;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ACTION_BELOW;
+   "_NET_WM_STRUT",
+   "_NET_WM_STRUT_PARTIAL",
 
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STRUT;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_STRUT_PARTIAL;
+   "_NET_FRAME_EXTENTS",
 
-Ecore_X_Atom        ECORE_X_ATOM_NET_FRAME_EXTENTS;
+   "_NET_WM_ICON",
 
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ICON;
-
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_USER_TIME;
+   "_NET_WM_USER_TIME",
 
 #if 0				/* Not used */
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_ICON_GEOMETRY;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_PID;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_HANDLED_ICONS;
+   "_NET_WM_ICON_GEOMETRY",
+   "_NET_WM_PID",
+   "_NET_WM_HANDLED_ICONS",
 
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_PING;
+   "_NET_WM_PING",
 #endif
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_SYNC_REQUEST;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_SYNC_REQUEST_COUNTER;
+   "_NET_WM_SYNC_REQUEST",
+   "_NET_WM_SYNC_REQUEST_COUNTER",
 
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_WINDOW_OPACITY;
+   "_NET_WM_WINDOW_OPACITY",
 
-/* Misc window ops */
-Ecore_X_Atom        ECORE_X_ATOM_NET_CLOSE_WINDOW;
-Ecore_X_Atom        ECORE_X_ATOM_NET_MOVERESIZE_WINDOW;
-Ecore_X_Atom        ECORE_X_ATOM_NET_WM_MOVERESIZE;
-Ecore_X_Atom        ECORE_X_ATOM_NET_RESTACK_WINDOW;
+   /* Misc window ops */
+   "_NET_CLOSE_WINDOW",
+   "_NET_MOVERESIZE_WINDOW",
+   "_NET_WM_MOVERESIZE",
+   "_NET_RESTACK_WINDOW",
 
 #if 0				/* Not yet implemented */
-Ecore_X_Atom        ECORE_X_ATOM_NET_REQUEST_FRAME_EXTENTS;
+   "_NET_REQUEST_FRAME_EXTENTS",
 #endif
 
-/* Startup notification */
-Ecore_X_Atom        ECORE_X_ATOM_NET_STARTUP_ID;
-Ecore_X_Atom        ECORE_X_ATOM_NET_STARTUP_INFO_BEGIN;
-Ecore_X_Atom        ECORE_X_ATOM_NET_STARTUP_INFO;
+   /* Startup notification */
+   "_NET_STARTUP_ID",
+   "_NET_STARTUP_INFO_BEGIN",
+   "_NET_STARTUP_INFO",
+
+/* *INDENT-ON* */
+};
+unsigned int        atoms_netwm[CHECK_COUNT_NETWM];
 
 void
 ecore_x_netwm_init(void)
 {
-   ECORE_X_ATOM_UTF8_STRING = _ATOM_GET("UTF8_STRING");
-
-   ECORE_X_ATOM_NET_SUPPORTED = _ATOM_GET("_NET_SUPPORTED");
-   ECORE_X_ATOM_NET_SUPPORTING_WM_CHECK = _ATOM_GET("_NET_SUPPORTING_WM_CHECK");
-
-   ECORE_X_ATOM_NET_NUMBER_OF_DESKTOPS = _ATOM_GET("_NET_NUMBER_OF_DESKTOPS");
-   ECORE_X_ATOM_NET_VIRTUAL_ROOTS = _ATOM_GET("_NET_VIRTUAL_ROOTS");
-   ECORE_X_ATOM_NET_DESKTOP_GEOMETRY = _ATOM_GET("_NET_DESKTOP_GEOMETRY");
-   ECORE_X_ATOM_NET_DESKTOP_NAMES = _ATOM_GET("_NET_DESKTOP_NAMES");
-   ECORE_X_ATOM_NET_DESKTOP_VIEWPORT = _ATOM_GET("_NET_DESKTOP_VIEWPORT");
-   ECORE_X_ATOM_NET_WORKAREA = _ATOM_GET("_NET_WORKAREA");
-   ECORE_X_ATOM_NET_CURRENT_DESKTOP = _ATOM_GET("_NET_CURRENT_DESKTOP");
-   ECORE_X_ATOM_NET_SHOWING_DESKTOP = _ATOM_GET("_NET_SHOWING_DESKTOP");
-
-   ECORE_X_ATOM_NET_ACTIVE_WINDOW = _ATOM_GET("_NET_ACTIVE_WINDOW");
-   ECORE_X_ATOM_NET_CLIENT_LIST = _ATOM_GET("_NET_CLIENT_LIST");
-   ECORE_X_ATOM_NET_CLIENT_LIST_STACKING =
-      _ATOM_GET("_NET_CLIENT_LIST_STACKING");
-
-   ECORE_X_ATOM_NET_WM_NAME = _ATOM_GET("_NET_WM_NAME");
-   ECORE_X_ATOM_NET_WM_VISIBLE_NAME = _ATOM_GET("_NET_WM_VISIBLE_NAME");
-   ECORE_X_ATOM_NET_WM_ICON_NAME = _ATOM_GET("_NET_WM_ICON_NAME");
-   ECORE_X_ATOM_NET_WM_VISIBLE_ICON_NAME =
-      _ATOM_GET("_NET_WM_VISIBLE_ICON_NAME");
-
-   ECORE_X_ATOM_NET_WM_DESKTOP = _ATOM_GET("_NET_WM_DESKTOP");
-
-   ECORE_X_ATOM_NET_WM_WINDOW_TYPE = _ATOM_GET("_NET_WM_WINDOW_TYPE");
-
-   ECORE_X_ATOM_NET_WM_WINDOW_TYPE_DESKTOP =
-      _ATOM_GET("_NET_WM_WINDOW_TYPE_DESKTOP");
-   ECORE_X_ATOM_NET_WM_WINDOW_TYPE_DOCK = _ATOM_GET("_NET_WM_WINDOW_TYPE_DOCK");
-   ECORE_X_ATOM_NET_WM_WINDOW_TYPE_TOOLBAR =
-      _ATOM_GET("_NET_WM_WINDOW_TYPE_TOOLBAR");
-   ECORE_X_ATOM_NET_WM_WINDOW_TYPE_MENU = _ATOM_GET("_NET_WM_WINDOW_TYPE_MENU");
-   ECORE_X_ATOM_NET_WM_WINDOW_TYPE_UTILITY =
-      _ATOM_GET("_NET_WM_WINDOW_TYPE_UTILITY");
-   ECORE_X_ATOM_NET_WM_WINDOW_TYPE_SPLASH =
-      _ATOM_GET("_NET_WM_WINDOW_TYPE_SPLASH");
-   ECORE_X_ATOM_NET_WM_WINDOW_TYPE_DIALOG =
-      _ATOM_GET("_NET_WM_WINDOW_TYPE_DIALOG");
-   ECORE_X_ATOM_NET_WM_WINDOW_TYPE_NORMAL =
-      _ATOM_GET("_NET_WM_WINDOW_TYPE_NORMAL");
-
-   ECORE_X_ATOM_NET_WM_STATE = _ATOM_GET("_NET_WM_STATE");
-
-   ECORE_X_ATOM_NET_WM_STATE_MODAL = _ATOM_GET("_NET_WM_STATE_MODAL");
-   ECORE_X_ATOM_NET_WM_STATE_STICKY = _ATOM_GET("_NET_WM_STATE_STICKY");
-   ECORE_X_ATOM_NET_WM_STATE_MAXIMIZED_VERT =
-      _ATOM_GET("_NET_WM_STATE_MAXIMIZED_VERT");
-   ECORE_X_ATOM_NET_WM_STATE_MAXIMIZED_HORZ =
-      _ATOM_GET("_NET_WM_STATE_MAXIMIZED_HORZ");
-   ECORE_X_ATOM_NET_WM_STATE_SHADED = _ATOM_GET("_NET_WM_STATE_SHADED");
-   ECORE_X_ATOM_NET_WM_STATE_SKIP_TASKBAR =
-      _ATOM_GET("_NET_WM_STATE_SKIP_TASKBAR");
-   ECORE_X_ATOM_NET_WM_STATE_SKIP_PAGER = _ATOM_GET("_NET_WM_STATE_SKIP_PAGER");
-   ECORE_X_ATOM_NET_WM_STATE_HIDDEN = _ATOM_GET("_NET_WM_STATE_HIDDEN");
-   ECORE_X_ATOM_NET_WM_STATE_FULLSCREEN = _ATOM_GET("_NET_WM_STATE_FULLSCREEN");
-   ECORE_X_ATOM_NET_WM_STATE_ABOVE = _ATOM_GET("_NET_WM_STATE_ABOVE");
-   ECORE_X_ATOM_NET_WM_STATE_BELOW = _ATOM_GET("_NET_WM_STATE_BELOW");
-   ECORE_X_ATOM_NET_WM_STATE_DEMANDS_ATTENTION =
-      _ATOM_GET("_NET_WM_STATE_DEMANDS_ATTENTION");
-
-   ECORE_X_ATOM_NET_WM_ALLOWED_ACTIONS = _ATOM_GET("_NET_WM_ALLOWED_ACTIONS");
-   ECORE_X_ATOM_NET_WM_ACTION_MOVE = _ATOM_GET("_NET_WM_ACTION_MOVE");
-   ECORE_X_ATOM_NET_WM_ACTION_RESIZE = _ATOM_GET("_NET_WM_ACTION_RESIZE");
-   ECORE_X_ATOM_NET_WM_ACTION_MINIMIZE = _ATOM_GET("_NET_WM_ACTION_MINIMIZE");
-   ECORE_X_ATOM_NET_WM_ACTION_SHADE = _ATOM_GET("_NET_WM_ACTION_SHADE");
-   ECORE_X_ATOM_NET_WM_ACTION_STICK = _ATOM_GET("_NET_WM_ACTION_STICK");
-   ECORE_X_ATOM_NET_WM_ACTION_MAXIMIZE_HORZ =
-      _ATOM_GET("_NET_WM_ACTION_MAXIMIZE_HORZ");
-   ECORE_X_ATOM_NET_WM_ACTION_MAXIMIZE_VERT =
-      _ATOM_GET("_NET_WM_ACTION_MAXIMIZE_VERT");
-   ECORE_X_ATOM_NET_WM_ACTION_FULLSCREEN =
-      _ATOM_GET("_NET_WM_ACTION_FULLSCREEN");
-   ECORE_X_ATOM_NET_WM_ACTION_CHANGE_DESKTOP =
-      _ATOM_GET("_NET_WM_ACTION_CHANGE_DESKTOP");
-   ECORE_X_ATOM_NET_WM_ACTION_CLOSE = _ATOM_GET("_NET_WM_ACTION_CLOSE");
-   ECORE_X_ATOM_NET_WM_ACTION_ABOVE = _ATOM_GET("_NET_WM_ACTION_ABOVE");
-   ECORE_X_ATOM_NET_WM_ACTION_BELOW = _ATOM_GET("_NET_WM_ACTION_BELOW");
-
-   ECORE_X_ATOM_NET_WM_STRUT = _ATOM_GET("_NET_WM_STRUT");
-   ECORE_X_ATOM_NET_WM_STRUT_PARTIAL = _ATOM_GET("_NET_WM_STRUT_PARTIAL");
-
-   ECORE_X_ATOM_NET_FRAME_EXTENTS = _ATOM_GET("_NET_FRAME_EXTENTS");
-
-   ECORE_X_ATOM_NET_WM_ICON = _ATOM_GET("_NET_WM_ICON");
-
-   ECORE_X_ATOM_NET_WM_USER_TIME = _ATOM_GET("_NET_WM_USER_TIME");
-
-#if 0				/* Not used */
-   ECORE_X_ATOM_NET_WM_ICON_GEOMETRY = _ATOM_GET("_NET_WM_ICON_GEOMETRY");
-   ECORE_X_ATOM_NET_WM_PID = _ATOM_GET("_NET_WM_PID");
-   ECORE_X_ATOM_NET_WM_HANDLED_ICONS = _ATOM_GET("_NET_WM_HANDLED_ICONS");
-
-   ECORE_X_ATOM_NET_WM_PING = _ATOM_GET("_NET_WM_PING");
+#if DEBUG_CHECK
+   assert(CHECK_COUNT_NETWM == N_ITEMS(atoms_netwm));
 #endif
-   ECORE_X_ATOM_NET_WM_SYNC_REQUEST = _ATOM_GET("_NET_WM_SYNC_REQUEST");
-   ECORE_X_ATOM_NET_WM_SYNC_REQUEST_COUNTER =
-      _ATOM_GET("_NET_WM_SYNC_REQUEST_COUNTER");
-
-   ECORE_X_ATOM_NET_WM_WINDOW_OPACITY = _ATOM_GET("_NET_WM_WINDOW_OPACITY");
-
-   ECORE_X_ATOM_NET_CLOSE_WINDOW = _ATOM_GET("_NET_CLOSE_WINDOW");
-   ECORE_X_ATOM_NET_MOVERESIZE_WINDOW = _ATOM_GET("_NET_MOVERESIZE_WINDOW");
-   ECORE_X_ATOM_NET_WM_MOVERESIZE = _ATOM_GET("_NET_WM_MOVERESIZE");
-   ECORE_X_ATOM_NET_RESTACK_WINDOW = _ATOM_GET("_NET_RESTACK_WINDOW");
-
-#if 0				/* Not yet implemented */
-   ECORE_X_ATOM_NET_REQUEST_FRAME_EXTENTS =
-      _ATOM_GET("_NET_REQUEST_FRAME_EXTENTS");
-#endif
-
-   ECORE_X_ATOM_NET_STARTUP_ID = _ATOM_GET("_NET_STARTUP_ID");
-   ECORE_X_ATOM_NET_STARTUP_INFO_BEGIN = _ATOM_GET("_NET_STARTUP_INFO_BEGIN");
-   ECORE_X_ATOM_NET_STARTUP_INFO = _ATOM_GET("_NET_STARTUP_INFO");
+   AtomListIntern(atoms_netwm_names, N_ITEMS(atoms_netwm), atoms_netwm);
 }
 
 /*
