@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -91,15 +91,13 @@
 
 typedef struct _cmhook ECmWinInfo;
 
-struct _cmhook
-{
+struct _cmhook {
    EObj               *next;	/* Paint order */
    EObj               *prev;	/* Paint order */
    Pixmap              pixmap;
    int                 rcx, rcy, rcw, rch;
    int                 mode;
    unsigned            damaged:1;
-   unsigned            fading:1;
    unsigned            fadeout:1;
    unsigned            has_shadow:1;
    unsigned            have_shape:1;	/* Region validity - shape */
@@ -143,8 +141,7 @@ struct _cmhook
 #define ECM_OR_ON_MAPUNMAP   2
 #define ECM_OR_ON_CREATE     3
 
-static struct
-{
+static struct {
    char                enable;
    char                resize_fix_enable;
    char                use_name_pixmap;
@@ -152,27 +149,22 @@ static struct
    char                use_cow;
 #endif
    int                 mode;
-   struct
-   {
+   struct {
       int                 mode;
       int                 offset_x, offset_y;
-      struct
-      {
+      struct {
 	 int                 opacity;
 	 int                 radius;
       } blur;
-      struct
-      {
+      struct {
 	 int                 opacity;
       } sharp;
    } shadows;
-   struct
-   {
+   struct {
       char                enable;
       unsigned int        time;	/* Fading time, ms */
    } fading;
-   struct
-   {
+   struct {
       int                 mode;
       int                 opacity;
    } override_redirect;
@@ -186,13 +178,13 @@ static struct
 #define ECM_MODE_WINDOW 2
 #define ECM_MODE_AUTO   3
 
-static struct
-{
+static struct {
    int                 mode;
    Window              root;
 #if USE_COMPOSITE_OVERLAY_WINDOW
    Window              cow;
 #endif
+   Pixmap              pmap;	/* Compositing buffer */
    char                active;
    char                use_pixmap;
    char                reorder;
@@ -310,11 +302,12 @@ ERegionCreateFromRects(XRectangle * rectangles, int nrectangles)
 #endif
 
 static              XserverRegion
-ERegionCreateFromWindow(Window win)
+ERegionCreateFromWindow(Win win)
 {
    XserverRegion       rgn;
 
-   rgn = XFixesCreateRegionFromWindow(disp, win, WindowRegionBounding);
+   rgn =
+      XFixesCreateRegionFromWindow(disp, WinGetXwin(win), WindowRegionBounding);
 
 #if DEBUG_REGIONS
    n_rgn_c++;
@@ -503,15 +496,14 @@ EPictureCreateSolid(Bool argb, double a, double r, double g, double b)
 }
 
 static              Picture
-EPictureCreateBuffer(Window win, int w, int h, int depth, Visual * vis,
-		     Pixmap * ppmap)
+EPictureCreateBuffer(Win win, int w, int h, Pixmap * ppmap)
 {
    Picture             pict;
    Pixmap              pmap;
    XRenderPictFormat  *pictfmt;
 
-   pmap = XCreatePixmap(disp, win, w, h, depth);
-   pictfmt = XRenderFindVisualFormat(disp, vis);
+   pmap = XCreatePixmap(disp, WinGetXwin(win), w, h, WinGetDepth(win));
+   pictfmt = XRenderFindVisualFormat(disp, WinGetVisual(win));
    pict = XRenderCreatePicture(disp, pmap, pictfmt, 0, 0);
    if (ppmap)
       *ppmap = pmap;
@@ -558,8 +550,7 @@ ECompMgrMoveResizeFix(EObj * eo, int x, int y, int w, int h)
      }
 
    /* Resizing - grab old contents */
-   pict = EPictureCreateBuffer(EobjGetXwin(eo), wo, ho, WinGetDepth(eo->win),
-			       WinGetVisual(eo->win), NULL);
+   pict = EPictureCreateBuffer(EobjGetWin(eo), wo, ho, NULL);
    XRenderComposite(disp, PictOpSrc, cw->picture, None, pict, 0, 0, 0, 0, 0, 0,
 		    wo, ho);
 
@@ -607,7 +598,7 @@ ECompMgrDeskConfigure(Desk * dsk)
      {
 	GC                  gc;
 
-	pmap = XCreatePixmap(disp, Mode_compmgr.root, 1, 1, VRoot.depth);
+	pmap = XCreatePixmap(disp, Mode_compmgr.root, 1, 1, WinGetDepth(VROOT));
 	gc = EXCreateGC(pmap, 0, NULL);
 	XSetClipMask(disp, gc, 0);
 	XSetFillStyle(disp, gc, FillSolid);
@@ -621,7 +612,7 @@ ECompMgrDeskConfigure(Desk * dsk)
      }
 
    pa.repeat = True;
-   pictfmt = XRenderFindVisualFormat(disp, VRoot.vis);
+   pictfmt = XRenderFindVisualFormat(disp, WinGetVisual(VROOT));
    pict = XRenderCreatePicture(disp, pmap, pictfmt, CPRepeat, &pa);
 
    if (pmap != dsk->bg.pmap)
@@ -761,8 +752,7 @@ ECompMgrDamageAll(void)
 
 static Picture      transBlackPicture;
 
-typedef struct
-{
+typedef struct {
    int                 size;
    double             *data;
 } conv;
@@ -1152,7 +1142,7 @@ ECompMgrWinSetShape(EObj * eo)
 
    if (cw->shape == None)
      {
-	cw->shape = ERegionCreateFromWindow(EobjGetXwin(eo));
+	cw->shape = ERegionCreateFromWindow(EobjGetWin(eo));
 
 	if (WinIsShaped(EobjGetWin(eo)))
 	  {
@@ -1311,12 +1301,12 @@ doECompMgrWinFade(void *data)
 
 #if DEBUG_OPACITY
    Eprintf("doECompMgrWinFade %#lx, %d/%d, %#x->%#x\n", EobjGetXwin(eo),
-	   cw->fading, cw->fadeout, cw->opacity, op);
+	   eo->fading, cw->fadeout, cw->opacity, op);
 #endif
-   if (!cw->fading)
+   if (!eo->fading)
       goto done;
 
-   cw->fading = cw->fadeout;
+   eo->fading = cw->fadeout;
 
    step = Conf_compmgr.fading.time / Conf.animation.step;
    if (step == 0)
@@ -1332,7 +1322,7 @@ doECompMgrWinFade(void *data)
 	if (op - cw->opacity > step)
 	  {
 	     op = cw->opacity + step;
-	     cw->fading = 1;
+	     eo->fading = 1;
 	  }
      }
    else
@@ -1340,7 +1330,7 @@ doECompMgrWinFade(void *data)
 	if (cw->opacity - op > step)
 	  {
 	     op = cw->opacity - step;
-	     cw->fading = 1;
+	     eo->fading = 1;
 	  }
      }
 
@@ -1349,7 +1339,7 @@ doECompMgrWinFade(void *data)
 #endif
    ECompMgrWinSetOpacity(eo, op);
 
-   if (cw->fading)
+   if (eo->fading)
       return 1;
 
    if (eo->type == EOBJ_TYPE_EWIN)
@@ -1367,7 +1357,7 @@ ECompMgrWinFade(EObj * eo, unsigned int op_from, unsigned int op_to)
 
    if (op_from == op_to && op_to == eo->opacity)
      {
-	if (cw->fading)
+	if (eo->fading)
 	   ECompMgrWinFadeEnd(eo, 0);
 	return;
      }
@@ -1376,20 +1366,18 @@ ECompMgrWinFade(EObj * eo, unsigned int op_from, unsigned int op_to)
       cw->anim_fade = AnimatorAdd(doECompMgrWinFade, eo);
    cw->opacity_to = op_to;
 
-   cw->fading = 1;
+   eo->fading = 1;
    ECompMgrWinSetOpacity(eo, op_from);
 }
 
 static void
 ECompMgrWinFadeIn(EObj * eo)
 {
-   ECmWinInfo         *cw = eo->cmhook;
-
 #if DEBUG_OPACITY
    Eprintf("ECompMgrWinFadeIn  %#lx %#x -> %#x\n", EobjGetXwin(eo), 0x10000000,
 	   eo->opacity);
 #endif
-   if (cw->fading)
+   if (eo->fading)
       ECompMgrWinFadeEnd(eo, 0);
 
    ECompMgrWinFade(eo, 0x10000000, eo->opacity);
@@ -1425,7 +1413,7 @@ ECompMgrWinFadeEnd(EObj * eo, int done)
 	ECompMgrDamageMergeObject(eo, cw->extents);
 	_ECM_SET_CLIP_CHANGED();
      }
-   cw->fading = 0;
+   eo->fading = 0;
    if (done)
      {
 	AnimatorDel(cw->anim_fade);
@@ -1524,7 +1512,7 @@ ECompMgrWinSetPicts(EObj * eo)
 	  {
 	     XserverRegion       clip;
 
-	     clip = ERegionCreateFromWindow(EobjGetXwin(eo));
+	     clip = ERegionCreateFromWindow(EobjGetWin(eo));
 	     XFixesSetPictureClipRegion(disp, cw->picture, 0, 0, clip);
 	     ERegionDestroy(clip);
 	  }
@@ -1539,7 +1527,7 @@ ECompMgrWinNew(EObj * eo)
    if (!Mode_compmgr.active)	/* FIXME - Here? */
       return;
 
-   if (eo->inputonly || eo->win == VRoot.win)
+   if (eo->inputonly || eo->win == VROOT)
      {
 	eo->noredir = 1;
 	return;
@@ -1718,7 +1706,8 @@ ECompMgrWinReparent(EObj * eo, Desk * dsk, int change_xy)
    ECmWinInfo         *cw = eo->cmhook;
 
    D1printf("ECompMgrWinReparent %#lx %#lx d=%d->%d x,y=%d,%d %d\n",
-	    EobjGetXwin(eo), cw->extents, eo->desk->num, dsk->num,
+	    EobjGetXwin(eo), cw->extents,
+	    (eo->desk) ? (int)eo->desk->num : -1, dsk->num,
 	    EobjGetX(eo), EobjGetY(eo), change_xy);
 
    if (!eo->shown)
@@ -1794,7 +1783,7 @@ ECompMgrWinDel(EObj * eo)
 
    D1printf("ECompMgrWinDel %#lx\n", EobjGetXwin(eo));
 
-   if (cw->fading)
+   if (eo->fading)
       ECompMgrWinFadeEnd(eo, 1);
 
    EventCallbackUnregister(eo->win, 0, ECompMgrHandleWindowEvent, eo);
@@ -1825,7 +1814,8 @@ ECompMgrWinDel(EObj * eo)
    REGION_DESTROY(cw->extents);
    REGION_DESTROY(cw->clip);
 
-   _EFREE(eo->cmhook);
+   Efree(eo->cmhook);
+   eo->cmhook = NULL;
 
    _ECM_SET_STACK_CHANGED();
 }
@@ -1868,6 +1858,9 @@ ECompMgrWinDamage(EObj * eo, XEvent * ev __UNUSED__)
      }
    eo->serial = ev->xany.serial;
    ECompMgrDamageMergeObject(eo, parts);
+
+   if (eo->type == EOBJ_TYPE_EWIN)
+      ModulesSignal(ESIGNAL_EWIN_DAMAGE, eo);
 }
 
 static void
@@ -1946,7 +1939,7 @@ ECompMgrDetermineOrder(EObj * const *lst, int num, EObj ** first,
 	if (!cw)
 	   continue;
 
-	if ((!eo->shown && !cw->fading) || eo->desk != dsk)
+	if ((!eo->shown && !eo->fading) || eo->desk != dsk)
 	   continue;
 
 	/* Region of shaped window in screen coordinates */
@@ -1958,7 +1951,7 @@ ECompMgrDetermineOrder(EObj * const *lst, int num, EObj ** first,
 	   ECompMgrWinSetExtents(eo);
 
 	D3printf(" - %#lx desk=%d shown=%d fading=%d fadeout=%d\n",
-		 EobjGetXwin(eo), eo->desk->num, eo->shown, cw->fading,
+		 EobjGetXwin(eo), eo->desk->num, eo->shown, eo->fading,
 		 cw->fadeout);
 
 	if (eo->type == EOBJ_TYPE_DESK)
@@ -2278,7 +2271,7 @@ ECompMgrRepaint(void)
    D1printf("ECompMgrRepaint desk picture=%#lx\n", pict);
    XFixesSetPictureClipRegion(dpy, pbuf, 0, 0, region);
    XRenderComposite(dpy, PictOpSrc, pict, None, pbuf,
-		    0, 0, 0, 0, 0, 0, VRoot.w, VRoot.h);
+		    0, 0, 0, 0, 0, 0, WinGetW(VROOT), WinGetH(VROOT));
 #endif
 
    /* Paint trans windows and shadows bottom up */
@@ -2293,7 +2286,7 @@ ECompMgrRepaint(void)
      {
 	XFixesSetPictureClipRegion(dpy, pbuf, 0, 0, Mode_compmgr.damage);
 	XRenderComposite(dpy, PictOpSrc, pbuf, None, rootPicture,
-			 0, 0, 0, 0, 0, 0, VRoot.w, VRoot.h);
+			 0, 0, 0, 0, 0, 0, WinGetW(VROOT), WinGetH(VROOT));
      }
 
    Mode_compmgr.got_damage = 0;
@@ -2312,8 +2305,7 @@ static void
 ECompMgrRootBufferCreate(unsigned int w, unsigned int h)
 {
    /* Root buffer picture and pixmap */
-   rootBuffer = EPictureCreateBuffer(VRoot.xwin, w, h,
-				     VRoot.depth, VRoot.vis, &VRoot.pmap);
+   rootBuffer = EPictureCreateBuffer(VROOT, w, h, &Mode_compmgr.pmap);
 
    /* Screen region */
    Mode_compmgr.rgn_screen = ERegionCreateRect(0, 0, w, h);
@@ -2326,12 +2318,18 @@ static void
 ECompMgrRootBufferDestroy(void)
 {
    PICTURE_DESTROY(rootBuffer);
-   PIXMAP_DESTROY(VRoot.pmap);
+   PIXMAP_DESTROY(Mode_compmgr.pmap);
 
    REGION_DESTROY(Mode_compmgr.rgn_screen);
    REGION_DESTROY(Mode_compmgr.rgn_clip);
 }
 
+Pixmap
+ECompMgrGetRootBuffer(void)
+{
+   return Mode_compmgr.pmap;
+}
+
 static void
 ECompMgrRootConfigure(void *prm __UNUSED__, XEvent * ev)
 {
@@ -2350,7 +2348,7 @@ ECompMgrRootExpose(void *prm __UNUSED__, XEvent * ev)
    static int          n_expose = 0;
    int                 more = ev->xexpose.count + 1;
 
-   if (ev->xexpose.window != VRoot.xwin)
+   if (ev->xexpose.window != WinGetXwin(VROOT))
       return;
 
    D1printf("ECompMgrRootExpose %d %d %d\n", n_expose, size_expose,
@@ -2392,8 +2390,7 @@ ECompMgrShadowsInit(int mode, int cleanup)
       OpacityFix(Conf_compmgr.shadows.sharp.opacity, 100);
    Mode_compmgr.opac_sharp = .01 * Conf_compmgr.shadows.sharp.opacity;
 
-   if (gaussianMap)
-      Efree(gaussianMap);
+   Efree(gaussianMap);
    gaussianMap = NULL;
 
    if (mode != ECM_SHADOWS_OFF)
@@ -2446,13 +2443,13 @@ ECompMgrStart(void)
    Conf_compmgr.override_redirect.opacity =
       OpacityFix(Conf_compmgr.override_redirect.opacity, 100);
 
-   ECompMgrRootBufferCreate(VRoot.w, VRoot.h);
+   ECompMgrRootBufferCreate(WinGetW(VROOT), WinGetH(VROOT));
 
-   Mode_compmgr.root = VRoot.xwin;
+   Mode_compmgr.root = WinGetXwin(VROOT);
 #if USE_COMPOSITE_OVERLAY_WINDOW
    if (Conf_compmgr.use_cow && !Mode.wm.window)
      {
-	Mode_compmgr.cow = XCompositeGetOverlayWindow(disp, VRoot.xwin);
+	Mode_compmgr.cow = XCompositeGetOverlayWindow(disp, WinGetXwin(VROOT));
 	if (Mode_compmgr.cow != None)
 	  {
 	     /* Ok, got the cow! */
@@ -2473,7 +2470,7 @@ ECompMgrStart(void)
 #endif
 
    pa.subwindow_mode = IncludeInferiors;
-   pictfmt = XRenderFindVisualFormat(disp, VRoot.vis);
+   pictfmt = XRenderFindVisualFormat(disp, WinGetVisual(VROOT));
    rootPicture =
       XRenderCreatePicture(disp, Mode_compmgr.root, pictfmt, CPSubwindowMode,
 			   &pa);
@@ -2485,18 +2482,19 @@ ECompMgrStart(void)
    switch (Mode_compmgr.mode)
      {
      case ECM_MODE_ROOT:
-	XCompositeRedirectSubwindows(disp, VRoot.xwin, CompositeRedirectManual);
+	XCompositeRedirectSubwindows(disp, WinGetXwin(VROOT),
+				     CompositeRedirectManual);
 #if USE_DESK_EXPOSE		/* FIXME - Remove? */
-	ESelectInputChange(VRoot.xwin, ExposureMask, 0);
+	ESelectInputChange(WinGetXwin(VROOT), ExposureMask, 0);
 #endif
 	break;
      case ECM_MODE_WINDOW:
 #if USE_DESK_EXPOSE		/* FIXME - Remove? */
-	ESelectInputChange(VRoot.xwin, ExposureMask, 0);
+	ESelectInputChange(WinGetXwin(VROOT), ExposureMask, 0);
 #endif
 	break;
      case ECM_MODE_AUTO:
-	XCompositeRedirectSubwindows(disp, VRoot.xwin,
+	XCompositeRedirectSubwindows(disp, WinGetXwin(VROOT),
 				     CompositeRedirectAutomatic);
 	break;
      }
@@ -2506,7 +2504,7 @@ ECompMgrStart(void)
    rgn_tmp = ERegionCreate();
    rgn_tmp2 = ERegionCreate();
 
-   EventCallbackRegister(VRoot.win, 0, ECompMgrHandleRootEvent, NULL);
+   EventCallbackRegister(VROOT, 0, ECompMgrHandleRootEvent, NULL);
 
    wm_cm_sel = SelectionAcquire("_NET_WM_CM_S", NULL, NULL);
 
@@ -2523,7 +2521,7 @@ ECompMgrStart(void)
 #endif
    _ECM_SET_CLIP_CHANGED();
    EUngrabServer();
-   ESync();
+   ESync(0);
 }
 
 static void
@@ -2571,14 +2569,15 @@ ECompMgrStop(void)
    REGION_DESTROY(rgn_tmp2);
 
    if (Mode_compmgr.mode == ECM_MODE_ROOT)
-      XCompositeUnredirectSubwindows(disp, VRoot.xwin, CompositeRedirectManual);
+      XCompositeUnredirectSubwindows(disp, WinGetXwin(VROOT),
+				     CompositeRedirectManual);
 
-   EventCallbackUnregister(VRoot.win, 0, ECompMgrHandleRootEvent, NULL);
+   EventCallbackUnregister(VROOT, 0, ECompMgrHandleRootEvent, NULL);
 
 #if USE_COMPOSITE_OVERLAY_WINDOW
    if (Mode_compmgr.cow != None)
      {
-	XCompositeReleaseOverlayWindow(disp, VRoot.xwin);
+	XCompositeReleaseOverlayWindow(disp, WinGetXwin(VROOT));
 	Mode_compmgr.cow = None;
      }
 #endif
@@ -2587,7 +2586,7 @@ ECompMgrStop(void)
    DesksBackgroundRefresh(NULL, DESK_BG_RECONFIGURE_ALL);
 #endif
    EUngrabServer();
-   ESync();
+   ESync(0);
 }
 
 void
@@ -2596,6 +2595,9 @@ ECompMgrConfigGet(cfg_composite * cfg)
    cfg->enable = Conf_compmgr.enable;
    cfg->shadow = Conf_compmgr.shadows.mode;
    cfg->fading = Conf_compmgr.fading.enable;
+   cfg->opacity_focused = Conf.opacity.focused;
+   cfg->opacity_unfocused = Conf.opacity.unfocused;
+   cfg->opacity_override = Conf_compmgr.override_redirect.opacity;
    cfg->fade_speed = 100 - (Conf_compmgr.fading.time / 10);
 }
 
@@ -2638,6 +2640,12 @@ ECompMgrConfigSet(const cfg_composite * cfg)
    Conf_compmgr.fading.enable = cfg->fading;
    Conf_compmgr.fading.time = (100 - cfg->fade_speed) * 10;
 
+   Conf.opacity.focused = cfg->opacity_focused;
+   Conf.opacity.unfocused = cfg->opacity_unfocused;
+   Conf_compmgr.override_redirect.opacity = cfg->opacity_override;
+
+   EobjsOpacityUpdate(Conf_compmgr.override_redirect.opacity);
+
    autosave();
 }
 
@@ -2723,13 +2731,13 @@ ECompMgrHandleRootEvent(Win win __UNUSED__, XEvent * ev, void *prm)
      case ReparentNotify:
      case EX_EVENT_REPARENT_GONE:
 	xwin = ev->xreparent.window;
-	if (ev->xreparent.parent == VRoot.xwin)
+	if (ev->xreparent.parent == WinGetXwin(VROOT))
 	   goto case_CreateNotify;
 	else
 	   goto case_DestroyNotify;
 
      case ConfigureNotify:
-	if (ev->xconfigure.window == VRoot.xwin)
+	if (ev->xconfigure.window == WinGetXwin(VROOT))
 	  {
 	     ECompMgrRootConfigure(prm, ev);
 	  }
