@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -25,7 +25,6 @@
 #include "aclass.h"
 #include "emodule.h"
 #include "events.h"
-#include "session.h"
 #include "timers.h"
 #include "tooltips.h"
 #include "xwin.h"
@@ -53,14 +52,16 @@
 static const char  *EventName(unsigned int type);
 #endif
 
-typedef struct
-{
+/*
+ * Server extension handling
+ */
+
+typedef struct {
    int                 event_base, error_base;
    int                 major, minor;
 } EServerExtData;
 
-typedef struct
-{
+typedef struct {
    const char         *name;
    unsigned int        ix;
    int                 (*query_ext) (Display * dpy, int *event, int *error);
@@ -129,11 +130,11 @@ ExtInitSS(int available)
      {
 	XScreenSaverInfo   *xssi = XScreenSaverAllocInfo();
 
-	XScreenSaverQueryInfo(disp, VRoot.xwin, xssi);
+	XScreenSaverQueryInfo(disp, WinGetXwin(VROOT), xssi);
 	Eprintf(" Screen saver window=%#lx\n", xssi->window);
 	XFree(xssi);
      }
-   XScreenSaverSelectInput(disp, VRoot.xwin,
+   XScreenSaverSelectInput(disp, WinGetXwin(VROOT),
 			   ScreenSaverNotifyMask | ScreenSaverCycleMask);
 }
 #endif
@@ -146,7 +147,7 @@ ExtInitRR(int available)
       return;
 
    /* Listen for RandR events */
-   XRRSelectInput(disp, VRoot.xwin, RRScreenChangeNotifyMask);
+   XRRSelectInput(disp, WinGetXwin(VROOT), RRScreenChangeNotifyMask);
 }
 #endif
 
@@ -199,6 +200,40 @@ ExtQuery(const EServerExt * ext)
       ext->init(available);
 }
 
+/*
+ * File descriptor handling
+ */
+
+struct _EventFdDesc {
+   const char         *name;
+   int                 fd;
+   void                (*handler) (void);
+};
+
+static int          nfds = 0;
+static EventFdDesc *pfds = NULL;
+
+EventFdDesc        *
+EventFdRegister(int fd, EventFdHandler * handler)
+{
+   nfds++;
+   pfds = EREALLOC(EventFdDesc, pfds, nfds);
+   pfds[nfds - 1].fd = fd;
+   pfds[nfds - 1].handler = handler;
+
+   return pfds + (nfds - 1);
+}
+
+void
+EventFdUnregister(EventFdDesc * efd)
+{
+   efd->fd = -1;
+}
+
+/*
+ * Event handling
+ */
+
 #define DOUBLE_CLICK_TIME 250	/* Milliseconds */
 
 void
@@ -219,6 +254,20 @@ EventsInit(void)
 	ExtData[XEXT_COMPOSITE].minor >= 2))
       Mode.server.extensions |= 1 << XEXT_CM_ALL;
 #endif
+
+   EventFdRegister(ConnectionNumber(disp), NULL);
+}
+
+int
+EventsGetXY(int *px, int *py)
+{
+   int                 ss;
+
+   ss = EQueryPointer(NULL, px, py, NULL, NULL);
+   Mode.events.cx = *px;
+   Mode.events.cy = *py;
+
+   return ss;
 }
 
 static void
@@ -228,13 +277,13 @@ ModeGetXY(Window rwin, int rx, int ry)
 
    if (Mode.wm.window)
      {
-	XTranslateCoordinates(disp, rwin, VRoot.xwin,
-			      rx, ry, &Mode.events.x, &Mode.events.y, &child);
+	XTranslateCoordinates(disp, rwin, WinGetXwin(VROOT), rx, ry,
+			      &Mode.events.cx, &Mode.events.cy, &child);
      }
    else
      {
-	Mode.events.x = rx;
-	Mode.events.y = ry;
+	Mode.events.cx = rx;
+	Mode.events.cy = ry;
      }
 }
 
@@ -258,13 +307,13 @@ HandleEvent(XEvent * ev)
 	Mode.events.time = ev->xkey.time;
 	ModeGetXY(ev->xbutton.root, ev->xkey.x_root, ev->xkey.y_root);
 #if 0				/* FIXME - Why? */
-	if (ev->xkey.root != VRoot.xwin)
+	if (ev->xkey.root != WinGetXwin(VROOT))
 	  {
 	     XSetInputFocus(disp, ev->xkey.root, RevertToPointerRoot,
 			    CurrentTime);
 	     ESync();
 	     ev->xkey.time = CurrentTime;
-	     XSendEvent(disp, ev->xkey.root, False, 0, ev);
+	     EXSendEvent(ev->xkey.root, 0, ev);
 	     return;
 	  }
 #endif
@@ -276,14 +325,16 @@ HandleEvent(XEvent * ev)
 	Mode.events.time = ev->xbutton.time;
 	ModeGetXY(ev->xbutton.root, ev->xbutton.x_root, ev->xbutton.y_root);
 	Mode.events.on_screen = ev->xbutton.same_screen;
-	TooltipsSetPending(0, NULL, NULL);
+	TooltipsHide();
 	goto do_stuff;
 
      case MotionNotify:
 	Mode.events.time = ev->xmotion.time;
-	Mode.events.px = Mode.events.x;
-	Mode.events.py = Mode.events.y;
+	Mode.events.px = Mode.events.mx;
+	Mode.events.py = Mode.events.my;
 	ModeGetXY(ev->xmotion.root, ev->xmotion.x_root, ev->xmotion.y_root);
+	Mode.events.mx = Mode.events.cx;
+	Mode.events.my = Mode.events.cy;
 	Mode.events.on_screen = ev->xmotion.same_screen;
 	break;
 
@@ -300,7 +351,7 @@ HandleEvent(XEvent * ev)
 	  }
 	ModeGetXY(ev->xcrossing.root, ev->xcrossing.x_root,
 		  ev->xcrossing.y_root);
-	TooltipsSetPending(0, NULL, NULL);
+	TooltipsHide();
 	goto do_stuff;
 
      case LeaveNotify:
@@ -314,7 +365,7 @@ HandleEvent(XEvent * ev)
 	  }
 	ModeGetXY(ev->xcrossing.root, ev->xcrossing.x_root,
 		  ev->xcrossing.y_root);
-	TooltipsSetPending(0, NULL, NULL);
+	TooltipsHide();
 	goto do_stuff;
 
      case PropertyNotify:
@@ -322,7 +373,7 @@ HandleEvent(XEvent * ev)
 	break;
 
       do_stuff:
-	if (ev->xany.window == VRoot.xwin)
+	if (ev->xany.window == WinGetXwin(VROOT))
 	   ActionclassesGlobalEvent(ev);
 	break;
      }
@@ -647,8 +698,7 @@ EventsMain(void)
    struct timeval      tval;
    double              time1, time2, dt;
    int                 count, pfetch;
-   int                 fdsize;
-   int                 xfd, smfd;
+   int                 fdsize, fd, i;
 
    time1 = GetTime();
 
@@ -691,12 +741,17 @@ EventsMain(void)
 	   continue;
 
 	FD_ZERO(&fdset);
-	xfd = ConnectionNumber(disp);
-	FD_SET(xfd, &fdset);
-	smfd = GetSMfd();
-	if (smfd >= 0)
-	   FD_SET(smfd, &fdset);
-	fdsize = MAX(xfd, smfd) + 1;
+	fdsize = -1;
+	for (i = 0; i < nfds; i++)
+	  {
+	     fd = pfds[i].fd;
+	     if (fd < 0)
+		continue;
+	     if (fdsize < fd)
+		fdsize = fd;
+	     FD_SET(fd, &fdset);
+	  }
+	fdsize++;
 
 	if (time2 > 0.)
 	  {
@@ -711,9 +766,8 @@ EventsMain(void)
 
 	if (EDebug(EDBUG_TYPE_EVENTS))
 	   Eprintf
-	      ("EventsMain - count=%d xfd=%d:%d smfd=%d:%d dt=%lf time2=%lf\n",
-	       count, xfd, FD_ISSET(xfd, &fdset), smfd,
-	       (smfd >= 0) ? FD_ISSET(smfd, &fdset) : 0, dt, time2);
+	      ("EventsMain - count=%d xfd=%d:%d dt=%lf time2=%lf\n",
+	       count, pfds[0].fd, FD_ISSET(pfds[0].fd, &fdset), dt, time2);
 
 	if (count == 0)
 	  {
@@ -722,11 +776,16 @@ EventsMain(void)
 	  }
 	else if (count > 0)
 	  {
-	     if ((smfd >= 0) && (FD_ISSET(smfd, &fdset)))
+	     /* Excluding X fd */
+	     for (i = 1; i < nfds; i++)
 	       {
-		  if (EDebug(EDBUG_TYPE_EVENTS))
-		     Eprintf("EventsMain - ICE\n");
-		  ProcessICEMSGS();
+		  fd = pfds[i].fd;
+		  if ((fd >= 0) && (FD_ISSET(fd, &fdset)))
+		    {
+		       if (EDebug(EDBUG_TYPE_EVENTS))
+			  Eprintf("Event fd %d\n", i);
+		       pfds[i].handler();
+		    }
 	       }
 	  }
      }
@@ -835,19 +894,26 @@ EventShow(const XEvent * ev)
      {
      case KeyPress:
      case KeyRelease:
-	goto case_common;
+	Eprintf("%s sub=%#lx x,y=%d,%d state=%#x keycode=%#x ss=%d\n", buf,
+		ev->xkey.subwindow, ev->xkey.x, ev->xkey.y,
+		ev->xkey.state, ev->xkey.keycode, ev->xkey.same_screen);
+	break;
      case ButtonPress:
      case ButtonRelease:
-	Eprintf("%s sub=%#lx state=%#x button=%#x\n", buf,
-		ev->xbutton.subwindow, ev->xbutton.state, ev->xbutton.button);
+	Eprintf("%s sub=%#lx x,y=%d,%d state=%#x button=%#x ss=%d\n", buf,
+		ev->xbutton.subwindow, ev->xbutton.x, ev->xbutton.y,
+		ev->xbutton.state, ev->xbutton.button, ev->xbutton.same_screen);
 	break;
      case MotionNotify:
-	Eprintf("%s sub=%#lx\n", buf, ev->xcrossing.subwindow);
+	Eprintf("%s sub=%#lx x,y=%d,%d rx,ry=%d,%d ss=%d\n", buf,
+		ev->xmotion.subwindow, ev->xmotion.x, ev->xmotion.y,
+		ev->xmotion.x_root, ev->xmotion.y_root,
+		ev->xmotion.same_screen);
 	break;
      case EnterNotify:
      case LeaveNotify:
-	Eprintf("%s sub=%#lx m=%s d=%s sscreen=%d focus=%d\n", buf,
-		ev->xcrossing.subwindow,
+	Eprintf("%s sub=%#lx x,y=%d,%d m=%s d=%s ss=%d focus=%d\n", buf,
+		ev->xcrossing.subwindow, ev->xcrossing.x, ev->xcrossing.y,
 		EventNotifyModeName(ev->xcrossing.mode),
 		EventNotifyDetailName(ev->xcrossing.detail),
 		ev->xcrossing.same_screen, ev->xcrossing.focus);
