@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -128,14 +128,14 @@ EwinFixPosition(EWin * ewin)
       return;
 
    x = EoGetX(ewin);
-   if ((x + ewin->border->border.left + 1) > VRoot.w)
-      x = VRoot.w - ewin->border->border.left - 1;
+   if ((x + ewin->border->border.left + 1) > WinGetW(VROOT))
+      x = WinGetW(VROOT) - ewin->border->border.left - 1;
    else if ((x + EoGetW(ewin) - ewin->border->border.right - 1) < 0)
       x = 0 - EoGetW(ewin) + ewin->border->border.right + 1;
 
    y = EoGetY(ewin);
-   if ((y + ewin->border->border.top + 1) > VRoot.h)
-      y = VRoot.h - ewin->border->border.top - 1;
+   if ((y + ewin->border->border.top + 1) > WinGetH(VROOT))
+      y = WinGetH(VROOT) - ewin->border->border.top - 1;
    else if ((y + EoGetH(ewin) - ewin->border->border.bottom - 1) < 0)
       y = 0 - EoGetH(ewin) + ewin->border->border.bottom + 1;
 
@@ -241,8 +241,8 @@ doEwinMoveResize(EWin * ewin, Desk * dsk, int x, int y, int w, int h, int flags)
    if (Mode.mode == MODE_NONE && Mode.move.check)
      {
 	/* Don't throw windows offscreen */
-	sw = VRoot.w;
-	sh = VRoot.h;
+	sw = WinGetW(VROOT);
+	sh = WinGetH(VROOT);
 	if (EoIsSticky(ewin))
 	  {
 	     xo = yo = 0;
@@ -309,6 +309,10 @@ doEwinMoveResize(EWin * ewin, Desk * dsk, int x, int y, int w, int h, int flags)
 	  {
 	     ICCCM_SizeMatch(ewin, w, h, &w, &h);
 	  }
+	if (w <= 0)
+	   w = 1;
+	if (h <= 0)
+	   h = 1;
 
 	if ((w != ewin->client.w) || (h != ewin->client.h))
 	   resize = 2;
@@ -415,8 +419,7 @@ doEwinMoveResize(EWin * ewin, Desk * dsk, int x, int y, int w, int h, int flags)
 			    EoGetY(lst[i]) + dy, 0, 0,
 			    flags & (MRF_DESK | MRF_MOVE |
 				     MRF_FLOAT | MRF_UNFLOAT));
-	if (lst)
-	   Efree(lst);
+	Efree(lst);
      }
 
    EwinDetermineArea(ewin);
@@ -584,8 +587,7 @@ EwinIconify(EWin * ewin)
    if (lst && call_depth == 1)
       GNOME_SetClientList();
 #endif
-   if (lst)
-      Efree(lst);
+   Efree(lst);
 
    EwinStateUpdate(ewin);
    HintsSetWindowState(ewin);
@@ -598,18 +600,19 @@ GetOnScreenPos(int x, int y, int w, int h, int *px, int *py)
 {
    int                 dx, dy;
 
-   if (x + w > 4 && x <= VRoot.w - 4 && y + h > 4 && y <= VRoot.h - 4)
+   if (x + w > 4 && x <= WinGetW(VROOT) - 4 &&
+       y + h > 4 && y <= WinGetH(VROOT) - 4)
       goto done;
 
    dx = w / 2;
    dy = h / 2;
-   x = (x + dx) % VRoot.w;
+   x = (x + dx) % WinGetW(VROOT);
    if (x < 0)
-      x += VRoot.w;
+      x += WinGetW(VROOT);
    x -= dx;
-   y = (y + dy) % VRoot.h;
+   y = (y + dy) % WinGetH(VROOT);
    if (y < 0)
-      y += VRoot.h;
+      y += WinGetH(VROOT);
    y -= dy;
 
  done:
@@ -659,8 +662,7 @@ EwinDeIconify1(EWin * ewin, int dx, int dy)
    if (lst && call_depth == 1)
       GNOME_SetClientList();
 #endif
-   if (lst)
-      Efree(lst);
+   Efree(lst);
 
    EwinStateUpdate(ewin);
    HintsSetWindowState(ewin);
@@ -714,13 +716,13 @@ EwinStick(EWin * ewin)
    /* Avoid "losing" windows made sticky while not in the current viewport */
    dx = EoGetW(ewin) / 2;
    dy = EoGetH(ewin) / 2;
-   x = (EoGetX(ewin) + dx) % VRoot.w;
+   x = (EoGetX(ewin) + dx) % WinGetW(VROOT);
    if (x < 0)
-      x += VRoot.w;
+      x += WinGetW(VROOT);
    x -= dx;
-   y = (EoGetY(ewin) + dy) % VRoot.h;
+   y = (EoGetY(ewin) + dy) % WinGetH(VROOT);
    if (y < 0)
-      y += VRoot.h;
+      y += WinGetH(VROOT);
    y -= dy;
 
    EoSetSticky(ewin, 1);
@@ -1304,8 +1306,7 @@ EwinOpFullscreen(EWin * ewin, int source __UNUSED__, int on)
 		  EoSetLayer(lst[i], lst[i]->save_fs.layer +
 			     EoGetLayer(ewin) - ewin->save_fs.layer);
 	       }
-	     if (lst)
-		Efree(lst);
+	     Efree(lst);
 	  }
 
 	EwinRaise(ewin);
@@ -1330,8 +1331,7 @@ EwinOpFullscreen(EWin * ewin, int source __UNUSED__, int on)
 	     lst = EwinListTransients(ewin, &num, 0);
 	     for (i = 0; i < num; i++)
 		EoSetLayer(lst[i], lst[i]->save_fs.layer);
-	     if (lst)
-		Efree(lst);
+	     Efree(lst);
 	  }
 	EoSetLayer(ewin, ewin->save_fs.layer);
 
@@ -1383,8 +1383,8 @@ void
 EwinMoveToArea(EWin * ewin, int ax, int ay)
 {
    DesksFixArea(&ax, &ay);
-   EwinMove(ewin, EoGetX(ewin) + (VRoot.w * (ax - ewin->area_x)),
-	    EoGetY(ewin) + (VRoot.h * (ay - ewin->area_y)));
+   EwinMove(ewin, EoGetX(ewin) + (WinGetW(VROOT) * (ax - ewin->area_x)),
+	    EoGetY(ewin) + (WinGetH(VROOT) * (ay - ewin->area_y)));
 }
 
 void
@@ -1424,8 +1424,7 @@ EwinOpClose(EWin * ewin, int source __UNUSED__)
 	ICCCM_Delete(gwins[i]);
 	SoundPlay("SOUND_WINDOW_CLOSE");
      }
-   if (gwins)
-      Efree(gwins);
+   Efree(gwins);
 }
 
 void
@@ -1520,8 +1519,7 @@ EwinOpRaiseLower(EWin * ewin)
 	   EwinRaise(gwins[j]);
      }
 
-   if (gwins)
-      Efree(gwins);
+   Efree(gwins);
 }
 #endif
 
@@ -1550,8 +1548,7 @@ EwinOpStick(EWin * ewin, int source __UNUSED__, int on)
 	     EwinStick(gwins[i]);
 	  }
      }
-   if (gwins)
-      Efree(gwins);
+   Efree(gwins);
 }
 
 void
@@ -1632,8 +1629,7 @@ EwinOpIconify(EWin * ewin, int source __UNUSED__, int on)
 	     EwinIconify(gwins[i]);
 	  }
      }
-   if (gwins)
-      Efree(gwins);
+   Efree(gwins);
 }
 
 void
@@ -1732,8 +1728,7 @@ EwinOpSetBorder(EWin * ewin, int source __UNUSED__, const char *name)
 	  }
 	SnapshotEwinUpdate(gwins[i], SNAP_USE_BORDER);
      }
-   if (gwins)
-      Efree(gwins);
+   Efree(gwins);
 }
 
 void
