@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -80,18 +80,6 @@ EwinGetContainerXwin(const EWin * ewin)
    return (win) ? WinGetXwin(win) : None;
 }
 
-static void
-EwinEventsConfigure(EWin * ewin, int mode)
-{
-   long                emask;
-
-   emask = (mode) ? ~((long)0) : ~(EnterWindowMask | LeaveWindowMask);
-
-   ESelectInput(EoGetWin(ewin), EWIN_TOP_EVENT_MASK & emask);
-   ESelectInput(EwinGetClientWin(ewin), ewin->client.event_mask & emask);
-   EwinBorderEventsConfigure(ewin, mode);
-}
-
 static EWin        *
 EwinCreate(int type)
 {
@@ -116,10 +104,10 @@ EwinCreate(int type)
 
    ewin->icccm.need_input = 1;
 
-   ewin->icccm.width.min = 0;
-   ewin->icccm.height.min = 0;
-   ewin->icccm.width.max = 65535;
-   ewin->icccm.height.max = 65535;
+   ewin->icccm.width_min = 0;
+   ewin->icccm.height_min = 0;
+   ewin->icccm.width_max = 65535;
+   ewin->icccm.height_max = 65535;
    ewin->icccm.base_w = 0;
    ewin->icccm.base_h = 0;
    ewin->icccm.w_inc = 1;
@@ -242,7 +230,7 @@ EwinManage(EWin * ewin)
    if (!frame)
      {
 	frame =
-	   ECreateObjectWindow(VRoot.win, ewin->client.x, ewin->client.y,
+	   ECreateObjectWindow(VROOT, ewin->client.x, ewin->client.y,
 			       ewin->client.w, ewin->client.h, 0, 0,
 			       EwinGetClientWin(ewin));
 	ewin->win_container =
@@ -274,6 +262,7 @@ EwinManage(EWin * ewin)
    EChangeWindowAttributes(EoGetWin(ewin), CWEventMask | CWDontPropagate, &att);
 
    ewin->client.event_mask = EWIN_CLIENT_EVENT_MASK;
+   ESelectInput(EwinGetClientWin(ewin), ewin->client.event_mask);
 
    if (EDebug(EDBUG_TYPE_EWINS))
       Eprintf("EwinManage %#lx frame=%#lx cont=%#lx st=%d\n",
@@ -344,17 +333,13 @@ EwinConfigure(EWin * ewin)
    EwinSetGeometry(ewin);	/* Calculate window geometry before border parts */
    EwinBorderSetTo(ewin, NULL);
 
-   EwinEventsConfigure(ewin, 1);
-
    if (!ewin->props.no_button_grabs)
       GrabButtonGrabs(ewin);
 
    if (ewin->state.shaded)
       EwinInstantShade(ewin, 1);
 
-   if (ewin->ewmh.opacity == 0)
-      ewin->ewmh.opacity = 0xffffffff;
-   EoChangeOpacity(ewin, ewin->ewmh.opacity);
+   EwinUpdateOpacity(ewin);
 
    HintsSetWindowState(ewin);
    HintsSetWindowOpacity(ewin);
@@ -407,8 +392,7 @@ EwinDestroy(EWin * ewin)
 	if (lst[i]->icccm.transient_count < 0)	/* Paranoia? */
 	   lst[i]->icccm.transient_count = 0;
      }
-   if (lst)
-      Efree(lst);
+   Efree(lst);
 
    EwinCleanup(ewin);
    EobjListOrderDel(&ewin->o);
@@ -417,26 +401,18 @@ EwinDestroy(EWin * ewin)
 
    HintsSetClientList();
 
-   if (ewin->icccm.wm_icon_name)
-      Efree(ewin->icccm.wm_icon_name);
-   if (ewin->icccm.wm_role)
-      Efree(ewin->icccm.wm_role);
-   if (ewin->icccm.wm_command)
-      Efree(ewin->icccm.wm_command);
-   if (ewin->icccm.wm_machine)
-      Efree(ewin->icccm.wm_machine);
-   if (ewin->ewmh.wm_name)
-      Efree(ewin->ewmh.wm_name);
-   if (ewin->ewmh.wm_icon_name)
-      Efree(ewin->ewmh.wm_icon_name);
-   if (ewin->ewmh.wm_icon)
-      Efree(ewin->ewmh.wm_icon);
-   if (ewin->bits)
-      Efree(ewin->bits);
-   if (ewin->session_id)
-      Efree(ewin->session_id);
+   Efree(ewin->icccm.wm_icon_name);
+   Efree(ewin->icccm.wm_role);
+   Efree(ewin->icccm.wm_command);
+   Efree(ewin->icccm.wm_machine);
+   Efree(ewin->ewmh.wm_name);
+   Efree(ewin->ewmh.wm_icon_name);
+   Efree(ewin->ewmh.wm_icon);
+   Efree(ewin->bits);
+   Efree(ewin->session_id);
    FreePmapMask(&ewin->mini_pmm);
    GroupsEwinRemove(ewin);
+
    Efree(ewin);
 }
 
@@ -463,23 +439,23 @@ DetermineEwinFloat(EWin * ewin, int dx, int dy)
 	  {
 	  case 0:
 	     if (((x + dx < 0) ||
-		  ((x + dx + w <= VRoot.w) &&
+		  ((x + dx + w <= WinGetW(VROOT)) &&
 		   (DesktopAt(xd + x + dx + w - 1, yd) != dsk))))
 		dofloat = 1;
 	     break;
 	  case 1:
-	     if (((x + dx + w > VRoot.w) ||
+	     if (((x + dx + w > WinGetW(VROOT)) ||
 		  ((x + dx >= 0) && (DesktopAt(xd + x + dx, yd) != dsk))))
 		dofloat = 1;
 	     break;
 	  case 2:
 	     if (((y + dy < 0) ||
-		  ((y + dy + h <= VRoot.h) &&
+		  ((y + dy + h <= WinGetH(VROOT)) &&
 		   (DesktopAt(xd, yd + y + dy + h - 1) != dsk))))
 		dofloat = 1;
 	     break;
 	  case 3:
-	     if (((y + dy + h > VRoot.h) ||
+	     if (((y + dy + h > WinGetH(VROOT)) ||
 		  ((y + dy >= 0) && (DesktopAt(xd, yd + y + dy) != dsk))))
 		dofloat = 1;
 	     break;
@@ -490,6 +466,7 @@ DetermineEwinFloat(EWin * ewin, int dx, int dy)
      }
 }
 
+#if 0				/* Remove? */
 EWin               *
 GetEwinByCurrentPointer(void)
 {
@@ -499,6 +476,7 @@ GetEwinByCurrentPointer(void)
 
    return EwinFindByFrame(child);
 }
+#endif
 
 EWin               *
 GetEwinPointerInClient(void)
@@ -508,7 +486,7 @@ GetEwinPointerInClient(void)
    int                 i, num;
    Desk               *dsk;
 
-   dsk = DesktopAt(Mode.events.x, Mode.events.y);
+   dsk = DesktopAt(Mode.events.cx, Mode.events.cy);
    EQueryPointer(EoGetWin(dsk), &px, &py, NULL, NULL);
 
    lst = EwinListGetForDesk(&num, dsk);
@@ -540,22 +518,10 @@ GetContextEwin(void)
 {
    EWin               *ewin;
 
-#if 0				/* FIXME - Remove? */
-   ewin = Mode.mouse_over_ewin;
-   if (ewin && ewin->type != EWIN_TYPE_MENU)
-      return ewin;
-#endif
-
    ewin = Mode.context_ewin;
    if (ewin)
       goto done;
 
-#if 0				/* FIXME - Remove? */
-   ewin = Mode.focuswin;
-   if (ewin && ewin->type != EWIN_TYPE_MENU)
-      goto done;
-#endif
-
    ewin = NULL;
 
  done:
@@ -704,7 +670,7 @@ EwinStateUpdate(EWin * ewin)
    SnapshotEwinUpdate(ewin, SNAP_USE_FLAGS);
 }
 
-void
+static void
 AddToFamily(EWin * ewin, Window xwin)
 {
    EWin               *ewin2;
@@ -744,10 +710,10 @@ AddToFamily(EWin * ewin, Window xwin)
    if (ewin->icccm.transient)
      {
 	if (ewin->icccm.transient_for == None ||
-	    ewin->icccm.transient_for == VRoot.xwin)
+	    ewin->icccm.transient_for == WinGetXwin(VROOT))
 	  {
 	     /* Group transient */
-	     ewin->icccm.transient_for = VRoot.xwin;
+	     ewin->icccm.transient_for = WinGetXwin(VROOT);
 #if 0				/* Maybe? */
 	     ewin->layer++;
 #endif
@@ -842,7 +808,7 @@ AddToFamily(EWin * ewin, Window xwin)
 	if (Conf.place.manual && !Mode.place.doing_manual &&
 	    !ewin->state.placed && !ewin->icccm.transient)
 	  {
-	     if (GrabPointerSet(VRoot.win, ECSR_GRAB, 0) == GrabSuccess)
+	     if (GrabPointerSet(VROOT, ECSR_GRAB, 0) == GrabSuccess)
 		manplace = 1;
 	  }
      }
@@ -853,35 +819,28 @@ AddToFamily(EWin * ewin, Window xwin)
 	/* Place the window below the mouse pointer */
 	if (Conf.place.manual_mouse_pointer)
 	  {
-	     int                 rx, ry;
-	     int                 newWinX = 0, newWinY = 0;
+	     int                 cx, cy;
 
 	     /* if the loser has manual placement on and the app asks to be on */
 	     /*  a desktop, then send E to that desktop so the user can place */
 	     /* the window there */
 	     DeskGoto(dsk);
 
-	     EQueryPointer(NULL, &rx, &ry, NULL, NULL);
-	     Mode.events.x = rx;
-	     Mode.events.y = ry;
+	     EventsGetXY(&cx, &cy);
 
 	     /* try to center the window on the mouse pointer */
-	     newWinX = rx;
-	     newWinY = ry;
-	     if (EoGetW(ewin))
-		newWinX -= EoGetW(ewin) / 2;
-	     if (EoGetH(ewin))
-		newWinY -= EoGetH(ewin) / 2;
+	     cx -= EoGetW(ewin) / 2;
+	     cy -= EoGetH(ewin) / 2;
 
 	     /* keep it all on this screen if possible */
-	     newWinX = MIN(newWinX, VRoot.w - EoGetW(ewin));
-	     newWinY = MIN(newWinY, VRoot.h - EoGetH(ewin));
-	     newWinX = MAX(newWinX, 0);
-	     newWinY = MAX(newWinY, 0);
+	     cx = MIN(cx, WinGetW(VROOT) - EoGetW(ewin));
+	     cy = MIN(cy, WinGetH(VROOT) - EoGetH(ewin));
+	     cx = MAX(cx, 0);
+	     cy = MAX(cy, 0);
 
 	     /* this works for me... */
-	     x = newWinX;
-	     y = newWinY;
+	     x = cx;
+	     y = cy;
 	  }
 	else if (ewin->ewmh.type.b.dialog)
 	  {
@@ -891,7 +850,7 @@ AddToFamily(EWin * ewin, Window xwin)
 	     ewin2 = NULL;
 	     if (EwinGetTransientFor(ewin) != None)
 		ewin2 = EwinFindByClient(EwinGetTransientFor(ewin));
-	     parent = (ewin2) ? EoGetWin(ewin) : VRoot.win;
+	     parent = (ewin2) ? EoGetWin(ewin) : VROOT;
 	     x = (WinGetW(parent) - EoGetW(ewin)) / 2;
 	     y = (WinGetH(parent) - EoGetH(ewin)) / 2;
 	  }
@@ -923,15 +882,15 @@ AddToFamily(EWin * ewin, Window xwin)
 	DeskGoto(dsk);
 
 	EQueryPointer(NULL, &rx, &ry, NULL, NULL);
-	Mode.events.x = rx;
-	Mode.events.y = ry;
+	Mode.events.cx = rx;
+	Mode.events.cy = ry;
 	ewin->state.placed = 1;
-	x = Mode.events.x + 1;
-	y = Mode.events.y + 1;
+	x = Mode.events.cx + 1;
+	y = Mode.events.cy + 1;
 	EwinMoveToDesktopAt(ewin, dsk, x, y);
 	EwinMove(ewin, x, y);
 	EwinShow(ewin);
-	GrabPointerSet(VRoot.win, ECSR_GRAB, 0);
+	GrabPointerSet(VROOT, ECSR_GRAB, 0);
 	Mode.place.doing_manual = 1;
 	EoSetFloating(ewin, 1);	/* Causes reparenting to root */
 	ActionMoveStart(ewin, 0, 0);
@@ -942,23 +901,23 @@ AddToFamily(EWin * ewin, Window xwin)
 	k = rand() % 4;
 	if (k == 0)
 	  {
-	     fx = (rand() % (VRoot.w)) - EoGetW(ewin);
+	     fx = (rand() % (WinGetW(VROOT))) - EoGetW(ewin);
 	     fy = -EoGetH(ewin);
 	  }
 	else if (k == 1)
 	  {
-	     fx = (rand() % (VRoot.w));
-	     fy = VRoot.h;
+	     fx = (rand() % (WinGetW(VROOT)));
+	     fy = WinGetH(VROOT);
 	  }
 	else if (k == 2)
 	  {
 	     fx = -EoGetW(ewin);
-	     fy = (rand() % (VRoot.h));
+	     fy = (rand() % (WinGetH(VROOT)));
 	  }
 	else
 	  {
-	     fx = VRoot.w;
-	     fy = (rand() % (VRoot.h)) - EoGetH(ewin);
+	     fx = WinGetW(VROOT);
+	     fy = (rand() % (WinGetH(VROOT))) - EoGetH(ewin);
 	  }
 	Mode.place.doing_slide = 1;
 	ewin->state.animated = 1;
@@ -1077,7 +1036,7 @@ EwinWithdraw(EWin * ewin, Win to)
 	/* Park the client window on the new root */
 	x = ewin->client.x;
 	y = ewin->client.y;
-	ETranslateCoordinates(EwinGetClientWin(ewin), VRoot.win,
+	ETranslateCoordinates(EwinGetClientWin(ewin), VROOT,
 			      -ewin->border->border.left,
 			      -ewin->border->border.top, &x, &y, &win);
 	EReparentWindow(EwinGetClientWin(ewin), to, x, y);
@@ -1085,7 +1044,7 @@ EwinWithdraw(EWin * ewin, Win to)
      }
    ICCCM_Withdraw(ewin);
 
-   ESync();
+   ESync(0);
    EUngrabServer();
 }
 
@@ -1231,7 +1190,7 @@ EwinEventUnmap(EWin * ewin, XEvent * ev)
    if (EoIsGone(ewin))
       return;
 
-   EwinWithdraw(ewin, VRoot.win);
+   EwinWithdraw(ewin, VROOT);
 }
 
 static void
@@ -1265,7 +1224,7 @@ EwinEventConfigureRequest(EWin * ewin, XEvent * ev)
 		y = ev->xconfigurerequest.y - EoGetY(EoGetDesk(ewin));
 #endif
 	  }
-	if (!EwinInhGetApp(ewin, move))
+	if (!EwinInhGetApp(ewin, size))
 	  {
 	     if (ev->xconfigurerequest.value_mask & CWWidth)
 		w = ev->xconfigurerequest.width;
@@ -1417,8 +1376,7 @@ EwinRaise(EWin * ewin)
    lst = EwinListTransients(ewin, &num, 1);
    for (i = 0; i < num; i++)
       EwinRaise(lst[i]);
-   if (lst)
-      Efree(lst);
+   Efree(lst);
 
    if (call_depth == 1)
      {
@@ -1453,8 +1411,7 @@ EwinLower(EWin * ewin)
    lst = EwinListTransientFor(ewin, &num);
    for (i = 0; i < num; i++)
       EwinLower(lst[i]);
-   if (lst)
-      Efree(lst);
+   Efree(lst);
 
    if (call_depth == 1)
      {
@@ -1641,7 +1598,7 @@ EwinIsOnScreen(const EWin * ewin)
    w = EoGetW(ewin);
    h = EoGetH(ewin);
 
-   if (x + w <= 0 || x >= VRoot.w || y + h <= 0 || y >= VRoot.h)
+   if (x + w <= 0 || x >= WinGetW(VROOT) || y + h <= 0 || y >= WinGetH(VROOT))
       return 0;
 
    return 1;
@@ -1660,8 +1617,8 @@ EwinRememberPositionSet(EWin * ewin)
    if (!EoIsSticky(ewin))
      {
 	DeskGetArea(EoGetDesk(ewin), &ax, &ay);
-	ewin->req_x += ax * VRoot.w;
-	ewin->req_y += ay * VRoot.h;
+	ewin->req_x += ax * WinGetW(VROOT);
+	ewin->req_y += ay * WinGetH(VROOT);
      }
 }
 
@@ -1678,8 +1635,8 @@ EwinRememberPositionGet(EWin * ewin, Desk * dsk, int *px, int *py)
    if (!EoIsSticky(ewin))
      {
 	DeskGetArea(dsk, &ax, &ay);
-	x -= ax * VRoot.w;
-	y -= ay * VRoot.h;
+	x -= ax * WinGetW(VROOT);
+	y -= ay * WinGetH(VROOT);
      }
 
    *px = x;
@@ -1758,8 +1715,8 @@ EwinReposition(EWin * ewin)
 
    wdo = Mode.screen.w_old;
    hdo = Mode.screen.h_old;
-   wdn = VRoot.w;
-   hdn = VRoot.h;
+   wdn = WinGetW(VROOT);
+   hdn = WinGetH(VROOT);
 
    x = EoGetX(ewin);
    y = EoGetY(ewin);
@@ -1824,11 +1781,22 @@ EwinReposition(EWin * ewin)
    EwinMove(ewin, xn, yn);
 }
 
-typedef union
+void
+EwinWarpTo(EWin * ewin)
 {
+   if (ewin == Mode.mouse_over_ewin)
+      return;
+
+   if (ewin->state.iconified)
+      return;
+
+   EXWarpPointer(EoGetXwin(ewin), EoGetW(ewin) / 2, EoGetH(ewin) / 2);
+   Mode.mouse_over_ewin = ewin;
+}
+
+typedef union {
    unsigned int        all;
-   struct
-   {
+   struct {
       unsigned char       rsvd;
       unsigned char       inh_app;
       unsigned char       inh_user;
@@ -1836,28 +1804,63 @@ typedef union
    } f;
 } EWinMiscFlags;
 
-unsigned int
-EwinFlagsEncode(const EWin * ewin)
+typedef union {
+   unsigned int        all;
+   struct {
+      unsigned            nua:1;
+      unsigned            ctf:1;
+      unsigned            nbg:1;
+      unsigned:           5;
+      unsigned:           16;
+      unsigned            no_fade:1;
+      unsigned            no_shadow:1;
+      unsigned:           6;
+   } f;
+} EWinMiscFlags2;
+
+void
+EwinFlagsEncode(const EWin * ewin, unsigned int *flags)
 {
    EWinMiscFlags       fm;
+   EWinMiscFlags2      fm2;
 
    fm.all = 0;
    fm.f.inh_app = ewin->inh_app.all;
    fm.f.inh_user = ewin->inh_user.all;
    fm.f.inh_wm = ewin->inh_wm.all;
 
-   return fm.all;
+   fm2.all = 0;
+   fm2.f.nua = ewin->props.never_use_area;
+   fm2.f.ctf = ewin->props.focusclick;
+   fm2.f.nbg = ewin->props.no_button_grabs;
+#if USE_COMPOSITE
+   fm2.f.no_fade = !EoGetFade(ewin);
+   fm2.f.no_shadow = !EoGetShadow(ewin);
+#endif
+
+   flags[0] = fm.all;
+   flags[1] = fm2.all;
 }
 
 void
-EwinFlagsDecode(EWin * ewin, unsigned int flags)
+EwinFlagsDecode(EWin * ewin, const unsigned int *flags)
 {
    EWinMiscFlags       fm;
+   EWinMiscFlags2      fm2;
 
-   fm.all = flags;
+   fm.all = flags[0];
    ewin->inh_app.all = fm.f.inh_app;
    ewin->inh_user.all = fm.f.inh_user;
    ewin->inh_wm.all = fm.f.inh_wm;
+
+   fm2.all = flags[1];
+   ewin->props.never_use_area = fm2.f.nua;
+   ewin->props.focusclick = fm2.f.ctf;
+   ewin->props.no_button_grabs = fm2.f.nbg;
+#if USE_COMPOSITE
+   EoSetFade(ewin, !fm2.f.no_fade);
+   EoSetShadow(ewin, !fm2.f.no_shadow);
+#endif
 }
 
 void
@@ -1872,6 +1875,12 @@ EwinUpdateOpacity(EWin * ewin)
       opacity = ewin->props.focused_opacity;
    if (opacity == 0)
       opacity = ewin->ewmh.opacity;
+   if (opacity == 0)
+      opacity = ewin->state.active ?
+	 OpacityFromPercent(Conf.opacity.focused) :
+	 OpacityFromPercent(Conf.opacity.unfocused);
+   if (opacity == 0)
+      opacity = 0xffffffff;	/* Fallback */
 
    EoChangeOpacity(ewin, opacity);
 }
@@ -1899,8 +1908,7 @@ EwinSlideIn(int val __UNUSED__, void *data)
 /*
  * Change requests
  */
-static struct
-{
+static struct {
    unsigned int        flags;
    EWin                ewin_old;
 } EWinChanges;
@@ -2008,7 +2016,7 @@ EwinListTransients(const EWin * ewin, int *num, int group)
 	if (ew == ewin)
 	   continue;
 
-	if (EwinGetTransientFor(ew) == VRoot.xwin &&
+	if (EwinGetTransientFor(ew) == WinGetXwin(VROOT) &&
 	    EwinGetWindowGroup(ew) == EwinGetWindowGroup(ewin))
 	  {
 	     lst = EREALLOC(EWin *, lst, j + 1);
@@ -2045,7 +2053,7 @@ EwinListTransientFor(const EWin * ewin, int *num)
 	/* Regular parent or if root trans, top level group members */
 	if ((EwinGetTransientFor(ewin) == EwinGetClientXwin(ew)) ||
 	    (!EwinIsTransient(ew) &&
-	     EwinGetTransientFor(ewin) == VRoot.xwin &&
+	     EwinGetTransientFor(ewin) == WinGetXwin(VROOT) &&
 	     EwinGetWindowGroup(ew) == EwinGetWindowGroup(ewin)))
 	  {
 	     lst = EREALLOC(EWin *, lst, j + 1);
@@ -2124,7 +2132,7 @@ EwinsManage(void)
 
    xwins = NULL;
    num = 0;
-   XQueryTree(disp, VRoot.xwin, &rt, &par, &xwins, &num);
+   XQueryTree(disp, WinGetXwin(VROOT), &rt, &par, &xwins, &num);
    if (!xwins)
       return;
 
@@ -2182,7 +2190,7 @@ EwinsSetFree(void)
 
 	/* This makes E determine the client window stacking at exit */
 	EwinInstantUnShade(ewin);
-	EReparentWindow(EwinGetClientWin(ewin), RRoot.win,
+	EReparentWindow(EwinGetClientWin(ewin), RROOT,
 			ewin->client.x, ewin->client.y);
      }
 }
@@ -2456,7 +2464,7 @@ EwinHandleEventsRoot(Win win __UNUSED__, XEvent * ev, void *prm __UNUSED__)
 static void
 EwinsInit(void)
 {
-   EventCallbackRegister(VRoot.win, 0, EwinHandleEventsRoot, NULL);
+   EventCallbackRegister(VROOT, 0, EwinHandleEventsRoot, NULL);
 }
 
 /*
