@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -34,11 +34,9 @@
 #define OPSRC_USER      2
 #define OPSRC_WM        3
 
-typedef union
-{
+typedef union {
    unsigned char       all:8;
-   struct
-   {
+   struct {
       unsigned char       rsvd:2;
       unsigned char       border:1;	/* W   */
       unsigned char       close:1;	/*  AU */
@@ -56,8 +54,7 @@ typedef union
 #define EwinInhGetWM(ewin, item)       (ewin->inh_wm.b.item)
 #define EwinInhSetWM(ewin, item, on)   ewin->inh_wm.b.item = (on)
 
-typedef struct
-{
+typedef struct {
    void                (*Init) (EWin * ewin);
    void                (*Layout) (EWin * ewin, int *px, int *py, int *pw,
 				  int *ph);
@@ -65,8 +62,7 @@ typedef struct
    void                (*Close) (EWin * ewin);
 } EWinOps;
 
-struct _ewin
-{
+struct _ewin {
    EObj                o;
    char                type;
    Win                 win_container;
@@ -76,8 +72,7 @@ struct _ewin
    const Border       *previous_border;
    EWinBit            *bits;
 
-   struct
-   {
+   struct {
       Win                 win;
       int                 x, y, w, h, bw;
       int                 grav;
@@ -85,8 +80,7 @@ struct _ewin
       long                event_mask;
    } client;
 
-   struct
-   {
+   struct {
       char                state;
       char                visibility;
       char                shaped;
@@ -132,8 +126,9 @@ struct _ewin
       unsigned            inhibit_actions:1;
       unsigned            inhibit_focus:1;
    } state;
-   struct
-   {
+   struct {
+      /* User config */
+      unsigned int        focused_opacity;
       unsigned            never_use_area:1;
       unsigned            ignorearrange:1;
       unsigned            skip_ext_task:1;
@@ -142,21 +137,23 @@ struct _ewin
       unsigned            skip_winlist:1;
       unsigned            focusclick:1;	/* Click to focus */
       unsigned            no_button_grabs:1;
-      unsigned            no_actions:1;
-      unsigned            no_resize_h:1;
-      unsigned            no_resize_v:1;
+
+      /* Derived from other properties */
+      unsigned            no_border:1;	/* Never apply border (MWM/netwm type) */
+      unsigned            no_resize_h:1;	/* ICCCM */
+      unsigned            no_resize_v:1;	/* ICCCM */
+
+      /* Internal */
+      unsigned            autosave:1;
       unsigned            donthide:1;	/* Don't hide on show desktop */
       unsigned            vroot:1;	/* Virtual root window */
-      unsigned            autosave:1;
-      unsigned            no_border:1;	/* Never apply border */
+      unsigned            no_actions:1;
       unsigned            focus_when_mapped:1;
-      unsigned int        focused_opacity;
    } props;
    EWinInhibit         inh_app;
    EWinInhibit         inh_user;
    EWinInhibit         inh_wm;
-   struct
-   {
+   struct {
       char               *wm_icon_name;
       char               *wm_role;
       char               *wm_command;
@@ -179,7 +176,8 @@ struct _ewin
       Window              client_leader;
 
       /* WM_NORMAL_HINTS */
-      Constraints         width, height;
+      int                 width_min, width_max;
+      int                 height_min, height_max;
       int                 base_w, base_h;
       int                 w_inc, h_inc;
       int                 grav;
@@ -187,8 +185,7 @@ struct _ewin
 
       char                is_group_leader;
    } icccm;
-   struct
-   {
+   struct {
       unsigned            valid:1;
       unsigned            decor_border:1;
       unsigned            decor_resizeh:1;
@@ -202,8 +199,7 @@ struct _ewin
       unsigned            func_maximize:1;
       unsigned            func_close:1;
    } mwm;
-   struct
-   {
+   struct {
       char               *wm_name;
       char               *wm_icon_name;
       unsigned int       *wm_icon, wm_icon_len;
@@ -213,11 +209,9 @@ struct _ewin
       XID                 sync_request_counter;
       long long           sync_request_count;
 #endif
-      union
-      {
+      union {
 	 unsigned char       all;
-	 struct
-	 {
+	 struct {
 	    unsigned            desktop:1;
 	    unsigned            dock:1;
 	    unsigned            toolbar:1;
@@ -229,18 +223,15 @@ struct _ewin
 	 } b;
       } type;
    } ewmh;
-   struct
-   {
+   struct {
       signed char         gravity;
       int                 ax, ay;	/* Current placed area */
       int                 gx, gy;	/* Distance to edge given by gravity */
    } place;
-   struct
-   {
+   struct {
       int                 left, right, top, bottom;
    } strut;
-   struct
-   {
+   struct {
       char                shape;
       char                border;
    } update;
@@ -259,13 +250,11 @@ struct _ewin
    int                 head;	/* Unused? */
 
    int                 vx, vy;	/* Position in virtual root */
-   struct
-   {				/* Saved state before maximization */
+   struct {			/* Saved state before maximization */
       int                 x, y;	/* Position */
       int                 w, h;	/* Size */
    } save_max;
-   struct
-   {				/* Saved state before fullscreen */
+   struct {			/* Saved state before fullscreen */
       int                 x, y;	/* Position */
       int                 w, h;	/* Size */
       int                 layer;	/* Layer */
@@ -365,7 +354,6 @@ void                EwinGetPosition(const EWin * ewin, int x, int y, int bw,
 void                EwinUpdateShapeInfo(EWin * ewin);
 void                EwinPropagateShapes(EWin * ewin);
 void                EwinStateUpdate(EWin * ewin);
-void                AddToFamily(EWin * ewin, Window win);
 EWin               *AddInternalToFamily(Win win, const char *bname, int type,
 					const EWinOps * ops, void *ptr);
 void                EwinReparent(EWin * ewin, Win parent);
@@ -384,12 +372,14 @@ void                EwinRememberPositionGet(EWin * ewin, Desk * dsk,
 					    int *px, int *py);
 void                EwinSetPlacementGravity(EWin * ewin, int x, int y);
 void                EwinReposition(EWin * ewin);
-unsigned int        EwinFlagsEncode(const EWin * ewin);
-void                EwinFlagsDecode(EWin * ewin, unsigned int flags);
+void                EwinFlagsEncode(const EWin * ewin, unsigned int *flags);
+void                EwinFlagsDecode(EWin * ewin, const unsigned int *flags);
 void                EwinUpdateOpacity(EWin * ewin);
 
 void                EwinChange(EWin * ewin, unsigned int flag);
 
+void                EwinWarpTo(EWin * ewin);
+
 EWin              **EwinListTransients(const EWin * ewin, int *num, int group);
 EWin              **EwinListTransientFor(const EWin * ewin, int *num);
 
@@ -403,7 +393,6 @@ void                SlideEwinTo(EWin * ewin, int fx, int fy, int tx, int ty,
 				int speed, int mode);
 void                SlideEwinsTo(EWin ** ewin, int *fx, int *fy, int *tx,
 				 int *ty, int num_wins, int speed, int mode);
-void                EwinFixPosition(EWin * ewin);
 void                EwinMove(EWin * ewin, int x, int y);
 void                EwinResize(EWin * ewin, int w, int h);
 void                EwinMoveResize(EWin * ewin, int x, int y, int w, int h);
@@ -464,9 +453,7 @@ void                EwinSyncRequestWait(EWin * ewin);
 
 /* moveresize.c */
 int                 ActionMoveStart(EWin * ewin, char constrained, int nogroup);
-int                 ActionMoveEnd(EWin * ewin);
 int                 ActionResizeStart(EWin * ewin, int hv);
-int                 ActionResizeEnd(EWin * ewin);
 
 int                 ActionsSuspend(void);
 int                 ActionsResume(void);
