@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -399,11 +399,8 @@ doFocusToEwin(EWin * ewin, int why)
 
    if (Conf.focus.warp_always)
       do_warp = 1;
-   if (do_warp && ewin != Mode.mouse_over_ewin)
-     {
-	EXWarpPointer(EoGetXwin(ewin), EoGetW(ewin) / 2, EoGetH(ewin) / 2);
-	Mode.mouse_over_ewin = ewin;
-     }
+   if (do_warp)
+      EwinWarpTo(ewin);
 
    RemoveTimerEvent("REVERSE_FOCUS_TIMEOUT");
    switch (why)
@@ -507,8 +504,8 @@ void
 FocusNewDeskBegin(void)
 {
    /* Freeze keyboard */
-   XGrabKeyboard(disp, VRoot.xwin, False, GrabModeAsync, GrabModeSync,
-		 CurrentTime);
+   XGrabKeyboard(disp, WinGetXwin(VROOT), False, GrabModeAsync,
+		 GrabModeSync, CurrentTime);
 
    focus_pending_new = NULL;
    doFocusToEwin(NULL, FOCUS_DESK_LEAVE);
@@ -517,8 +514,10 @@ FocusNewDeskBegin(void)
 void
 FocusNewDesk(void)
 {
+#if 0				/* Remove? */
    /* Set the mouse-over window */
    Mode.mouse_over_ewin = GetEwinByCurrentPointer();
+#endif
 
    FocusToEWin(NULL, FOCUS_DESK_ENTER);
 
@@ -532,8 +531,10 @@ FocusInit(void)
    /* Start focusing windows */
    FocusEnable(1);
 
+#if 0				/* Remove? */
    /* Set the mouse-over window */
    Mode.mouse_over_ewin = GetEwinByCurrentPointer();
+#endif
 
    focus_pending_why = 0;
    focus_pending_ewin = focus_pending_new = NULL;
@@ -563,7 +564,7 @@ FocusHandleEnter(EWin * ewin, XEvent * ev)
    if (!ewin)
      {
 	/* Entering root may mean entering this screen */
-	if (win == VRoot.xwin &&
+	if (win == WinGetXwin(VROOT) &&
 	    (ev->xcrossing.mode == NotifyNormal &&
 	     ev->xcrossing.detail != NotifyInferior))
 	  {
@@ -598,7 +599,7 @@ FocusHandleLeave(EWin * ewin __UNUSED__, XEvent * ev)
    Window              win = ev->xcrossing.window;
 
    /* Leaving root may mean entering other screen */
-   if (win == VRoot.xwin &&
+   if (win == WinGetXwin(VROOT) &&
        (ev->xcrossing.mode == NotifyNormal &&
 	ev->xcrossing.detail != NotifyInferior))
       FocusToEWin(NULL, FOCUS_DESK_LEAVE);
@@ -627,9 +628,9 @@ FocusHandleClick(EWin * ewin, Win win)
 	      EwinGetContainerXwin(ewin));
    if (win == EwinGetContainerWin(ewin))
      {
-	ESync();
+	ESync(ESYNC_FOCUS);
 	XAllowEvents(disp, ReplayPointer, CurrentTime);
-	ESync();
+	ESync(ESYNC_FOCUS);
      }
 }
 
