@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -41,14 +41,12 @@
 
 static Ecore_List  *group_list = NULL;
 
-static struct
-{
+static struct {
    GroupConfig         dflt;
    char                swapmove;
 } Conf_groups;
 
-static struct
-{
+static struct {
    Group              *current;
 } Mode_groups;
 
@@ -102,8 +100,8 @@ GroupDestroy(Group * g)
 
    if (g == Mode_groups.current)
       Mode_groups.current = NULL;
-   if (g->members)
-      Efree(g->members);
+   Efree(g->members);
+
    Efree(g);
 }
 
@@ -741,8 +739,7 @@ ChooseGroupDialog(EWin * ewin, const char *message, char group_select,
    DialogShowSimple(&DlgGroupChoose, (void *)message);
 }
 
-typedef struct
-{
+typedef struct {
    EWin               *ewin;
    GroupConfig         cfg;	/* Dialog data for current group */
    GroupConfig        *cfgs;	/* Work copy of ewin group cfgs */
