@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -32,27 +32,24 @@
 #include "tclass.h"
 #include "xwin.h"
 
+#define ENABLE_DESTROY 0	/* Broken */
+
 #if ENABLE_COLOR_MODIFIERS
-typedef struct
-{
+typedef struct {
    int                 num;
    unsigned char      *px;
    unsigned char      *py;
    unsigned char       map[256];
-}
-ModCurve;
+} ModCurve;
 
-typedef struct
-{
+typedef struct {
    char               *name;
    ModCurve            red, green, blue;
    unsigned int        ref_count;
-}
-ColorModifierClass;
+} ColorModifierClass;
 #endif
 
-struct _imagestate
-{
+struct _imagestate {
    char               *im_file;
    char               *real_file;
    char                got_colors;
@@ -68,17 +65,14 @@ struct _imagestate
 #endif
 };
 
-typedef struct
-{
+typedef struct {
    ImageState         *normal;
    ImageState         *hilited;
    ImageState         *clicked;
    ImageState         *disabled;
-}
-ImageStateArray;
+} ImageStateArray;
 
-struct _imageclass
-{
+struct _imageclass {
    char               *name;
    ImageStateArray     norm, active, sticky, sticky_active;
    EImageBorder        padding;
@@ -207,7 +201,7 @@ ImagestateColorsSetGray(ImageState * is,
 }
 
 static ImageState  *
-ImagestateCreate(void)
+ImagestateCreate(const char *file)
 {
    ImageState         *is;
 
@@ -218,44 +212,43 @@ ImagestateCreate(void)
    is->pixmapfillstyle = FILL_STRETCH;
    ImagestateColorsSetGray(is, 255, 200, 160, 120, 64);
    is->bevelstyle = BEVEL_NONE;
+   is->im_file = Estrdup(file);
 
    return is;
 }
 
+#if ENABLE_DESTROY
 static void
-FreeImageState(ImageState * i)
+ImagestateDestroy(ImageState * is)
 {
+   if (!is)
+      return;
 
-   Efree(i->im_file);
-   Efree(i->real_file);
+   Efree(is->im_file);
+   Efree(is->real_file);
 
-   if (i->im)
-     {
-	EImageFree(i->im);
-	i->im = NULL;
-     }
+   if (is->im)
+      EImageFree(is->im);
 
-   if (i->border)
-      Efree(i->border);
+   Efree(is->border);
 
 #if ENABLE_COLOR_MODIFIERS
-   if (i->colmod)
-      i->colmod->ref_count--;
+   if (is->colmod)
+      is->colmod->ref_count--;
 #endif
+
+   Efree(is);
 }
 
 static void
 FreeImageStateArray(ImageStateArray * isa)
 {
-   FreeImageState(isa->normal);
-   Efree(isa->normal);
-   FreeImageState(isa->hilited);
-   Efree(isa->hilited);
-   FreeImageState(isa->clicked);
-   Efree(isa->clicked);
-   FreeImageState(isa->disabled);
-   Efree(isa->disabled);
+   ImagestateDestroy(isa->normal);
+   ImagestateDestroy(isa->hilited);
+   ImagestateDestroy(isa->clicked);
+   ImagestateDestroy(isa->disabled);
 }
+#endif /* ENABLE_DESTROY */
 
 static void
 ImagestateColorsAlloc(ImageState * is)
@@ -263,11 +256,11 @@ ImagestateColorsAlloc(ImageState * is)
    if (!is || is->got_colors)
       return;
 
-   EAllocColor(VRoot.cmap, &is->bg);
-   EAllocColor(VRoot.cmap, &is->hi);
-   EAllocColor(VRoot.cmap, &is->lo);
-   EAllocColor(VRoot.cmap, &is->hihi);
-   EAllocColor(VRoot.cmap, &is->lolo);
+   EAllocColor(WinGetCmap(VROOT), &is->bg);
+   EAllocColor(WinGetCmap(VROOT), &is->hi);
+   EAllocColor(WinGetCmap(VROOT), &is->lo);
+   EAllocColor(WinGetCmap(VROOT), &is->hihi);
+   EAllocColor(WinGetCmap(VROOT), &is->lolo);
 
    is->got_colors = 1;
 }
@@ -346,6 +339,7 @@ ImageclassCreate(const char *name)
    return ic;
 }
 
+#if ENABLE_DESTROY
 static void
 ImageclassDestroy(ImageClass * ic)
 {
@@ -361,8 +355,7 @@ ImageclassDestroy(ImageClass * ic)
 
    ecore_list_node_remove(iclass_list, ic);
 
-   if (ic->name)
-      Efree(ic->name);
+   Efree(ic->name);
 
    FreeImageStateArray(&(ic->norm));
    FreeImageStateArray(&(ic->active));
@@ -373,7 +366,10 @@ ImageclassDestroy(ImageClass * ic)
    if (ic->colmod)
       ic->colmod->ref_count--;
 #endif
+
+   Efree(ic);
 }
+#endif /* ENABLE_DESTROY */
 
 ImageClass         *
 ImageclassAlloc(const char *name, int fallback)
@@ -618,84 +614,52 @@ ImageclassConfigLoad(FILE * fs)
 	  case CONFIG_DESKTOP:
 	     /* don't ask... --mandrake */
 	  case ICLASS_NORMAL:
-	     ic->norm.normal = ImagestateCreate();
-	     ic->norm.normal->im_file = Estrdup(s2);
-	     ICToRead = ic->norm.normal;
+	     ic->norm.normal = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_CLICKED:
-	     ic->norm.clicked = ImagestateCreate();
-	     ic->norm.clicked->im_file = Estrdup(s2);
-	     ICToRead = ic->norm.clicked;
+	     ic->norm.clicked = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_HILITED:
-	     ic->norm.hilited = ImagestateCreate();
-	     ic->norm.hilited->im_file = Estrdup(s2);
-	     ICToRead = ic->norm.hilited;
+	     ic->norm.hilited = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_DISABLED:
-	     ic->norm.disabled = ImagestateCreate();
-	     ic->norm.disabled->im_file = Estrdup(s2);
-	     ICToRead = ic->norm.disabled;
+	     ic->norm.disabled = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_STICKY_NORMAL:
-	     ic->sticky.normal = ImagestateCreate();
-	     ic->sticky.normal->im_file = Estrdup(s2);
-	     ICToRead = ic->sticky.normal;
+	     ic->sticky.normal = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_STICKY_CLICKED:
-	     ic->sticky.clicked = ImagestateCreate();
-	     ic->sticky.clicked->im_file = Estrdup(s2);
-	     ICToRead = ic->sticky.clicked;
+	     ic->sticky.clicked = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_STICKY_HILITED:
-	     ic->sticky.hilited = ImagestateCreate();
-	     ic->sticky.hilited->im_file = Estrdup(s2);
-	     ICToRead = ic->sticky.hilited;
+	     ic->sticky.hilited = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_STICKY_DISABLED:
-	     ic->sticky.disabled = ImagestateCreate();
-	     ic->sticky.disabled->im_file = Estrdup(s2);
-	     ICToRead = ic->sticky.disabled;
+	     ic->sticky.disabled = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_ACTIVE_NORMAL:
-	     ic->active.normal = ImagestateCreate();
-	     ic->active.normal->im_file = Estrdup(s2);
-	     ICToRead = ic->active.normal;
+	     ic->active.normal = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_ACTIVE_CLICKED:
-	     ic->active.clicked = ImagestateCreate();
-	     ic->active.clicked->im_file = Estrdup(s2);
-	     ICToRead = ic->active.clicked;
+	     ic->active.clicked = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_ACTIVE_HILITED:
-	     ic->active.hilited = ImagestateCreate();
-	     ic->active.hilited->im_file = Estrdup(s2);
-	     ICToRead = ic->active.hilited;
+	     ic->active.hilited = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_ACTIVE_DISABLED:
-	     ic->active.disabled = ImagestateCreate();
-	     ic->active.disabled->im_file = Estrdup(s2);
-	     ICToRead = ic->active.disabled;
+	     ic->active.disabled = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_STICKY_ACTIVE_NORMAL:
-	     ic->sticky_active.normal = ImagestateCreate();
-	     ic->sticky_active.normal->im_file = Estrdup(s2);
-	     ICToRead = ic->sticky_active.normal;
+	     ic->sticky_active.normal = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_STICKY_ACTIVE_CLICKED:
-	     ic->sticky_active.clicked = ImagestateCreate();
-	     ic->sticky_active.clicked->im_file = Estrdup(s2);
-	     ICToRead = ic->sticky_active.clicked;
+	     ic->sticky_active.clicked = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_STICKY_ACTIVE_HILITED:
-	     ic->sticky_active.hilited = ImagestateCreate();
-	     ic->sticky_active.hilited->im_file = Estrdup(s2);
-	     ICToRead = ic->sticky_active.hilited;
+	     ic->sticky_active.hilited = ICToRead = ImagestateCreate(s2);
 	     break;
 	  case ICLASS_STICKY_ACTIVE_DISABLED:
-	     ic->sticky_active.disabled = ImagestateCreate();
-	     ic->sticky_active.disabled->im_file = Estrdup(s2);
-	     ICToRead = ic->sticky_active.disabled;
+	     ic->sticky_active.disabled = ICToRead = ImagestateCreate(s2);
 	     break;
 	  default:
 	     ConfigParseError("ImageClass", s);
@@ -714,8 +678,10 @@ ImageclassCreateSimple(const char *name, const char *image)
    ImageClass         *ic;
 
    ic = ImageclassCreate(name);
-   ic->norm.normal = ImagestateCreate();
-   ic->norm.normal->im_file = Estrdup(image);
+   if (!ic)
+      return NULL;
+
+   ic->norm.normal = ImagestateCreate(image);
    ic->norm.normal->unloadable = 1;
    ImageclassPopulate(ic);
 
@@ -806,29 +772,18 @@ ImageclassGetImage(ImageClass * ic, int active, int sticky, int state)
    return im;
 }
 
-Pixmap
+void
 ImageclassApplySimple(ImageClass * ic, Win win, Drawable draw, int state,
 		      int x, int y, int w, int h)
 {
-   Pixmap              pmap;
    EImage             *im;
 
    im = ImageclassGetImage(ic, 0, 0, state);
    if (!im)
-      return None;
-
-   pmap = None;
-   if (draw == None)
-     {
-	pmap = ECreatePixmap(win, w, h, 0);
-	draw = pmap;
-	x = y = 0;
-     }
+      return;
 
    EImageRenderOnDrawable(im, win, draw, 0, x, y, w, h);
    EImageFree(im);
-
-   return pmap;
 }
 
 #ifdef ENABLE_TRANSPARENCY
@@ -885,28 +840,28 @@ pt_type_to_flags(int image_type)
 }
 
 static EImage      *
-pt_get_bg_image(Window win, int w, int h, int use_root)
+pt_get_bg_image(Win win, int w, int h, int use_root)
 {
    EImage             *ii = NULL;
-   Window              cr, dummy;
+   Win                 cr;
    Drawable            bg;
    int                 xx, yy;
 
    bg = DeskGetBackgroundPixmap(DesksGetCurrent());
    if (use_root || bg == None)
      {
-	cr = VRoot.xwin;
-	bg = VRoot.xwin;
+	cr = VROOT;
+	bg = WinGetXwin(VROOT);
      }
    else
      {
-	cr = EoGetXwin(DesksGetCurrent());
+	cr = EoGetWin(DesksGetCurrent());
      }
-   XTranslateCoordinates(disp, win, cr, 0, 0, &xx, &yy, &dummy);
+   ETranslateCoordinates(win, cr, 0, 0, &xx, &yy, NULL);
 #if 0
    Eprintf("pt_get_bg_image %#lx %d %d %d %d\n", win, xx, yy, w, h);
 #endif
-   if (xx < VRoot.w && yy < VRoot.h && xx + w >= 0 && yy + h >= 0)
+   if (xx < WinGetW(VROOT) && yy < WinGetH(VROOT) && xx + w >= 0 && yy + h >= 0)
      {
 	/* Create the background base image */
 	ii = EImageGrabDrawable(bg, None, xx, yy, w, h, !EServerIsGrabbed());
@@ -935,7 +890,7 @@ ImageclassGetImageBlended(ImageClass * ic, Win win, int w, int h, int active,
    flags = pt_type_to_flags(image_type);
    if (flags != ICLASS_ATTR_OPAQUE)
      {
-	bg = pt_get_bg_image(WinGetXwin(win), w, h, flags & ICLASS_ATTR_GLASS);
+	bg = pt_get_bg_image(win, w, h, flags & ICLASS_ATTR_GLASS);
 	if (bg)
 	  {
 	     EImageBlendCM(bg, im, (flags & ICLASS_ATTR_USE_CM) ? icm : NULL);
@@ -957,7 +912,7 @@ ImageclassGetImageBlended(ImageClass * ic, Win win, int w, int h, int active,
 
 static void
 ImagestateMakePmapMask(ImageState * is, Win win, PmapMask * pmm,
-		       int make_mask, int w, int h, int image_type)
+		       int pmapflags, int w, int h, int image_type)
 {
 #ifdef ENABLE_TRANSPARENCY
    EImage             *ii = NULL;
@@ -977,7 +932,7 @@ ImagestateMakePmapMask(ImageState * is, Win win, PmapMask * pmm,
 
    if (flags != ICLASS_ATTR_OPAQUE)
      {
-	ii = pt_get_bg_image(WinGetXwin(win), w, h, flags & ICLASS_ATTR_GLASS);
+	ii = pt_get_bg_image(win, w, h, flags & ICLASS_ATTR_GLASS);
      }
    else
      {
@@ -997,7 +952,7 @@ ImagestateMakePmapMask(ImageState * is, Win win, PmapMask * pmm,
 	pmm->h = h;
 	EImageRenderOnDrawable(ii, win, pmap, 0, 0, 0, w, h);
 
-	if (make_mask && !(flags & ICLASS_ATTR_NO_CLIP))
+	if ((pmapflags & IC_FLAG_MAKE_MASK) && !(flags & ICLASS_ATTR_NO_CLIP))
 	  {
 	     if (EImageHasAlpha(is->im))
 	       {
@@ -1008,14 +963,14 @@ ImagestateMakePmapMask(ImageState * is, Win win, PmapMask * pmm,
 		  /* Replace the mask with the correct one */
 		  pmm->mask = EXCreatePixmapCopy(mask, w, h, 1);
 
-		  EImagePixmapFree(pmap);
+		  EImagePixmapsFree(pmap, mask);
 	       }
 	  }
 	EImageDecache(ii);
      }
    else
 #else
-   make_mask = 0;
+   pmapflags = 0;
    image_type = 0;
 #endif /* ENABLE_TRANSPARENCY */
    if (is->pixmapfillstyle == FILL_STRETCH)
@@ -1165,11 +1120,11 @@ ImagestateDrawBevel(ImageState * is, Drawable win, GC gc, int w, int h)
 void
 ITApply(Win win, ImageClass * ic, ImageState * is,
 	int state, int active, int sticky, int image_type,
-	TextClass * tc, TextState * ts, const char *text)
+	TextClass * tc, TextState * ts, const char *text, int flags)
 {
    int                 w, h;
 
-   if (win == NoWin || !ic)
+   if (win == NULL || !ic)
       return;
 
    w = WinGetW(win);
@@ -1196,7 +1151,8 @@ ITApply(Win win, ImageClass * ic, ImageState * is,
      {
 	PmapMask            pmm;
 
-	ImagestateMakePmapMask(is, win, &pmm, 1, w, h, image_type);
+	ImagestateMakePmapMask(is, win, &pmm, IC_FLAG_MAKE_MASK, w, h,
+			       image_type);
 
 	if (pmm.pmap)
 	  {
@@ -1212,12 +1168,15 @@ ITApply(Win win, ImageClass * ic, ImageState * is,
 
 		  TextstateTextDraw(ts, win, pmap, text, 0, 0, w, h,
 				    &(ic->padding), 0,
-				    TextclassGetJustification(tc));
+				    TextclassGetJustification(tc), flags);
 	       }
 
 	     /* Set window pixmap */
 	     if (pmap == pmm.pmap)
-		ESetWindowBackgroundPixmap(win, pmap);
+	       {
+		  ESetWindowBackgroundPixmap(win, pmap);
+		  EFreeWindowBackgroundPixmap(win);
+	       }
 
 	     if (pmm.w == w && pmm.h == h)
 		EShapeCombineMask(win, ShapeBounding, 0, 0, pmm.mask, ShapeSet);
@@ -1258,16 +1217,40 @@ void
 ImageclassApply(ImageClass * ic, Win win, int active, int sticky, int state,
 		int image_type)
 {
-   ITApply(win, ic, NULL, state, active, sticky, image_type, NULL, NULL, NULL);
+   ITApply(win, ic, NULL, state, active, sticky, image_type, NULL, NULL, NULL,
+	   0);
+}
+
+static void
+PmapMaskTile(PmapMask * pmm, Win win, unsigned int w, unsigned int h)
+{
+   Pixmap              pmap, mask;
+
+   pmap = ECreatePixmap(win, w, h, 0);
+   if (pmap == None)
+      return;
+   EXCopyAreaTiled(pmm->pmap, None, pmap, 0, 0, w, h, 0, 0);
+
+   mask = None;
+   if (pmm->mask != None)
+      mask = ECreatePixmap(win, w, h, 1);
+   if (mask != None)
+      EXCopyAreaTiled(pmm->mask, None, mask, 0, 0, w, h, 0, 0);
+
+   FreePmapMask(pmm);
+   pmm->type = 0;
+   pmm->w = w;
+   pmm->h = h;
+   pmm->pmap = pmap;
+   pmm->mask = mask;
 }
 
 void
 ImageclassApplyCopy(ImageClass * ic, Win win, int w, int h,
 		    int active, int sticky, int state,
-		    PmapMask * pmm, int make_mask, int image_type)
+		    PmapMask * pmm, int pmapflags, int image_type)
 {
    ImageState         *is;
-   GC                  gc;
 
    if (pmm == NULL)
       return;
@@ -1288,44 +1271,14 @@ ImageclassApplyCopy(ImageClass * ic, Win win, int w, int h,
    /* Imlib2 will not render pixmaps with dimensions > 8192 */
    if (is->im && w <= 8192 && h <= 8192)
      {
-	ImagestateMakePmapMask(is, win, pmm, make_mask, w, h, image_type);
+	ImagestateMakePmapMask(is, win, pmm, pmapflags, w, h, image_type);
 
-	if (pmm->pmap)
+	if ((pmapflags & IC_FLAG_FULL_SIZE) && pmm->pmap &&
+	    (pmm->w != w || pmm->h != h))
 	  {
-	     if (pmm->w != w || pmm->h != h)
-	       {
-		  /* Create new full sized pixmaps and fill them with the */
-		  /* pmap and mask tiles                                  */
-		  Pixmap              tp = 0, tm = 0;
-		  XGCValues           gcv;
-
-		  tp = ECreatePixmap(win, w, h, 0);
-		  gcv.fill_style = FillTiled;
-		  gcv.tile = pmm->pmap;
-		  gcv.ts_x_origin = 0;
-		  gcv.ts_y_origin = 0;
-		  gc = EXCreateGC(tp, GCFillStyle | GCTile |
-				  GCTileStipXOrigin | GCTileStipYOrigin, &gcv);
-		  XFillRectangle(disp, tp, gc, 0, 0, w, h);
-		  EXFreeGC(gc);
-		  if (pmm->mask)
-		    {
-		       tm = ECreatePixmap(win, w, h, 1);
-		       gcv.fill_style = FillTiled;
-		       gcv.tile = pmm->mask;
-		       gcv.ts_x_origin = 0;
-		       gcv.ts_y_origin = 0;
-		       gc = EXCreateGC(tm, GCFillStyle | GCTile |
-				       GCTileStipXOrigin | GCTileStipYOrigin,
-				       &gcv);
-		       XFillRectangle(disp, tm, gc, 0, 0, w, h);
-		       EXFreeGC(gc);
-		    }
-		  FreePmapMask(pmm);
-		  pmm->type = 0;
-		  pmm->pmap = tp;
-		  pmm->mask = tm;
-	       }
+	     /* Create new full sized pixmaps and fill them with the */
+	     /* pmap and mask tiles                                  */
+	     PmapMaskTile(pmm, win, w, h);
 	  }
 
 	if ((is->unloadable) || (Conf.memory_paranoia))
@@ -1336,6 +1289,7 @@ ImageclassApplyCopy(ImageClass * ic, Win win, int w, int h,
      }
    else
      {
+	GC                  gc;
 	Pixmap              pmap;
 
 	ImagestateColorsAlloc(is);
@@ -1368,29 +1322,29 @@ ImageclassSetupFallback(void)
    /* create a fallback imageclass in case no imageclass can be found */
    ic = ImageclassCreate("__FALLBACK_ICLASS");
 
-   ic->norm.normal = ImagestateCreate();
+   ic->norm.normal = ImagestateCreate(NULL);
    ImagestateColorsSetGray(ic->norm.normal, 255, 255, 160, 0, 0);
    ic->norm.normal->bevelstyle = BEVEL_AMIGA;
 
-   ic->norm.hilited = ImagestateCreate();
+   ic->norm.hilited = ImagestateCreate(NULL);
    ImagestateColorsSetGray(ic->norm.hilited, 255, 255, 192, 0, 0);
    ic->norm.hilited->bevelstyle = BEVEL_AMIGA;
 
-   ic->norm.clicked = ImagestateCreate();
+   ic->norm.clicked = ImagestateCreate(NULL);
    ImagestateColorsSetGray(ic->norm.clicked, 0, 0, 192, 255, 255);
    ic->norm.clicked->bevelstyle = BEVEL_AMIGA;
 
-   ic->active.normal = ImagestateCreate();
+   ic->active.normal = ImagestateCreate(NULL);
    ImagestateColorsSetGray(ic->active.normal, 255, 255, 0, 0, 0);
    SET_COLOR(&(ic->active.normal->bg), 180, 140, 160);
    ic->active.normal->bevelstyle = BEVEL_AMIGA;
 
-   ic->active.hilited = ImagestateCreate();
+   ic->active.hilited = ImagestateCreate(NULL);
    ImagestateColorsSetGray(ic->active.hilited, 255, 255, 0, 0, 0);
    SET_COLOR(&(ic->active.hilited->bg), 230, 190, 210);
    ic->active.hilited->bevelstyle = BEVEL_AMIGA;
 
-   ic->active.clicked = ImagestateCreate();
+   ic->active.clicked = ImagestateCreate(NULL);
    ImagestateColorsSetGray(ic->active.clicked, 0, 0, 0, 255, 255);
    SET_COLOR(&(ic->active.clicked->bg), 230, 190, 210);
    ic->active.clicked->bevelstyle = BEVEL_AMIGA;
@@ -1405,7 +1359,7 @@ ImageclassSetupFallback(void)
    /* Create all black image class for filler borders */
    ic = ImageclassCreate("__BLACK");
 
-   ic->norm.normal = ImagestateCreate();
+   ic->norm.normal = ImagestateCreate(NULL);
    ImagestateColorsSetGray(ic->norm.normal, 0, 0, 0, 0, 0);
 
    ImageclassPopulate(ic);
@@ -1469,7 +1423,7 @@ ImageclassIpc(const char *params)
 	Pixmap              pmap;
 
 	pmap = (Pixmap) strtol(p, NULL, 0);
-	EImagePixmapFree(pmap);
+	EImagePixmapsFree(pmap, None);
 	return;
      }
 
@@ -1482,7 +1436,9 @@ ImageclassIpc(const char *params)
 
    if (!strcmp(param2, "delete"))
      {
+#if ENABLE_DESTROY
 	ImageclassDestroy(ic);
+#endif
      }
    else if (!strcmp(param2, "modify"))
      {
@@ -1572,7 +1528,8 @@ ImageclassIpc(const char *params)
 	     return;
 	  }
 
-	ImageclassApplyCopy(ic, win, w, h, 0, 0, st, &pmm, 1, ST_SOLID);
+	ImageclassApplyCopy(ic, win, w, h, 0, 0, st, &pmm,
+			    IC_FLAG_MAKE_MASK | IC_FLAG_FULL_SIZE, ST_SOLID);
 	IpcPrintf("0x%08lx 0x%08lx\n", pmm.pmap, pmm.mask);
 	EDestroyWin(win);
      }
@@ -1594,7 +1551,7 @@ static const IpcItem ImageclassIpcArray[] = {
    {
     ImageclassIpc,
     "imageclass", NULL,
-    "List imageclasses, create/delete/modify/apply an imageclass",
+    "List imageclasses, apply an imageclass",
     NULL}
    ,
 };
