@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -24,7 +24,6 @@
 #include "E.h"
 #include "aclass.h"
 #include "borders.h"		/* FIXME - Should not be here */
-#include "comms.h"
 #include "desktops.h"
 #include "emodule.h"
 #include "eobj.h"
@@ -43,30 +42,34 @@
 #define SS(s) ((s) ? (s) : NoText)
 static const char   NoText[] = "-NONE-";
 
-static size_t       ipc_bufsiz;
-static char        *ipc_bufptr;
+static char        *ipc_bufptr = NULL;
+static size_t       ipc_bufsiz = 0;
+static char         ipc_active = 0;
 
 static void
 IpcPrintInit(void)
 {
-   ipc_bufsiz = 0;
    ipc_bufptr = NULL;
+   ipc_bufsiz = 0;
+   ipc_active = 1;
 }
 
 static void
-IpcPrintFlush(Client * c)
+IpcPrintDone(void)
 {
-   if (ipc_bufptr == NULL)
-      return;
-
-   ipc_bufptr[ipc_bufsiz] = '\0';
-
-   if (c)
-      CommsSend(c, ipc_bufptr);
-
    Efree(ipc_bufptr);
-   ipc_bufsiz = 0;
    ipc_bufptr = NULL;
+   ipc_bufsiz = 0;
+   ipc_active = 0;
+}
+
+static const char  *
+IpcPrintGetBuffer(void)
+{
+   if (!ipc_bufptr)
+      return NULL;
+   ipc_bufptr[ipc_bufsiz] = '\0';
+   return ipc_bufptr;
 }
 
 void
@@ -76,6 +79,9 @@ IpcPrintf(const char *fmt, ...)
    int                 len;
    va_list             args;
 
+   if (!ipc_active)
+      return;
+
    va_start(args, fmt);
    len = Evsnprintf(tmp, sizeof(tmp), fmt, args);
    va_end(args);
@@ -138,7 +144,8 @@ IPC_Screen(const char *params)
      }
    else if (!strncmp(param, "size", 2))
      {
-	IpcPrintf("Screen %d  size %dx%d\n", VRoot.scr, VRoot.w, VRoot.h);
+	IpcPrintf("Screen %d  size %dx%d\n", Dpy.screen,
+		  WinGetW(VROOT), WinGetH(VROOT));
      }
    else if (!strcmp(param, "split"))
      {
@@ -715,13 +722,13 @@ IpcWinop(const WinOp * wop, EWin * ewin, const char *prm)
 	on = ewin->props.never_use_area;
 	SetEwinBoolean(wop->name, &on, param1, 1);
 	ewin->props.never_use_area = on;
-	break;
+	goto ewin_update_snap_flags;
 
      case EWIN_OP_FOCUS_CLICK:
 	on = ewin->props.focusclick;
 	SetEwinBoolean(wop->name, &on, param1, 1);
 	ewin->props.focusclick = on;
-	break;
+	goto ewin_update_snap_flags;
 
      case EWIN_OP_NO_BUTTON_GRABS:
 	on = ewin->props.no_button_grabs;
@@ -733,56 +740,49 @@ IpcWinop(const WinOp * wop, EWin * ewin, const char *prm)
 	     else
 		GrabButtonGrabs(ewin);
 	  }
-	break;
+	goto ewin_update_snap_flags;
 
      case EWIN_OP_INH_APP_FOCUS:
 	on = EwinInhGetApp(ewin, focus);
 	SetEwinBoolean(wop->name, &on, param1, 1);
 	EwinInhSetApp(ewin, focus, on);
-	break;
+	goto ewin_update_snap_flags;
 
      case EWIN_OP_INH_APP_MOVE:
 	on = EwinInhGetApp(ewin, move);
 	SetEwinBoolean(wop->name, &on, param1, 1);
 	EwinInhSetApp(ewin, move, on);
-	break;
+	goto ewin_update_snap_flags;
 
      case EWIN_OP_INH_APP_SIZE:
 	on = EwinInhGetApp(ewin, size);
 	SetEwinBoolean(wop->name, &on, param1, 1);
 	EwinInhSetApp(ewin, size, on);
-	break;
+	goto ewin_update_snap_flags;
 
      case EWIN_OP_INH_USER_CLOSE:
 	on = EwinInhGetUser(ewin, close);
 	SetEwinBoolean(wop->name, &on, param1, 1);
 	EwinInhSetUser(ewin, close, on);
-	EwinStateUpdate(ewin);
-	HintsSetWindowState(ewin);
-	break;
+	goto ewin_update_state_hints;
 
      case EWIN_OP_INH_USER_MOVE:
 	on = EwinInhGetUser(ewin, move);
 	SetEwinBoolean(wop->name, &on, param1, 1);
 	EwinInhSetUser(ewin, move, on);
-	EwinStateUpdate(ewin);
-	HintsSetWindowState(ewin);
-	break;
+	goto ewin_update_state_hints;
 
      case EWIN_OP_INH_USER_SIZE:
 	on = EwinInhGetUser(ewin, size);
 	SetEwinBoolean(wop->name, &on, param1, 1);
 	EwinInhSetUser(ewin, size, on);
-	EwinStateUpdate(ewin);
-	HintsSetWindowState(ewin);
-	break;
+	goto ewin_update_state_hints;
 
      case EWIN_OP_INH_WM_FOCUS:
 	on = EwinInhGetWM(ewin, focus);
 	SetEwinBoolean(wop->name, &on, param1, 1);
 	EwinInhSetWM(ewin, focus, on);
-	EwinStateUpdate(ewin);
-	break;
+	goto ewin_update_state;
 
 #if USE_COMPOSITE
      case EWIN_OP_FADE:
@@ -804,6 +804,18 @@ IpcWinop(const WinOp * wop, EWin * ewin, const char *prm)
 	   EoSetNoRedirect(ewin, on);
 	break;
 #endif
+      ewin_update_snap_flags:
+	SnapshotEwinUpdate(ewin, SNAP_USE_FLAGS);
+	break;
+
+      ewin_update_state:
+	EwinStateUpdate(ewin);
+	break;
+
+      ewin_update_state_hints:
+	EwinStateUpdate(ewin);
+	HintsSetWindowState(ewin);
+	break;
      }
 
  done:
@@ -1082,8 +1094,8 @@ EwinShowInfo(const EWin * ewin)
 	     EwinIsTransient(ewin), EwinGetTransientFor(ewin),
 	     ewin->props.no_resize_h, ewin->props.no_resize_v,
 	     ewin->state.shaped, ewin->icccm.base_w, ewin->icccm.base_h,
-	     ewin->icccm.width.min, ewin->icccm.height.min,
-	     ewin->icccm.width.max, ewin->icccm.height.max,
+	     ewin->icccm.width_min, ewin->icccm.height_min,
+	     ewin->icccm.width_max, ewin->icccm.height_max,
 	     ewin->icccm.w_inc, ewin->icccm.h_inc,
 	     ewin->icccm.aspect_min, ewin->icccm.aspect_max,
 	     ewin->strut.left, ewin->strut.right,
@@ -1202,7 +1214,7 @@ IPC_Warp(const char *params)
    else if (!strncmp(params, "abs", 3))
      {
 	sscanf(params, "%*s %i %i", &x, &y);
-	EXWarpPointer(VRoot.xwin, x, y);
+	EXWarpPointer(WinGetXwin(VROOT), x, y);
      }
    else if (!strncmp(params, "rel", 3))
      {
@@ -1211,7 +1223,7 @@ IPC_Warp(const char *params)
      }
    else if (!strncmp(params, "scr", 3))
      {
-	x = (VRoot.scr + 1) % ScreenCount(disp);
+	x = (Dpy.screen + 1) % ScreenCount(disp);
 	sscanf(params, "%*s %i", &x);
 	if (x >= 0 && x < ScreenCount(disp))
 	   EXWarpPointer(RootWindow(disp, x), DisplayWidth(disp, x) / 2,
@@ -1225,8 +1237,7 @@ IPC_Warp(const char *params)
 }
 
 #if ENABLE_IPC_INSERT_KEYS
-struct _keyset
-{
+struct _keyset {
    const char         *sym;
    int                 state;
    const char         *ch;
@@ -1354,22 +1365,22 @@ IPC_InsertKeys(const char *params, Client * c __UNUSED__)
      {
 	int                 j;
 
-	ev.x = Mode.events.x;
-	ev.y = Mode.events.y;
-	ev.x_root = Mode.events.x;
-	ev.y_root = Mode.events.y;
+	ev.x = Mode.events.cx;
+	ev.y = Mode.events.cy;
+	ev.x_root = Mode.events.cx;
+	ev.y_root = Mode.events.cy;
 	for (j = 0; j < (int)(sizeof(ks) / sizeof(struct _keyset)); j++)
 	  {
 	     if (strncmp(ks[j].ch, &(s[i]), strlen(ks[j].ch)))
 		continue;
 
 	     i += strlen(ks[j].ch) - 1;
-	     ev.keycode = XKeysymToKeycode(disp, XStringToKeysym(ks[j].sym));
+	     ev.keycode = EKeynameToKeycode(ks[j].sym);
 	     ev.state = ks[j].state;
 	     ev.type = KeyPress;
-	     XSendEvent(disp, win, False, 0, (XEvent *) & ev);
+	     EXSendEvent(win, 0, (XEvent *) & ev);
 	     ev.type = KeyRelease;
-	     XSendEvent(disp, win, False, 0, (XEvent *) & ev);
+	     EXSendEvent(win, 0, (XEvent *) & ev);
 	     break;
 	  }
      }
@@ -1409,8 +1420,15 @@ IPC_Compat(const char *params)
 #if !USE_COMPOSITE
    else if (!strcmp(param1, "cm"))
      {
-	DialogOKstr(_("Message"),
-		    _("Enlightenment was built without composite support"));
+	DialogOK(_("Message"), _("e16 was built without %s support"),
+		 _("composite"));
+     }
+#endif
+#if !HAVE_SOUND
+   else if (!strcmp(param1, "sound"))
+     {
+	DialogOK(_("Message"), _("e16 was built without %s support"),
+		 _("sound"));
      }
 #endif
    else
@@ -1632,8 +1650,8 @@ IPC_GetList(int *pnum)
  * you shouldn't have to touch this function
  * - Mandrake
  */
-int
-HandleIPC(const char *params, Client * c)
+static int
+IpcExec(const char *params)
 {
    int                 i, num, ok;
    char                cmd[128];
@@ -1641,9 +1659,7 @@ HandleIPC(const char *params, Client * c)
    const IpcItem     **lst, *ipc;
 
    if (EDebug(EDBUG_TYPE_IPC))
-      Eprintf("HandleIPC: %s\n", params);
-
-   IpcPrintInit();
+      Eprintf("IpcExec: %s\n", params);
 
    cmd[0] = 0;
    num = 0;
@@ -1669,8 +1685,18 @@ HandleIPC(const char *params, Client * c)
    if (!ok && params)
       ok = IPC_Compat(params);
 
-   IpcPrintFlush(c);
-   CommsFlush(c);
+   return ok;
+}
+
+int
+IpcExecReply(const char *params, IpcReplyFunc * reply, void *data)
+{
+   int                 ok;
+
+   IpcPrintInit();
+   ok = IpcExec(params);
+   reply(data, IpcPrintGetBuffer());
+   IpcPrintDone();
 
    return ok;
 }
@@ -1678,13 +1704,13 @@ HandleIPC(const char *params, Client * c)
 int
 EFunc(EWin * ewin, const char *params)
 {
-   int                 err;
+   int                 ok;
 
    SetContextEwin(ewin);
-   err = HandleIPC(params, NULL);
+   ok = IpcExec(params);
    SetContextEwin(NULL);
 
-   return err;
+   return ok;
 }
 
 static void
