@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -24,6 +24,7 @@
 #include "E.h"
 #include "emodule.h"
 #include "lang.h"
+#include <X11/Xlib.h>
 
 #ifdef HAVE_LOCALE_H
 #include <locale.h>
@@ -137,8 +138,7 @@ EstrInt2EncFree(const char *str, int want_utf8)
    if (Mode.locale.utf8_int == want_utf8)
       return;
 
-   if (str)
-      Efree((char *)str);
+   Efree((char *)str);
 #else
    str = NULL;
    want_utf8 = 0;
@@ -273,16 +273,14 @@ EwcWcsToStr(const wchar_t * wcs, int wcl, char *str, int len)
  * Setup
  */
 
-static struct
-{
+static struct {
    char               *internal;
    char               *exported;
 } Conf_locale =
 {
 NULL, NULL};
 
-static struct
-{
+static struct {
    char                init;
    char               *env_language;
    char               *env_lc_all;
