@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -48,8 +48,7 @@
 
 #define MENU_UNLOAD_CHECK_IMTERVAL 300	/* Seconds */
 
-static struct
-{
+static struct {
    Menu               *first;
    Menu               *active;
    EWin               *context_ewin;
@@ -57,8 +56,7 @@ static struct
    char                just_shown;
 } Mode_menus;
 
-struct _menustyle
-{
+struct _menustyle {
    char               *name;
    TextClass          *tclass;
    ImageClass         *bg_iclass;
@@ -72,8 +70,7 @@ struct _menustyle
    unsigned int        ref_count;
 };
 
-struct _menuitem
-{
+struct _menuitem {
    Menu               *menu;
    ImageClass         *icon_iclass;
    char               *text;
@@ -91,8 +88,7 @@ struct _menuitem
    short               text_y;
 };
 
-struct _menu
-{
+struct _menu {
    EWin               *ewin;
    Win                 win;
    PmapMask            pmm;
@@ -173,7 +169,7 @@ MenuHide(Menu * m)
    if (ewin)
      {
 	EUnmapWindow(m->win);
-	EReparentWindow(m->win, VRoot.win, ewin->client.x, ewin->client.y);
+	EReparentWindow(m->win, VROOT, ewin->client.x, ewin->client.y);
 	EwinHide(ewin);
      }
    m->ewin = NULL;
@@ -231,7 +227,10 @@ _MenuEwinMoveResize(EWin * ewin, int resize __UNUSED__)
       m->redraw = 1;
 
    if ((!m->style->use_item_bg && m->pmm.pmap == 0) || m->redraw)
-      MenuRedraw(m);
+     {
+	MenuRedraw(m);
+	EwinUpdateShapeInfo(ewin);
+     }
 }
 
 static void
@@ -369,7 +368,7 @@ MenuShow(Menu * m, char noshow)
 	Eprintf("Mode_menus.context_ewin set %s\n",
 		EwinGetTitle(Mode_menus.context_ewin));
 #endif
-	ESync();
+	ESync(ESYNC_MENUS);
 	Mode_menus.first = m;
 	MenuShowMasker(m);
 	TooltipsEnable(0);
@@ -381,8 +380,7 @@ MenuShow(Menu * m, char noshow)
 static void
 MenuStyleSetName(MenuStyle * ms, const char *name)
 {
-   if (ms->name)
-      Efree(ms->name);
+   Efree(ms->name);
    ms->name = Estrdup(name);
 }
 
@@ -483,8 +481,7 @@ MenuSetIconSize(Menu * m, int size)
 void
 MenuSetData(Menu * m, char *data)
 {
-   if (m->data)
-      Efree(m->data);
+   Efree(m->data);
    m->data = data;
 }
 
@@ -575,14 +572,10 @@ MenuDestroy(Menu * m)
    if (m->win)
       EDestroyWindow(m->win);
 
-   if (m->name)
-      Efree(m->name);
-   if (m->alias)
-      Efree(m->alias);
-   if (m->title)
-      Efree(m->title);
-   if (m->data)
-      Efree(m->data);
+   Efree(m->name);
+   Efree(m->alias);
+   Efree(m->title);
+   Efree(m->data);
 
    Efree(m);
 }
@@ -609,20 +602,16 @@ MenuEmpty(Menu * m, int destroying)
 	     mi->child->ref_count--;
 	     MenuDestroy(mi->child);
 	  }
-	if (mi->text)
-	   Efree(mi->text);
-	if (mi->params)
-	   Efree(mi->params);
+	Efree(mi->text);
+	Efree(mi->params);
 	for (j = 0; j < 3; j++)
 	   FreePmapMask(&(mi->pmm[j]));
 	if (!destroying && mi->win)
 	   EDestroyWindow(mi->win);
 	ImageclassFree(mi->icon_iclass);
-	if (mi)
-	   Efree(mi);
+	Efree(mi);
      }
-   if (m->items)
-      Efree(m->items);
+   Efree(m->items);
    m->items = NULL;
    m->num = 0;
    m->sel_item = NULL;
@@ -677,11 +666,18 @@ MenuRepack(Menu * m)
 void
 MenuAddItem(Menu * m, MenuItem * item)
 {
+   MenuItem          **items;
+
    if (!item)
       return;
+
+   items = EREALLOC(MenuItem *, m->items, m->num + 1);
+   if (!items)
+      return;
+
+   items[m->num] = item;
+   m->items = items;
    m->num++;
-   m->items = EREALLOC(MenuItem *, m->items, m->num);
-   m->items[m->num - 1] = item;
 }
 
 static void
@@ -693,6 +689,9 @@ MenuRealize(Menu * m)
    EImageBorder       *pad, *pad_item, *pad_sub;
    char                has_i, has_s;
 
+   if (m->num <= 0)
+      return;
+
    if (!m->style)
      {
 	MenuSetStyle(m, NULL);
@@ -702,7 +701,7 @@ MenuRealize(Menu * m)
 
    if (!m->win)
      {
-	m->win = ECreateClientWindow(VRoot.win, 0, 0, 1, 1);
+	m->win = ECreateClientWindow(VROOT, 0, 0, 1, 1);
 	EventCallbackRegister(m->win, 0, MenuHandleEvents, m);
      }
 
@@ -809,7 +808,7 @@ MenuRealize(Menu * m)
    mmw = 0;
    mmh = 0;
 
-   nmaxy = 3 * VRoot.h / (4 * maxh + 1);
+   nmaxy = 3 * WinGetH(VROOT) / (4 * maxh + 1);
    if (m->style->maxy && nmaxy > m->style->maxy)
       nmaxy = m->style->maxy;
 
@@ -899,7 +898,7 @@ MenuRealize(Menu * m)
 static void
 MenuRedraw(Menu * m)
 {
-   int                 i, j, w, h;
+   int                 i, j;
 
    if (m->redraw)
      {
@@ -914,12 +913,13 @@ MenuRedraw(Menu * m)
 
    if (!m->style->use_item_bg)
      {
-	w = m->w;
-	h = m->h;
 	FreePmapMask(&m->pmm);
-	ImageclassApplyCopy(m->style->bg_iclass, m->win, w, h, 0,
-			    0, STATE_NORMAL, &m->pmm, 1, ST_MENU);
-	ESetWindowBackgroundPixmap(m->win, m->pmm.pmap);
+	ImageclassApplyCopy(m->style->bg_iclass, m->win, m->w, m->h, 0,
+			    0, STATE_NORMAL, &m->pmm, IC_FLAG_MAKE_MASK,
+			    ST_MENU);
+	EGetWindowBackgroundPixmap(m->win);
+	EXCopyAreaTiled(m->pmm.pmap, None, WinGetPmap(m->win),
+			0, 0, m->w, m->h, 0, 0);
 	EShapeCombineMask(m->win, ShapeBounding, 0, 0, m->pmm.mask, ShapeSet);
 	EClearWindow(m->win);
 	for (i = 0; i < m->num; i++)
@@ -946,10 +946,11 @@ MenuDrawItem(Menu * m, MenuItem * mi, char shape, int state)
 
    if (!mi_pmm->pmap)
      {
-	GC                  gc;
 	int                 x, y, w, h;
 	int                 item_type;
 	ImageClass         *ic;
+	GC                  gc;
+	PmapMask            pmm;
 
 	EGetGeometry(mi->win, NULL, &x, &y, &w, &h, NULL, NULL);
 
@@ -963,27 +964,25 @@ MenuDrawItem(Menu * m, MenuItem * mi, char shape, int state)
 	if (!m->style->use_item_bg)
 	  {
 	     gc = EXCreateGC(m->pmm.pmap, 0, NULL);
-	     XCopyArea(disp, m->pmm.pmap, mi_pmm->pmap, gc, x, y, w, h, 0, 0);
+	     XCopyArea(disp, WinGetPmap(m->win), mi_pmm->pmap, gc, x, y, w, h,
+		       0, 0);
 	     if ((mi->state != STATE_NORMAL) || (mi->child))
 	       {
-		  PmapMask            pmm;
-
-		  ImageclassApplyCopy(ic, mi->win, w, h, 0, 0,
-				      mi->state, &pmm, 1, item_type);
-		  if (pmm.mask)
-		    {
-		       XSetClipMask(disp, gc, pmm.mask);
-		       XSetClipOrigin(disp, gc, 0, 0);
-		    }
-		  XCopyArea(disp, pmm.pmap, mi_pmm->pmap, gc, 0, 0, w, h, 0, 0);
+		  ImageclassApplyCopy(ic, mi->win, w, h, 0, 0, mi->state, &pmm,
+				      IC_FLAG_MAKE_MASK, item_type);
+		  EXCopyAreaTiled(pmm.pmap, pmm.mask, mi_pmm->pmap,
+				  0, 0, w, h, 0, 0);
 		  FreePmapMask(&pmm);
 	       }
 	     EXFreeGC(gc);
 	  }
 	else
 	  {
-	     ImageclassApplyCopy(ic, mi->win, w, h, 0, 0, mi->state,
-				 mi_pmm, 1, item_type);
+	     ImageclassApplyCopy(ic, mi->win, w, h, 0, 0, mi->state, &pmm,
+				 IC_FLAG_MAKE_MASK, item_type);
+	     EXCopyAreaTiled(pmm.pmap, pmm.mask, mi_pmm->pmap,
+			     0, 0, w, h, 0, 0);
+	     FreePmapMask(&pmm);
 	  }
 
 	if (mi->text)
@@ -1010,8 +1009,9 @@ MenuShowMasker(Menu * m __UNUSED__)
 
    if (!eo)
      {
-	eo = EobjWindowCreate(EOBJ_TYPE_EVENT, 0, 0, VRoot.w, VRoot.h, 0,
-			      "Masker");
+	eo = EobjWindowCreate(EOBJ_TYPE_EVENT,
+			      0, 0, WinGetW(VROOT), WinGetH(VROOT),
+			      0, "Masker");
 	if (!eo)
 	   return;
 
@@ -1342,26 +1342,27 @@ MenusHandleMotion(void)
 	int                 my_width, my_height, x_org, y_org, head_num = 0;
 
 	head_num =
-	   ScreenGetGeometry(Mode.events.x, Mode.events.y, &x_org, &y_org,
+	   ScreenGetGeometry(Mode.events.mx, Mode.events.my, &x_org, &y_org,
 			     &my_width, &my_height);
 
-	if (Mode.events.x > ((x_org + my_width) - (menu_scroll_dist + 1)))
+	if (Mode.events.mx > ((x_org + my_width) - (menu_scroll_dist + 1)))
 	  {
-	     xdist = -(menu_scroll_dist + (Mode.events.x - (x_org + my_width)));
+	     xdist =
+		-(menu_scroll_dist + (Mode.events.mx - (x_org + my_width)));
 	  }
-	else if (Mode.events.x < (menu_scroll_dist + x_org))
+	else if (Mode.events.mx < (menu_scroll_dist + x_org))
 	  {
-	     xdist = x_org + menu_scroll_dist - (Mode.events.x);
+	     xdist = x_org + menu_scroll_dist - (Mode.events.mx);
 	  }
 
-	if (Mode.events.y > (VRoot.h - (menu_scroll_dist + 1)))
+	if (Mode.events.my > (WinGetH(VROOT) - (menu_scroll_dist + 1)))
 	  {
 	     ydist =
-		-(menu_scroll_dist + (Mode.events.y - (y_org + my_height)));
+		-(menu_scroll_dist + (Mode.events.my - (y_org + my_height)));
 	  }
-	else if (Mode.events.y < (menu_scroll_dist + y_org))
+	else if (Mode.events.my < (menu_scroll_dist + y_org))
 	  {
-	     ydist = y_org + menu_scroll_dist - (Mode.events.y);
+	     ydist = y_org + menu_scroll_dist - (Mode.events.my);
 	  }
 
 	/* That's a hack to avoid unwanted events:
@@ -1490,8 +1491,7 @@ MenusHandleMotion(void)
 }
 #endif
 
-struct _mdata
-{
+struct _mdata {
    Menu               *m;
    MenuItem           *mi;
 };
@@ -2106,7 +2106,10 @@ MenusIpc(const char *params)
 	if (*p == '\0')
 	   p = NULL;
 	if (p && !strcmp(prm, "named"))
-	   Esnprintf(prm, sizeof(prm), "%s", p);
+	  {
+	     Esnprintf(prm, sizeof(prm), "%s", p);
+	     p = NULL;
+	  }
 	SoundPlay("SOUND_MENU_SHOW");
 	MenusShowNamed(prm, p);
      }
