@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -25,6 +25,7 @@
 #include "cursors.h"
 #include "desktops.h"
 #include "emodule.h"
+#include "events.h"
 #include "ewins.h"
 #include "focus.h"
 #include "grabs.h"
@@ -33,8 +34,7 @@
 #include "timers.h"
 #include "xwin.h"
 
-static struct
-{
+static struct {
    EWin               *ewin;
    int                 mode;
    int                 start_x, start_y;
@@ -80,7 +80,7 @@ int
 ActionMoveStart(EWin * ewin, char constrained, int nogroup)
 {
    EWin              **gwins;
-   int                 i, num;
+   int                 i, num, cx, cy;
 
    if (!ewin || ewin->state.inhibit_move)
       return 0;
@@ -88,6 +88,8 @@ ActionMoveStart(EWin * ewin, char constrained, int nogroup)
    Mode_mr.ewin = ewin;
    Mode_mr.nogroup = nogroup;
 
+   EventsGetXY(&cx, &cy);
+
    SoundPlay("SOUND_MOVE_START");
 
    GrabPointerSet(EoGetWin(ewin), ECSR_ACT_MOVE, 1);
@@ -95,8 +97,11 @@ ActionMoveStart(EWin * ewin, char constrained, int nogroup)
    Mode.mode = MODE_MOVE_PENDING;
    Mode.constrained = constrained;
 
-   Mode_mr.win_x = Mode.events.x - (EoGetX(ewin) + EoGetX(EoGetDesk(ewin)));
-   Mode_mr.win_y = Mode.events.y - (EoGetY(ewin) + EoGetY(EoGetDesk(ewin)));
+   Mode_mr.start_x = cx;
+   Mode_mr.start_y = cy;
+
+   Mode_mr.win_x = Mode_mr.start_x - (EoGetX(ewin) + EoGetX(EoGetDesk(ewin)));
+   Mode_mr.win_y = Mode_mr.start_y - (EoGetY(ewin) + EoGetY(EoGetDesk(ewin)));
 
    EwinRaise(ewin);
    gwins = ListWinGroupMembersForEwin(ewin, GROUP_ACTION_MOVE, nogroup
@@ -127,7 +132,7 @@ ActionMoveStart(EWin * ewin, char constrained, int nogroup)
    return 0;
 }
 
-int
+static int
 ActionMoveEnd(EWin * ewin)
 {
    EWin              **gwins;
@@ -159,7 +164,7 @@ ActionMoveEnd(EWin * ewin)
      }
    Mode.mode = MODE_NONE;
 
-   d2 = DesktopAt(Mode.events.x, Mode.events.y);
+   d2 = DesktopAt(Mode.events.mx, Mode.events.my);
 
    for (i = 0; i < num; i++)
      {
@@ -180,7 +185,7 @@ ActionMoveEnd(EWin * ewin)
 
    Efree(gwins);
 
-   ESync();
+   ESync(ESYNC_MOVRES);
 
  done:
    Mode.mode = MODE_NONE;
@@ -221,8 +226,7 @@ ActionMoveSuspend(void)
 	     DrawEwinShape(ewin, Mode_mr.mode, ewin->shape_x,
 			   ewin->shape_y, ewin->client.w, ewin->client.h, 3, i);
 	  }
-	if (lst)
-	   Efree(lst);
+	Efree(lst);
 
 	EUngrabServer();
      }
@@ -253,8 +257,10 @@ ActionMoveResume(void)
    if (Mode_mr.grab_server)
       EGrabServer();
 
-   dx = Mode.events.x - Mode_mr.win_x - EoGetX(EoGetDesk(ewin)) - ewin->shape_x;
-   dy = Mode.events.y - Mode_mr.win_y - EoGetY(EoGetDesk(ewin)) - ewin->shape_y;
+   dx =
+      Mode.events.mx - Mode_mr.win_x - EoGetX(EoGetDesk(ewin)) - ewin->shape_x;
+   dy =
+      Mode.events.my - Mode_mr.win_y - EoGetY(EoGetDesk(ewin)) - ewin->shape_y;
 
    /* Redraw any windows that were in "move mode" */
    lst = ListWinGroupMembersForEwin(ewin, GROUP_ACTION_MOVE,
@@ -271,8 +277,7 @@ ActionMoveResume(void)
 	DrawEwinShape(ewin, Mode_mr.mode, x, y,
 		      ewin->client.w, ewin->client.h, fl, i);
      }
-   if (lst)
-      Efree(lst);
+   Efree(lst);
 
    return 0;
 }
@@ -284,7 +289,7 @@ ActionMoveResume(void)
 int
 ActionResizeStart(EWin * ewin, int hv)
 {
-   int                 x, y, w, h, ww, hh;
+   int                 x, y, w, h, ww, hh, cx, cy;
    unsigned int        csr;
 
    if (!ewin || ewin->state.inhibit_resize)
@@ -292,6 +297,8 @@ ActionResizeStart(EWin * ewin, int hv)
 
    Mode_mr.ewin = ewin;
 
+   EventsGetXY(&cx, &cy);
+
    SoundPlay("SOUND_RESIZE_START");
 
    if (Conf.movres.mode_resize < 0 || Conf.movres.mode_resize > 4)
@@ -316,8 +323,8 @@ ActionResizeStart(EWin * ewin, int hv)
      default:
      case MODE_RESIZE:
 	Mode.mode = hv;
-	x = Mode.events.x - EoGetX(ewin);
-	y = Mode.events.y - EoGetY(ewin);
+	x = cx - EoGetX(ewin);
+	y = cy - EoGetY(ewin);
 	w = EoGetW(ewin) >> 1;
 	h = EoGetH(ewin) >> 1;
 	ww = EoGetW(ewin) / 6;
@@ -376,7 +383,7 @@ ActionResizeStart(EWin * ewin, int hv)
 
      case MODE_RESIZE_H:
 	Mode.mode = hv;
-	x = Mode.events.x - EoGetX(ewin);
+	x = cx - EoGetX(ewin);
 	w = EoGetW(ewin) >> 1;
 	if (x < w)
 	   Mode_mr.resize_detail = RD(1, 0);
@@ -387,7 +394,7 @@ ActionResizeStart(EWin * ewin, int hv)
 
      case MODE_RESIZE_V:
 	Mode.mode = hv;
-	y = Mode.events.y - EoGetY(ewin);
+	y = cy - EoGetY(ewin);
 	h = EoGetH(ewin) >> 1;
 	if (y < h)
 	   Mode_mr.resize_detail = RD(0, 1);
@@ -397,8 +404,8 @@ ActionResizeStart(EWin * ewin, int hv)
 	break;
      }
 
-   Mode_mr.start_x = Mode.events.x;
-   Mode_mr.start_y = Mode.events.y;
+   Mode_mr.start_x = cx;
+   Mode_mr.start_y = cy;
    Mode_mr.win_x = EoGetX(ewin);
    Mode_mr.win_y = EoGetY(ewin);
    Mode_mr.win_w = ewin->client.w;
@@ -414,7 +421,7 @@ ActionResizeStart(EWin * ewin, int hv)
    return 0;
 }
 
-int
+static int
 ActionResizeEnd(EWin * ewin)
 {
    if (ewin && ewin != Mode_mr.ewin)
@@ -448,7 +455,7 @@ ActionResizeEnd(EWin * ewin)
 			    ewin->shape_w, ewin->shape_h);
      }
 
-   ESync();
+   ESync(ESYNC_MOVRES);
 
  done:
    if (Mode_mr.grab_server)
@@ -475,7 +482,7 @@ ActionMoveHandleMotion(void)
    if (!ewin)
       return;
 
-   EdgeCheckMotion(Mode.events.x, Mode.events.y);
+   EdgeCheckMotion(Mode.events.mx, Mode.events.my);
 
    gwins = ListWinGroupMembersForEwin(ewin, GROUP_ACTION_MOVE,
 				      Mode_mr.nogroup || Mode.move.swap, &num);
@@ -495,15 +502,25 @@ ActionMoveHandleMotion(void)
 	for (i = 0; i < num; i++)
 	  {
 	     ewin1 = gwins[i];
-	     DrawEwinShape(ewin1, Mode_mr.mode, EoGetX(ewin1),
-			   EoGetY(ewin1), ewin1->client.w, ewin1->client.h, 0,
-			   i);
+	     DrawEwinShape(ewin1, Mode_mr.mode, EoGetX(ewin1), EoGetY(ewin1),
+			   ewin1->client.w, ewin1->client.h, 0, i);
+	     if (Conf.movres.mode_move == 0)
+		Mode_mr.mode = 0;
 	  }
 	Mode.mode = MODE_MOVE;
+	dx = Mode.events.mx - Mode_mr.start_x;
+	dy = Mode.events.my - Mode_mr.start_y;
+     }
+   else if (Mode.mode == MODE_MOVE)
+     {
+	dx = Mode.events.mx - Mode.events.px;
+	dy = Mode.events.my - Mode.events.py;
+     }
+   else
+     {
+	/* It should not be possible to get here. */
+	goto done;
      }
-
-   dx = Mode.events.x - Mode.events.px;
-   dy = Mode.events.y - Mode.events.py;
 
    jumpx = 0;
    jumpy = 0;
@@ -557,7 +574,8 @@ ActionMoveHandleMotion(void)
       ndy = min_dy;
 
    screen_snap_dist =
-      Mode.constrained ? (VRoot.w + VRoot.h) : Conf.snap.screen_snap_dist;
+      Mode.constrained ? (WinGetW(VROOT) +
+			  WinGetH(VROOT)) : Conf.snap.screen_snap_dist;
 
    for (i = 0; i < num; i++)
      {
@@ -570,10 +588,10 @@ ActionMoveHandleMotion(void)
 	if ((ndx != dx) &&
 	    (((ewin1->shape_x == 0) &&
 	      (dd > screen_snap_dist)) ||
-	     ((ewin1->shape_x == (VRoot.w - EoGetW(ewin1))) &&
+	     ((ewin1->shape_x == (WinGetW(VROOT) - EoGetW(ewin1))) &&
 	      (dd > screen_snap_dist)) ||
 	     ((ewin1->shape_x != 0) &&
-	      (ewin1->shape_x != (VRoot.w - EoGetW(ewin1)) &&
+	      (ewin1->shape_x != (WinGetW(VROOT) - EoGetW(ewin1)) &&
 	       (dd > Conf.snap.edge_snap_dist)))))
 	  {
 	     jumpx = 1;
@@ -587,10 +605,10 @@ ActionMoveHandleMotion(void)
 	if ((ndy != dy) &&
 	    (((ewin1->shape_y == 0) &&
 	      (dd > screen_snap_dist)) ||
-	     ((ewin1->shape_y == (VRoot.h - EoGetH(ewin1))) &&
+	     ((ewin1->shape_y == (WinGetH(VROOT) - EoGetH(ewin1))) &&
 	      (dd > screen_snap_dist)) ||
 	     ((ewin1->shape_y != 0) &&
-	      (ewin1->shape_y != (VRoot.h - EoGetH(ewin1)) &&
+	      (ewin1->shape_y != (WinGetH(VROOT) - EoGetH(ewin1)) &&
 	       (dd > Conf.snap.edge_snap_dist)))))
 	  {
 	     jumpy = 1;
@@ -648,19 +666,17 @@ ActionMoveHandleMotion(void)
 
 		  /* check for sufficient overlap and avoid flickering */
 		  if (((ewin1->shape_x >= ewin2->shape_x &&
-			ewin1->shape_x <= ewin2->shape_x +
-			EoGetW(ewin2) / 2 && Mode.events.x <= Mode.events.px) ||
+			ewin1->shape_x <= ewin2->shape_x + EoGetW(ewin2) / 2 &&
+			dx <= 0) ||
 		       (ewin1->shape_x <= ewin2->shape_x &&
-			ewin1->shape_x + EoGetW(ewin1) / 2 >=
-			ewin2->shape_x &&
-			Mode.events.x >= Mode.events.px)) &&
-		      ((ewin1->shape_y >= ewin2->shape_y
-			&& ewin1->shape_y <=
-			ewin2->shape_y + EoGetH(ewin2) / 2
-			&& Mode.events.y <= Mode.events.py)
-		       || (EoGetY(ewin1) <= EoGetY(ewin2)
-			   && ewin1->shape_y + EoGetH(ewin1) / 2 >=
-			   ewin2->shape_y && Mode.events.y >= Mode.events.py)))
+			ewin1->shape_x + EoGetW(ewin1) / 2 >= ewin2->shape_x &&
+			dx >= 0)) &&
+		      ((ewin1->shape_y >= ewin2->shape_y &&
+			ewin1->shape_y <= ewin2->shape_y + EoGetH(ewin2) / 2 &&
+			dy <= 0) ||
+		       (EoGetY(ewin1) <= EoGetY(ewin2) &&
+			ewin1->shape_y + EoGetH(ewin1) / 2 >= ewin2->shape_y &&
+			dy >= 0)))
 		    {
 		       int                 tmp_swapcoord_x;
 		       int                 tmp_swapcoord_y;
@@ -678,6 +694,8 @@ ActionMoveHandleMotion(void)
 	     Efree(all_gwins);
 	  }
      }
+
+ done:
    Efree(gwins);
 }
 
@@ -701,12 +719,12 @@ ActionResizeHandleMotion(void)
      default:
 	break;
      case 1:			/* Left */
-	w = Mode_mr.win_w - (Mode.events.x - Mode_mr.start_x);
+	w = Mode_mr.win_w - (Mode.events.mx - Mode_mr.start_x);
 	ICCCM_SizeMatch(ewin, w, h, &w, &h);
 	x = Mode_mr.win_x + (Mode_mr.win_w - w);
 	break;
      case 2:			/* Right */
-	w = Mode_mr.win_w + (Mode.events.x - Mode_mr.start_x);
+	w = Mode_mr.win_w + (Mode.events.mx - Mode_mr.start_x);
 	ICCCM_SizeMatch(ewin, w, h, &w, &h);
 	break;
      }
@@ -716,12 +734,12 @@ ActionResizeHandleMotion(void)
      default:
 	break;
      case 1:			/* Top */
-	h = Mode_mr.win_h - (Mode.events.y - Mode_mr.start_y);
+	h = Mode_mr.win_h - (Mode.events.my - Mode_mr.start_y);
 	ICCCM_SizeMatch(ewin, w, h, &w, &h);
 	y = Mode_mr.win_y + (Mode_mr.win_h - h);
 	break;
      case 2:			/* Bottom */
-	h = Mode_mr.win_h + (Mode.events.y - Mode_mr.start_y);
+	h = Mode_mr.win_h + (Mode.events.my - Mode_mr.start_y);
 	ICCCM_SizeMatch(ewin, w, h, &w, &h);
 	break;
      }
