@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -40,7 +40,7 @@
 #include "tooltips.h"
 #include "xwin.h"
 
-#define DEBUG_PAGER 1
+#define DEBUG_PAGER 0
 #if DEBUG_PAGER
 #define Dprintf(fmt...) if(EDebug(EDBUG_TYPE_PAGER))Eprintf(fmt)
 #else
@@ -48,19 +48,22 @@
 #endif
 
 #define USE_PAGER_BACKGROUND_CACHE 1
-#define IMG_SCALE 2
+#define HIQ ((Conf_pagers.hiq) ? 0x200 : 0x100)
 
 #define PAGER_MODE_SIMPLE 0
 #define PAGER_MODE_SNAP   1
 #define PAGER_MODE_LIVE   2
 
+#define PAGER_UPD_EWIN_GEOM     0
+#define PAGER_UPD_EWIN_GONE     1
+#define PAGER_UPD_EWIN_DAMAGE   2
+
 #define EwinGetVX(ew) (ew->vx)
 #define EwinGetVY(ew) (ew->vy)
 #define EwinGetVX2(ew) (ew->vx + EoGetW(ew))
 #define EwinGetVY2(ew) (ew->vy + EoGetH(ew))
 
-static struct
-{
+static struct {
    char                enable;
    char                zoom;
    char                title;
@@ -72,14 +75,14 @@ static struct
    int                 menu_button;
 } Conf_pagers;
 
-static struct
-{
+static struct {
    int                 zoom;
    Idler              *idler;
+   char                update_pending;
+   char                timer_pending;
 } Mode_pagers;
 
-typedef struct
-{
+typedef struct {
    EWin               *ewin;
    Win                 win;
    int                 w, h;
@@ -103,19 +106,14 @@ typedef struct
 
 static void         PagerScanCancel(Pager * p);
 static void         PagerScanTimeout(int val, void *data);
-static void         PagerUpdateTimeout(int val, void *data);
 static void         PagerCheckUpdate(Pager * p, void *prm);
 static void         PagerUpdateEwinsFromPager(Pager * p);
 static void         PagerHiwinHide(void);
-static void         PagerEwinGroupSet(void);
 static void         PagerEvent(Win win, XEvent * ev, void *prm);
 static void         PagerHiwinEvent(Win win, XEvent * ev, void *prm);
-static void         doPagerUpdate(Pager * p);
 
 static Ecore_List  *pager_list = NULL;
 
-static char         pager_update_pending = 0;
-
 static Hiwin       *hiwin = NULL;
 
 static int
@@ -146,7 +144,7 @@ PagerCreate(void)
    ecore_list_append(pager_list, p);
 
    p->name = NULL;
-   p->win = ECreateClientWindow(VRoot.win, 0, 0, 1, 1);
+   p->win = ECreateClientWindow(VROOT, 0, 0, 1, 1);
    EventCallbackRegister(p->win, 0, PagerEvent, p);
    p->sel_win = ECreateWindow(p->win, 0, 0, 1, 1, 0);
 
@@ -159,8 +157,7 @@ PagerDestroy(Pager * p)
    ecore_list_node_remove(pager_list, p);
 
    PagerScanCancel(p);
-   if (p->name)
-      Efree(p->name);
+   Efree(p->name);
    EDestroyWindow(p->win);
    PagerHiwinHide();
    if (p->bgpmap != None)
@@ -204,7 +201,7 @@ PagerScanTimeout(int val __UNUSED__, void *data)
    static int          offsets[8] = { 0, 4, 2, 6, 1, 5, 3, 7 };
    int                 pager_mode = PagersGetMode();
 
-   if (pager_mode == PAGER_MODE_SIMPLE)
+   if (pager_mode != PAGER_MODE_SNAP)
       return;
 
    p = (Pager *) data;
@@ -213,13 +210,10 @@ PagerScanTimeout(int val __UNUSED__, void *data)
    ewin = p->ewin;
    if (!ewin || !EoIsShown(ewin))
       return;
-   if (pager_mode == PAGER_MODE_SNAP)
-     {
-	if (p->dsk != DesksGetCurrent())
-	   return;
-	if (ewin->state.visibility == VisibilityFullyObscured)
-	   return;
-     }
+   if (p->dsk != DesksGetCurrent())
+      return;
+   if (ewin->state.visibility == VisibilityFullyObscured)
+      return;
 
    if (Conf_pagers.scanspeed > 0)
       PagerScanTrig(p);
@@ -227,12 +221,6 @@ PagerScanTimeout(int val __UNUSED__, void *data)
    if (Mode.mode != MODE_NONE)
       return;
 
-   if (pager_mode == PAGER_MODE_LIVE)
-     {
-	doPagerUpdate(p);
-	return;
-     }
-
    DeskCurrentGetArea(&cx, &cy);
    ww = p->dw;
    hh = p->dh;
@@ -246,19 +234,19 @@ PagerScanTimeout(int val __UNUSED__, void *data)
    /* Due to a bug in imlib2 <= 1.2.0 we have to scan left->right in stead
     * of top->bottom, at least for now. */
    y = ((phase & 0xfffffff8) + offsets[phase % 8]) % hh;
-   y2 = (y * VRoot.h) / hh;
+   y2 = (y * WinGetH(VROOT)) / hh;
 
-   ScaleRect(VRoot.win, VRoot.xwin, p->win, WinGetPmap(p->win), NULL, 0, y2,
-	     VRoot.w, VRoot.h / hh, xx, yy + y, ww, 1, Conf_pagers.hiq);
-   EClearArea(p->win, xx, yy + y, ww, 1, False);
+   ScaleRect(VROOT, WinGetXwin(VROOT), p->win, WinGetPmap(p->win), 0,
+	     y2, WinGetW(VROOT), WinGetH(VROOT) / hh, xx, yy + y, ww, 1, HIQ);
+   EClearArea(p->win, xx, yy + y, ww, 1);
    y2 = p->h;
 #else
    y = ((phase & 0xfffffff8) + offsets[phase % 8]) % ww;
-   y2 = (y * VRoot.w) / ww;
+   y2 = (y * WinGetW(VROOT)) / ww;
 
-   ScaleRect(VRoot.win, VRoot.xwin, p->win, WinGetPmap(p->win), NULL, y2, 0,
-	     VRoot.w / ww, VRoot.h, xx + y, yy, 1, hh, Conf_pagers.hiq);
-   EClearArea(p->win, xx + y, yy, 1, hh, False);
+   ScaleRect(VROOT, WinGetXwin(VROOT), p->win, WinGetPmap(p->win), y2,
+	     0, WinGetW(VROOT) / ww, WinGetH(VROOT), xx + y, yy, 1, hh, HIQ);
+   EClearArea(p->win, xx + y, yy, 1, hh);
    y2 = p->w;
 #endif
    p->update_phase++;
@@ -292,8 +280,8 @@ PagerEwinUpdateMini(Pager * p, EWin * ewin)
    Drawable            draw;
    int                 pager_mode = PagersGetMode();
 
-   w = (EoGetW(ewin) * p->dw) / VRoot.w;
-   h = (EoGetH(ewin) * p->dh) / VRoot.h;
+   w = (EoGetW(ewin) * p->dw) / WinGetW(VROOT);
+   h = (EoGetH(ewin) * p->dh) / WinGetH(VROOT);
 
    if (w < 1)
       w = 1;
@@ -327,6 +315,8 @@ PagerEwinUpdateMini(Pager * p, EWin * ewin)
 
    ewin->mini_w = w;
    ewin->mini_h = h;
+   ewin->mini_pmm.type = 0;
+   ewin->mini_pmm.pmap = ECreatePixmap(p->win, w, h, 0);
 
    draw = None;
    if (pager_mode != PAGER_MODE_SIMPLE)
@@ -342,23 +332,14 @@ PagerEwinUpdateMini(Pager * p, EWin * ewin)
 	ImageClass         *ic;
 
 	ic = ImageclassFind("PAGER_WIN", 0);
-	if (ic)
-	  {
-	     ewin->mini_pmm.type = 0;
-	     ewin->mini_pmm.mask = None;
-	     ewin->mini_pmm.pmap =
-		ImageclassApplySimple(ic, p->win, None, STATE_NORMAL,
-				      0, 0, w, h);
-	  }
+	ImageclassApplySimple(ic, p->win, ewin->mini_pmm.pmap,
+			      STATE_NORMAL, 0, 0, w, h);
 	Dprintf("Use Iclass, pmap=%#lx\n", ewin->mini_pmm.pmap);
      }
    else
      {
-	ewin->mini_pmm.type = 1;
-	ewin->mini_pmm.mask = None;
-	ScaleRect(EoGetWin(ewin), draw, p->win, None, &ewin->mini_pmm.pmap,
-		  0, 0, EoGetW(ewin), EoGetH(ewin), 0, 0, w, h,
-		  Conf_pagers.hiq);
+	ScaleRect(EoGetWin(ewin), draw, p->win, ewin->mini_pmm.pmap,
+		  0, 0, EoGetW(ewin), EoGetH(ewin), 0, 0, w, h, HIQ);
 	Dprintf("Grab scaled, pmap=%#lx\n", ewin->mini_pmm.pmap);
      }
 
@@ -381,8 +362,8 @@ doPagerUpdate(Pager * p)
    p->update_phase = 0;
    DesksGetAreaSize(&ax, &ay);
    DeskGetArea(p->dsk, &cx, &cy);
-   vx = cx * VRoot.w;
-   vy = cy * VRoot.h;
+   vx = cx * WinGetW(VROOT);
+   vy = cy * WinGetH(VROOT);
 
    update_screen_included = update_screen_only = 0;
    if (pager_mode == PAGER_MODE_SNAP && p->dsk == DesksGetCurrent()
@@ -390,12 +371,12 @@ doPagerUpdate(Pager * p)
      {
 	/* Update from screen unless update area is entirely off-screen */
 	if (!(p->x2 <= vx || p->y2 <= vy ||
-	      p->x1 >= vx + VRoot.w || p->y1 >= vy + VRoot.h))
+	      p->x1 >= vx + WinGetW(VROOT) || p->y1 >= vy + WinGetH(VROOT)))
 	   update_screen_included = 1;
 
 	/* Check if update area is entirely on-screen */
 	if (p->x1 >= vx && p->y1 >= vy &&
-	    p->x2 <= vx + VRoot.w && p->y2 <= vy + VRoot.h)
+	    p->x2 <= vx + WinGetW(VROOT) && p->y2 <= vy + WinGetH(VROOT))
 	   update_screen_only = 1;
      }
    p->x1 = p->y1 = 99999;
@@ -452,10 +433,10 @@ doPagerUpdate(Pager * p)
 	if (!EoIsShown(ewin))
 	   continue;
 
-	wx = (EwinGetVX(ewin) * p->dw) / VRoot.w;
-	wy = (EwinGetVY(ewin) * p->dh) / VRoot.h;
-	ww = (EoGetW(ewin) * p->dw) / VRoot.w;
-	wh = (EoGetH(ewin) * p->dh) / VRoot.h;
+	wx = (EwinGetVX(ewin) * p->dw) / WinGetW(VROOT);
+	wy = (EwinGetVY(ewin) * p->dh) / WinGetH(VROOT);
+	ww = (EoGetW(ewin) * p->dw) / WinGetW(VROOT);
+	wh = (EoGetH(ewin) * p->dh) / WinGetH(VROOT);
 
 	if (ewin->mini_pmm.pmap)
 	  {
@@ -471,9 +452,9 @@ doPagerUpdate(Pager * p)
 	  }
 	else
 	  {
-	     XSetForeground(disp, gc, BlackPixel(disp, VRoot.scr));
+	     XSetForeground(disp, gc, Dpy.pixel_black);
 	     XDrawRectangle(disp, pmap, gc, wx - 1, wy - 1, ww + 1, wh + 1);
-	     XSetForeground(disp, gc, WhitePixel(disp, VRoot.scr));
+	     XSetForeground(disp, gc, Dpy.pixel_white);
 	     XFillRectangle(disp, pmap, gc, wx, wy, ww, wh);
 	  }
      }
@@ -487,9 +468,9 @@ doPagerUpdate(Pager * p)
  do_screen_update:
    Dprintf("doPagerUpdate %d: Snap screen\n", p->dsk->num);
    /* Update pager area by snapshotting entire screen */
-   ScaleRect(VRoot.win, VRoot.xwin, p->win, pmap, NULL, 0, 0,
-	     VRoot.w, VRoot.h, cx * p->dw, cy * p->dh, p->dw, p->dh,
-	     Conf_pagers.hiq);
+   ScaleRect(VROOT, WinGetXwin(VROOT), p->win, pmap, 0, 0,
+	     WinGetW(VROOT), WinGetH(VROOT), cx * p->dw, cy * p->dh,
+	     p->dw, p->dh, HIQ);
 
    EClearWindow(p->win);
 
@@ -502,7 +483,7 @@ doPagerUpdate(Pager * p)
 }
 
 static void
-PagerUpdate(Pager * p, int x1, int y1, int x2, int y2)
+PagerUpdate(Pager * p, int why, int x1, int y1, int x2, int y2)
 {
    if (!Conf_pagers.enable)
       return;
@@ -517,12 +498,7 @@ PagerUpdate(Pager * p, int x1, int y1, int x2, int y2)
       p->y2 = y2;
 
    p->do_update = 1;
-   pager_update_pending = 1;
-
-   if (PagersGetMode() == PAGER_MODE_SIMPLE)
-      return;
-
-   DoIn("pg-upd", .2, PagerUpdateTimeout, 0, NULL);
+   Mode_pagers.update_pending |= 1 << why;
 }
 
 static void
@@ -533,7 +509,7 @@ PagerReconfigure(Pager * p, int apply)
 
    DesksGetAreaSize(&ax, &ay);
 
-   aspect = ((double)VRoot.w) / ((double)VRoot.h);
+   aspect = ((double)WinGetW(VROOT)) / ((double)WinGetH(VROOT));
 
    dx = 2;
    for (;;)
@@ -551,16 +527,17 @@ PagerReconfigure(Pager * p, int apply)
      }
 
    ICCCM_SetSizeConstraints(p->ewin,
-			    VRoot.w / 64 * ax, VRoot.h / 64 * ay,
-			    VRoot.w / 4 * ax, VRoot.h / 4 * ay,
-			    0, 0, dx * ax, dy * ay,
+			    WinGetW(VROOT) / 64 * ax,
+			    WinGetH(VROOT) / 64 * ay,
+			    WinGetW(VROOT) / 4 * ax,
+			    WinGetH(VROOT) / 4 * ay, 0, 0, dx * ax, dy * ay,
 			    aspect * ((double)ax / (double)ay),
 			    aspect * ((double)ax / (double)ay));
 
    if (apply)
      {
-	w = (int)((double)ax * (double)VRoot.w / (double)p->scale + .5);
-	h = (int)((double)ay * (double)VRoot.h / (double)p->scale + .5);
+	w = (int)((double)ax * (double)WinGetW(VROOT) / (double)p->scale + .5);
+	h = (int)((double)ay * (double)WinGetH(VROOT) / (double)p->scale + .5);
 	EwinResize(p->ewin, w + (dx * ax) / 2, h + (dy * ay) / 2);
 	EwinReposition(p->ewin);
      }
@@ -617,8 +594,8 @@ PagerUpdateBg(Pager * p)
 
    if (pager_mode != PAGER_MODE_SIMPLE && p->dsk->bg.pmap)
      {
-	ScaleRect(VRoot.win, p->dsk->bg.pmap, p->win, pmap, NULL, 0, 0,
-		  VRoot.w, VRoot.h, 0, 0, p->dw, p->dh, Conf_pagers.hiq);
+	ScaleTile(VROOT, p->dsk->bg.pmap, p->win, pmap,
+		  0, 0, p->dw, p->dh, HIQ);
 	return;
      }
 
@@ -634,9 +611,9 @@ PagerUpdateBg(Pager * p)
    if (gc == None)
       return;
 
-   XSetForeground(disp, gc, BlackPixel(disp, VRoot.scr));
+   XSetForeground(disp, gc, Dpy.pixel_black);
    XDrawRectangle(disp, pmap, gc, 0, 0, p->dw, p->dh);
-   XSetForeground(disp, gc, WhitePixel(disp, VRoot.scr));
+   XSetForeground(disp, gc, Dpy.pixel_white);
    XFillRectangle(disp, pmap, gc, 1, 1, p->dw - 2, p->dh - 2);
 
    EXFreeGC(gc);
@@ -687,7 +664,8 @@ _PagerEwinMoveResize(EWin * ewin, int resize __UNUSED__)
    p->dh = h / ay;
 
    if (p->scale <= 0. || Mode.op_source == OPSRC_USER)
-      p->scale = ((float)VRoot.w / p->dw + (float)VRoot.h / p->dh) / 2;
+      p->scale = ((float)WinGetW(VROOT) / p->dw +
+		  (float)WinGetH(VROOT) / p->dh) / 2;
 
    p->do_newbg = 1;
    PagerCheckUpdate(p, NULL);
@@ -737,8 +715,8 @@ PagerShow(Pager * p)
    if (!ewin)
       return;
 
-   p->screen_w = VRoot.w;
-   p->screen_h = VRoot.h;
+   p->screen_w = WinGetW(VROOT);
+   p->screen_h = WinGetH(VROOT);
 
    PagerReconfigure(p, 0);
 
@@ -761,39 +739,46 @@ PagerShow(Pager * p)
 	int                 ax, ay;
 
 	DesksGetAreaSize(&ax, &ay);
-	w = ((48 * VRoot.w) / VRoot.h) * ax;
+	w = ((48 * WinGetW(VROOT)) / WinGetH(VROOT)) * ax;
 	h = 48 * ay;
 	EwinResize(ewin, w, h);	/* Does layout */
-	EwinMove(ewin, 0,
-		 VRoot.h - (DesksGetNumber() - p->dsk->num) * EoGetH(ewin));
+	EwinMove(ewin, 0, WinGetH(VROOT) -
+		 (DesksGetNumber() - p->dsk->num) * EoGetH(ewin));
      }
 
    EwinShow(ewin);
 }
 
+typedef struct {
+   Desk               *dsk;
+   void                (*func) (Pager * p, void *prm);
+   void               *prm;
+} pager_foreach_data;
+
+static void
+_PagersForeachFunc(void *item, void *prm)
+{
+   Pager              *p = (Pager *) item;
+   pager_foreach_data *data = (pager_foreach_data *) prm;
+
+   if (data->dsk && data->dsk != p->dsk)
+      return;
+   data->func(p, data->prm);
+}
+
 static void
 PagersForeach(Desk * dsk, void (*func) (Pager * p, void *prm), void *prm)
 {
-   Pager              *p, *p_cur;
+   pager_foreach_data  data;
 
-   if (!pager_list)
-      return;
-
-   /* We may get here recursively */
-   p_cur = (Pager *) ecore_list_current(pager_list);
-
-   ECORE_LIST_FOR_EACH(pager_list, p)
-   {
-      if (dsk && dsk != p->dsk)
-	 continue;
-      func(p, prm);
-   }
-
-   ecore_list_goto(pager_list, p_cur);
+   data.dsk = dsk;
+   data.func = func;
+   data.prm = prm;
+   ecore_list_for_each(pager_list, _PagersForeachFunc, &data);
 }
 
-typedef struct
-{
+typedef struct {
+   int                 why;
    int                 x1, y1, x2, y2;
 } pager_update_data;
 
@@ -802,17 +787,18 @@ _PagerUpdate(Pager * p, void *prm)
 {
    pager_update_data  *pud = (pager_update_data *) prm;
 
-   PagerUpdate(p, pud->x1, pud->y1, pud->x2, pud->y2);
+   PagerUpdate(p, pud->why, pud->x1, pud->y1, pud->x2, pud->y2);
 }
 
 static void
-PagersUpdate(Desk * dsk, int x1, int y1, int x2, int y2)
+PagersUpdate(Desk * dsk, int why, int x1, int y1, int x2, int y2)
 {
    pager_update_data   pud;
 
    if (ecore_list_count(pager_list) <= 0)
       return;
 
+   pud.why = why;
    pud.x1 = x1;
    pud.y1 = y1;
    pud.x2 = x2;
@@ -836,20 +822,38 @@ PagerCheckUpdate(Pager * p, void *prm __UNUSED__)
 }
 
 static void
-PagersCheckUpdate(void)
+_PagersUpdateTimeout(int val __UNUSED__, void *data __UNUSED__)
 {
-   if (!pager_update_pending || !Conf_pagers.enable)
-      return;
-
-   PagersForeach(NULL, PagerCheckUpdate, NULL);
-
-   pager_update_pending = 0;
+   Mode_pagers.timer_pending = 0;
 }
 
 static void
-PagerUpdateTimeout(int val __UNUSED__, void *data __UNUSED__)
+PagersCheckUpdate(void)
 {
-   PagersCheckUpdate();
+   static double       tlast = 0.;
+   double              t, dt;
+
+   if (!Mode_pagers.update_pending || !Conf_pagers.enable)
+      return;
+
+   if (Mode_pagers.update_pending == (1 << PAGER_UPD_EWIN_DAMAGE))
+     {
+	t = GetTime();
+	dt = (Conf_pagers.scanspeed > 0) ? 1. / Conf_pagers.scanspeed : .1;
+	if (t - tlast < dt)
+	  {
+	     /* The purpose of this timer is to trigger the idler */
+	     if (!Mode_pagers.timer_pending)
+		DoIn("pg-upd", dt, _PagersUpdateTimeout, 0, NULL);
+	     Mode_pagers.timer_pending = 1;
+	     return;
+	  }
+	tlast = t;
+     }
+
+   PagersForeach(NULL, PagerCheckUpdate, NULL);
+
+   Mode_pagers.update_pending = 0;
 }
 
 static void
@@ -873,10 +877,10 @@ PagerEwinUpdateFromPager(Pager * p, EWin * ewin)
    y = EwinGetVY(ewin);
    w = EoGetW(ewin);
    h = EoGetH(ewin);
-   x = (x * p->dw) / VRoot.w;
-   y = (y * p->dh) / VRoot.h;
-   w = (w * p->dw) / VRoot.w;
-   h = (h * p->dh) / VRoot.h;
+   x = (x * p->dw) / WinGetW(VROOT);
+   y = (y * p->dh) / WinGetH(VROOT);
+   w = (w * p->dw) / WinGetW(VROOT);
+   h = (h * p->dh) / WinGetH(VROOT);
    if (w <= 0)
       w = 1;
    if (h <= 0)
@@ -924,21 +928,35 @@ PagerUpdateEwinsFromPager(Pager * p)
 }
 
 static void
-PagersUpdateEwin(EWin * ewin, int gone)
+PagersUpdateEwin(EWin * ewin, int why)
 {
    Desk               *dsk;
 
    if (!Conf_pagers.enable)
       return;
 
-   if (!gone && (!EoIsShown(ewin) || ewin->state.animated))
-      return;
+   switch (why)
+     {
+     case PAGER_UPD_EWIN_GEOM:
+	if (!EoIsShown(ewin) || ewin->state.animated)
+	   return;
+	break;
 
-   if (gone && ewin == HiwinGetEwin(hiwin, 0))
-      PagerHiwinHide();
+     case PAGER_UPD_EWIN_GONE:
+	if (ewin == HiwinGetEwin(hiwin, 0))
+	   PagerHiwinHide();
+	break;
+
+     case PAGER_UPD_EWIN_DAMAGE:
+	if (ewin->type == EWIN_TYPE_PAGER)
+	   return;
+	if (PagersGetMode() != PAGER_MODE_LIVE)
+	   return;
+	break;
+     }
 
    dsk = (EoIsFloating(ewin)) ? DesksGetCurrent() : EoGetDesk(ewin);
-   PagersUpdate(dsk, EwinGetVX(ewin), EwinGetVY(ewin),
+   PagersUpdate(dsk, why, EwinGetVX(ewin), EwinGetVY(ewin),
 		EwinGetVX2(ewin), EwinGetVY2(ewin));
 }
 
@@ -958,10 +976,10 @@ EwinInPagerAt(Pager * p, int px, int py)
 	if (!EoIsShown(ewin))
 	   continue;
 
-	x = (EwinGetVX(ewin) * p->dw) / VRoot.w;
-	y = (EwinGetVY(ewin) * p->dh) / VRoot.h;
-	w = (EoGetW(ewin) * p->dw) / VRoot.w;
-	h = (EoGetH(ewin) * p->dh) / VRoot.h;
+	x = (EwinGetVX(ewin) * p->dw) / WinGetW(VROOT);
+	y = (EwinGetVY(ewin) * p->dh) / WinGetH(VROOT);
+	w = (EoGetW(ewin) * p->dw) / WinGetW(VROOT);
+	h = (EoGetH(ewin) * p->dh) / WinGetH(VROOT);
 
 	if (px >= x && py >= y && px < (x + w) && py < (y + h))
 	   return ewin;
@@ -1094,8 +1112,8 @@ PagerShowTt(EWin * ewin)
    if (tt)
      {
 	if (ewin)
-	   TooltipShow(tt, EwinGetIconName(ewin), NULL, Mode.events.x,
-		       Mode.events.y);
+	   TooltipShow(tt, EwinGetIconName(ewin), NULL, Mode.events.cx,
+		       Mode.events.cy);
 	else
 	   TooltipHide(tt);
      }
@@ -1117,11 +1135,11 @@ PagerHiwinInit(Pager * p, EWin * ewin)
 	hiwin = phi;
      }
 
-   wx = (EwinGetVX(ewin) * p->dw) / VRoot.w;
-   wy = (EwinGetVY(ewin) * p->dh) / VRoot.h;
-   ww = (EoGetW(ewin) * p->dw) / VRoot.w;
-   wh = (EoGetH(ewin) * p->dh) / VRoot.h;
-   ETranslateCoordinates(p->win, VRoot.win, 0, 0, &px, &py, NULL);
+   wx = (EwinGetVX(ewin) * p->dw) / WinGetW(VROOT);
+   wy = (EwinGetVY(ewin) * p->dh) / WinGetH(VROOT);
+   ww = (EoGetW(ewin) * p->dw) / WinGetW(VROOT);
+   wh = (EoGetH(ewin) * p->dh) / WinGetH(VROOT);
+   ETranslateCoordinates(p->win, VROOT, 0, 0, &px, &py, NULL);
 
    HiwinInit(phi, ewin);
    HiwinSetGeom(phi, px + wx, py + wy, ww, wh);
@@ -1240,7 +1258,7 @@ PagersUpdateBackground(Desk * dsk)
 {
    PagersForeach(dsk, _PagerUpdateBackground, NULL);
 
-   pager_update_pending = 1;
+   Mode_pagers.update_pending = 1;
 }
 
 static void
@@ -1289,7 +1307,7 @@ PagersSetMode(int mode)
 
    PagersUpdateBackground(NULL);
 
-   if (Conf_pagers.mode != PAGER_MODE_SIMPLE && Conf_pagers.scanspeed > 0)
+   if (Conf_pagers.mode == PAGER_MODE_SNAP && Conf_pagers.scanspeed > 0)
       PagersForeach(DesksGetCurrent(), _PagerSetSnap, NULL);
 
    autosave();
@@ -1365,8 +1383,7 @@ EwinGroupMove(EWin * ewin, Desk * dsk, int x, int y)
 	   EwinOpMove(gwins[i], OPSRC_USER, EoGetX(gwins[i]) + dx,
 		      EoGetY(gwins[i]) + dy);
      }
-   if (gwins)
-      Efree(gwins);
+   Efree(gwins);
 }
 
 static void
@@ -1377,8 +1394,8 @@ PagerEwinMove(Pager * p __UNUSED__, Pager * pd, EWin * ewin)
    Hiwin              *phi = hiwin;
 
    /* Delta in pager coords */
-   dx = Mode.events.x - Mode.events.px;
-   dy = Mode.events.y - Mode.events.py;
+   dx = Mode.events.mx - Mode.events.px;
+   dy = Mode.events.my - Mode.events.py;
 
    if (dx == 0 && dy == 0)
       return;
@@ -1388,10 +1405,10 @@ PagerEwinMove(Pager * p __UNUSED__, Pager * pd, EWin * ewin)
    HiwinMove(phi, x + dx, y + dy);
 
    /* Find real window position */
-   ETranslateCoordinates(VRoot.win, pd->win, x, y, &px, &py, NULL);
+   ETranslateCoordinates(VROOT, pd->win, x, y, &px, &py, NULL);
    DeskGetArea(pd->dsk, &cx, &cy);
-   x = (px * VRoot.w) / pd->dw - cx * VRoot.w;
-   y = (py * VRoot.h) / pd->dh - cy * VRoot.h;
+   x = (px * WinGetW(VROOT)) / pd->dw - cx * WinGetW(VROOT);
+   y = (py * WinGetH(VROOT)) / pd->dh - cy * WinGetH(VROOT);
 
    /* Move all group members */
    EwinGroupMove(ewin, pd->dsk, x, y);
@@ -1485,8 +1502,8 @@ PagerHiwinHandleMouseUp(Pager * p, int px, int py, int button)
    if (!ewin)
       goto done;
 
-   in_vroot = (Mode.events.x >= 0 && Mode.events.x < VRoot.w &&
-	       Mode.events.y >= 0 && Mode.events.y < VRoot.h);
+   in_vroot = (Mode.events.cx >= 0 && Mode.events.cx < WinGetW(VROOT) &&
+	       Mode.events.cy >= 0 && Mode.events.cy < WinGetH(VROOT));
 
    if (button == Conf_pagers.win_button)
      {
@@ -1511,8 +1528,7 @@ PagerHiwinHandleMouseUp(Pager * p, int px, int py, int button)
 		       EwinIconify(gwins[i]);
 		    }
 	       }
-	     if (gwins)
-		Efree(gwins);
+	     Efree(gwins);
 	  }
 	else if (ewin2 && ewin2->props.vroot)
 	  {
@@ -1522,14 +1538,14 @@ PagerHiwinHandleMouseUp(Pager * p, int px, int py, int button)
 	else if (!in_vroot)
 	  {
 	     /* Move back to real root */
-	     EwinReparent(ewin, RRoot.win);
+	     EwinReparent(ewin, RROOT);
 	  }
 	else
 	  {
 	     /* Pointer is not in pager or iconbox */
 	     /* Move window(s) to pointer location */
-	     x = Mode.events.x - EoGetW(ewin) / 2;
-	     y = Mode.events.y - EoGetH(ewin) / 2;
+	     x = Mode.events.cx - EoGetW(ewin) / 2;
+	     y = Mode.events.cy - EoGetH(ewin) / 2;
 	     EwinGroupMove(ewin, DesksGetCurrent(), x, y);
 	  }
      }
@@ -2012,7 +2028,8 @@ PagersSighan(int sig, void *prm)
 	PagerHiwinHide();
 	break;
      case ESIGNAL_AREA_SWITCH_DONE:
-	PagersUpdate(DesksGetCurrent(), 0, 0, 99999, 99999);
+	PagersUpdate(DesksGetCurrent(), PAGER_UPD_EWIN_GEOM,
+		     0, 0, 99999, 99999);
 	UpdatePagerSel();
 	break;
 
@@ -2039,10 +2056,13 @@ PagersSighan(int sig, void *prm)
 	break;
 
      case ESIGNAL_EWIN_UNMAP:
-	PagersUpdateEwin((EWin *) prm, 1);
+	PagersUpdateEwin((EWin *) prm, PAGER_UPD_EWIN_GONE);
 	break;
      case ESIGNAL_EWIN_CHANGE:
-	PagersUpdateEwin((EWin *) prm, 0);
+	PagersUpdateEwin((EWin *) prm, PAGER_UPD_EWIN_GEOM);
+	break;
+     case ESIGNAL_EWIN_DAMAGE:
+	PagersUpdateEwin((EWin *) prm, PAGER_UPD_EWIN_DAMAGE);
 	break;
      }
 }
