@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -25,6 +25,7 @@
 #include "dialog.h"
 #include "e16-ecore_hints.h"
 #include "emodule.h"
+#include "events.h"
 #include "ewins.h"
 #include "hints.h"
 #include "session.h"
@@ -54,8 +55,6 @@ static Window       new_init_win_ext = None;
 #endif
 #endif
 
-static int          sm_fd = -1;
-
 /* True if we are saving state for a doExit("restart") */
 static int          restarting = False;
 
@@ -71,6 +70,8 @@ static int          restarting = False;
 static char        *sm_client_id = NULL;
 static SmcConn      sm_conn = NULL;
 
+static EventFdDesc *sm_efd = NULL;
+
 static void
 set_save_props(SmcConn smc_conn, int master_flag)
 {
@@ -247,8 +248,7 @@ set_save_props(SmcConn smc_conn, int master_flag)
    props[n++] = &priorityProp;
 
    SmcSetProperties(smc_conn, n, props);
-   if (user)
-      Efree(user);
+   Efree(user);
 }
 
 /* This function is usually exclusively devoted to saving data.
@@ -321,24 +321,49 @@ ice_io_error_handler(IceConn connection __UNUSED__)
     * exit(1) instead of closing the losing connection. */
 }
 
-#endif /* USE_SM */
+static void
+ice_exit(void)
+{
+   SmcCloseConnection(sm_conn, 0, NULL);
+   sm_conn = NULL;
+   EventFdUnregister(sm_efd);
+}
 
-void
-SessionInit(void)
+static void
+ice_msgs_process(void)
+{
+   IceProcessMessagesStatus status;
+
+   status = IceProcessMessages(ice_conn, NULL, NULL);
+   if (status == IceProcessMessagesIOError)
+     {
+	/* Less of the hope.... E survives */
+	DialogAlert(_("ERROR!\n" "\n"
+		      "Lost the Session Manager that was there?\n"
+		      "Here here session manager... come here... want a bone?\n"
+		      "Oh come now! Stop sulking! Bugger. Oh well. "
+		      "Will continue without\n" "a session manager.\n" "\n"
+		      "I'll survive somehow.\n" "\n" "\n" "... I hope.\n"));
+	ice_exit();
+     }
+}
+
+static void
+ice_init(void)
 {
-#if USE_SM
    static SmPointer    context;
    SmcCallbacks        callbacks;
-#endif
+   char                error_string_ret[4096];
+   char               *client_id;
+   char                style[2];
+   SmPropValue         styleVal;
+   SmProp              styleProp;
+   SmProp             *props[1];
+   int                 sm_fd;
 
-   if (Mode.wm.window)
+   if (!getenv("SESSION_MANAGER"))
       return;
 
-#if USE_SM
-#if 0				/* Unused */
-   atom_sm_client_id = XInternAtom(disp, "SM_CLIENT_ID", False);
-#endif
-
    IceSetIOErrorHandler(ice_io_error_handler);
 
    callbacks.save_yourself.callback = callback_save_yourself;
@@ -349,54 +374,63 @@ SessionInit(void)
    callbacks.save_yourself.client_data = callbacks.die.client_data =
       callbacks.save_complete.client_data =
       callbacks.shutdown_cancelled.client_data = (SmPointer) NULL;
-   if (getenv("SESSION_MANAGER"))
-     {
-	char                error_string_ret[4096] = "";
-	char               *client_id = NULL;
 
-	if (sm_client_id)
-	   client_id = Estrdup(sm_client_id);
-	sm_conn =
-	   SmcOpenConnection(NULL, &context, SmProtoMajor, SmProtoMinor,
-			     SmcSaveYourselfProcMask | SmcDieProcMask |
-			     SmcSaveCompleteProcMask |
-			     SmcShutdownCancelledProcMask, &callbacks,
-			     client_id, &sm_client_id, 4096, error_string_ret);
-	if (client_id)
-	   Efree(client_id);
+   client_id = Estrdup(sm_client_id);
 
-	if (error_string_ret[0])
-	   Eprintf("While connecting to session manager: %s.",
-		   error_string_ret);
-     }
-   if (sm_conn)
-     {
-	char                style[2];
-	SmPropValue         styleVal;
-	SmProp              styleProp;
-	SmProp             *props[1];
+   error_string_ret[0] = '\0';
 
-	style[0] = SmRestartIfRunning;
-	style[1] = 0;
+   sm_conn =
+      SmcOpenConnection(NULL, &context, SmProtoMajor, SmProtoMinor,
+			SmcSaveYourselfProcMask | SmcDieProcMask |
+			SmcSaveCompleteProcMask |
+			SmcShutdownCancelledProcMask, &callbacks,
+			client_id, &sm_client_id, 4096, error_string_ret);
+   Efree(client_id);
 
-	styleVal.length = 1;
-	styleVal.value = style;
+   if (error_string_ret[0])
+      Eprintf("While connecting to session manager: %s.", error_string_ret);
 
-	styleProp.name = (char *)SmRestartStyleHint;
-	styleProp.type = (char *)SmCARD8;
-	styleProp.num_vals = 1;
-	styleProp.vals = &styleVal;
+   if (!sm_conn)
+      return;
 
-	props[0] = &styleProp;
+   style[0] = SmRestartIfRunning;
+   style[1] = 0;
+
+   styleVal.length = 1;
+   styleVal.value = style;
+
+   styleProp.name = (char *)SmRestartStyleHint;
+   styleProp.type = (char *)SmCARD8;
+   styleProp.num_vals = 1;
+   styleProp.vals = &styleVal;
+
+   props[0] = &styleProp;
+
+   ice_conn = SmcGetIceConnection(sm_conn);
+   sm_fd = IceConnectionNumber(ice_conn);
+   /* Just in case we are a copy of E created by a doExit("restart") */
+   SmcSetProperties(sm_conn, 1, props);
+   fcntl(sm_fd, F_SETFD, fcntl(sm_fd, F_GETFD, 0) | FD_CLOEXEC);
+
+   sm_efd = EventFdRegister(sm_fd, ice_msgs_process);
+}
 
-	ice_conn = SmcGetIceConnection(sm_conn);
-	sm_fd = IceConnectionNumber(ice_conn);
-	/* Just in case we are a copy of E created by a doExit("restart") */
-	SmcSetProperties(sm_conn, 1, props);
-	fcntl(sm_fd, F_SETFD, fcntl(sm_fd, F_GETFD, 0) | FD_CLOEXEC);
-     }
 #endif /* USE_SM */
 
+void
+SessionInit(void)
+{
+   if (Mode.wm.window)
+      return;
+
+#if 0				/* Unused */
+   atom_sm_client_id = EInternAtom("SM_CLIENT_ID");
+#endif
+
+#if USE_SM
+   ice_init();
+#endif
+
    if (!Conf.session.script)
       Conf.session.script = Estrdup("$EROOT/scripts/session.sh");
    if (!Conf.session.cmd_reboot)
@@ -405,37 +439,6 @@ SessionInit(void)
       Conf.session.cmd_halt = Estrdup("poweroff");
 }
 
-void
-ProcessICEMSGS(void)
-{
-#if USE_SM
-   IceProcessMessagesStatus status;
-
-   if (sm_fd < 0)
-      return;
-   status = IceProcessMessages(ice_conn, NULL, NULL);
-   if (status == IceProcessMessagesIOError)
-     {
-	/* Less of the hope.... E survives */
-	DialogAlert(_("ERROR!\n" "\n"
-		      "Lost the Session Manager that was there?\n"
-		      "Here here session manager... come here... want a bone?\n"
-		      "Oh come now! Stop sulking! Bugger. Oh well. "
-		      "Will continue without\n" "a session manager.\n" "\n"
-		      "I'll survive somehow.\n" "\n" "\n" "... I hope.\n"));
-	SmcCloseConnection(sm_conn, 0, NULL);
-	sm_conn = NULL;
-	sm_fd = -1;
-     }
-#endif /* USE_SM */
-}
-
-int
-GetSMfd(void)
-{
-   return sm_fd;
-}
-
 void
 SessionGetInfo(EWin * ewin)
 {
@@ -465,21 +468,17 @@ SetSMID(const char *smid)
 #endif /* USE_SM */
 }
 
-void
+static void
 SessionSave(int shutdown)
 {
    if (EDebug(EDBUG_TYPE_SESSION))
       Eprintf("SessionSave(%d)\n", shutdown);
 
-   /* dont' need anymore */
-   /* autosave(); */
+   Real_SaveSnapInfo(0, NULL);
+
 #if USE_SM
    if (shutdown && sm_conn)
-     {
-	SmcCloseConnection(sm_conn, 0, NULL);
-	sm_conn = NULL;
-	sm_fd = -1;
-     }
+      ice_exit();
 #endif /* USE_SM */
 }
 
@@ -504,9 +503,7 @@ doSMExit(int mode, const char *params)
 
    restarting = True;
 
-   if (!params)
-      SessionSave(1);
-   Real_SaveSnapInfo(0, NULL);
+   SessionSave(1);
 
    if (mode != EEXIT_THEME && mode != EEXIT_RESTART)
       SessionHelper(ESESSION_STOP);
@@ -517,9 +514,9 @@ doSMExit(int mode, const char *params)
      {
 	/* We may get here from HandleXIOError */
 	EwinsSetFree();
-	XSelectInput(disp, VRoot.xwin, 0);
+	ESelectInput(VROOT, 0);
 	ExtInitWinKill();
-	ESync();
+	ESync(0);
 
 	/* Forget about cleaning up if no disp */
 	ModulesSignal(ESIGNAL_EXIT, NULL);
@@ -543,7 +540,7 @@ doSMExit(int mode, const char *params)
      case EEXIT_RESTART:
 	SoundPlay("SOUND_WAIT");
 #ifdef USE_EXT_INIT_WIN
-	if (disp)
+	if (disp && !Mode.wm.window)
 	   new_init_win_ext = ExtInitWinCreate();
 #endif
 	EDisplayClose();
@@ -551,10 +548,13 @@ doSMExit(int mode, const char *params)
 	l = 0;
 	l += Esnprintf(s + l, sizeof(s) - l, "exec %s -f", Mode.wm.exec_name);
 	if (Mode.wm.single)
-	   l += Esnprintf(s + l, sizeof(s) - l, " -s %d", VRoot.scr);
+	   l += Esnprintf(s + l, sizeof(s) - l, " -s %d", Dpy.screen);
 	else if (!Mode.wm.master)
 	   l +=
 	      Esnprintf(s + l, sizeof(s) - l, " -m %d", Mode.wm.master_screen);
+	if (Mode.wm.window)
+	   l += Esnprintf(s + l, sizeof(s) - l, " -w %dx%d",
+			  WinGetW(VROOT), WinGetH(VROOT));
 #if USE_SM
 	if (sm_client_id)
 	   l += Esnprintf(s + l, sizeof(s) - l, " -S %s", sm_client_id);
@@ -676,7 +676,10 @@ SessionExit(int mode, const char *param)
       Eprintf("SessionExit: mode=%d(%d) prm=%s\n", mode, Mode.wm.exit_mode,
 	      param);
 
-   if (Mode.wm.startup || Mode.wm.in_signal_handler)
+   if (Mode.wm.exiting)
+      return;
+
+   if (Mode.wm.startup || Mode.wm.exit_now)
       goto done;
 
    switch (mode)
