@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -223,6 +223,7 @@ static char         tmp_with_leader;
 static char         tmp_switch_popup;
 static char         tmp_manual_placement;
 static char         tmp_manual_placement_mouse_pointer;
+static char         tmp_center_if_desk_full;
 static char         tmp_map_slide;
 static char         tmp_cleanup_slide;
 static int          tmp_slide_mode;
@@ -247,6 +248,7 @@ CB_ConfigurePlacement(Dialog * d __UNUSED__, int val, void *data __UNUSED__)
 
 	Conf.place.manual = tmp_manual_placement;
 	Conf.place.manual_mouse_pointer = tmp_manual_placement_mouse_pointer;
+	Conf.place.center_if_desk_full = tmp_center_if_desk_full;
 
 	Conf.place.slidein = tmp_map_slide;
 	Conf.place.cleanupslide = tmp_cleanup_slide;
@@ -277,6 +279,7 @@ _DlgFillPlacement(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
 
    tmp_manual_placement = Conf.place.manual;
    tmp_manual_placement_mouse_pointer = Conf.place.manual_mouse_pointer;
+   tmp_center_if_desk_full = Conf.place.center_if_desk_full;
 
    tmp_map_slide = Conf.place.slidein;
    tmp_cleanup_slide = Conf.place.cleanupslide;
@@ -319,6 +322,11 @@ _DlgFillPlacement(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSetText(di, _("Place windows under mouse"));
    DialogItemCheckButtonSetPtr(di, &tmp_manual_placement_mouse_pointer);
 
+   di = DialogAddItem(table, DITEM_CHECKBUTTON);
+   DialogItemSetColSpan(di, 2);
+   DialogItemSetText(di, _("Center windows when desk is full"));
+   DialogItemCheckButtonSetPtr(di, &tmp_center_if_desk_full);
+
    di = DialogAddItem(table, DITEM_CHECKBUTTON);
    DialogItemSetColSpan(di, 2);
    DialogItemSetText(di, _("Slide windows in when they appear"));
@@ -606,6 +614,17 @@ _DlgFillComposite(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemSetText(di, _("Enable Fading"));
    DialogItemCheckButtonSetPtr(di, &Cfg_composite.fading);
 
+   di = DialogAddItem(table, DITEM_TEXT);
+   DialogItemSetFill(di, 0, 0);
+   DialogItemSetAlign(di, 0, 512);
+   DialogItemSetText(di, _("Fading Speed:"));
+
+   di = DialogAddItem(table, DITEM_SLIDER);
+   DialogItemSliderSetBounds(di, 0, 100);
+   DialogItemSliderSetUnits(di, 5);
+   DialogItemSliderSetJump(di, 5);
+   DialogItemSliderSetValPtr(di, &Cfg_composite.fade_speed);
+
    di = DialogAddItem(table, DITEM_SEPARATOR);
    DialogItemSetColSpan(di, 2);
 
@@ -634,16 +653,41 @@ _DlgFillComposite(Dialog * d __UNUSED__, DItem * table, void *data __UNUSED__)
    DialogItemRadioButtonGroupSetVal(di, 2);
    DialogItemRadioButtonGroupSetValPtr(radio, &Cfg_composite.shadow);
 
+   di = DialogAddItem(table, DITEM_SEPARATOR);
+   DialogItemSetColSpan(di, 2);
+
    di = DialogAddItem(table, DITEM_TEXT);
    DialogItemSetFill(di, 0, 0);
    DialogItemSetAlign(di, 0, 512);
-   DialogItemSetText(di, _("Fading Speed:"));
+   DialogItemSetText(di, _("Default focused window opacity:"));
 
    di = DialogAddItem(table, DITEM_SLIDER);
    DialogItemSliderSetBounds(di, 0, 100);
    DialogItemSliderSetUnits(di, 5);
    DialogItemSliderSetJump(di, 5);
-   DialogItemSliderSetValPtr(di, &Cfg_composite.fade_speed);
+   DialogItemSliderSetValPtr(di, &Cfg_composite.opacity_focused);
+
+   di = DialogAddItem(table, DITEM_TEXT);
+   DialogItemSetFill(di, 0, 0);
+   DialogItemSetAlign(di, 0, 512);
+   DialogItemSetText(di, _("Default unfocused window opacity:"));
+
+   di = DialogAddItem(table, DITEM_SLIDER);
+   DialogItemSliderSetBounds(di, 0, 100);
+   DialogItemSliderSetUnits(di, 5);
+   DialogItemSliderSetJump(di, 5);
+   DialogItemSliderSetValPtr(di, &Cfg_composite.opacity_unfocused);
+
+   di = DialogAddItem(table, DITEM_TEXT);
+   DialogItemSetFill(di, 0, 0);
+   DialogItemSetAlign(di, 0, 512);
+   DialogItemSetText(di, _("Default pop-up window opacity:"));
+
+   di = DialogAddItem(table, DITEM_SLIDER);
+   DialogItemSliderSetBounds(di, 0, 100);
+   DialogItemSliderSetUnits(di, 5);
+   DialogItemSliderSetJump(di, 5);
+   DialogItemSliderSetValPtr(di, &Cfg_composite.opacity_override);
 }
 
 const DialogDef     DlgComposite = {
