@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -39,8 +39,7 @@
    ButtonPressMask | ButtonReleaseMask | EnterWindowMask | LeaveWindowMask | \
    PointerMotionMask)
 
-typedef struct
-{
+typedef struct {
    EObj                o;
    char               *name;
    char                direction;
@@ -54,8 +53,7 @@ static void         SlideoutCalcSize(Slideout * s);
 
 static Ecore_List  *slideout_list = NULL;
 
-static struct
-{
+static struct {
    Slideout           *active;
 } Mode_slideouts =
 {
@@ -98,7 +96,7 @@ SlideoutShow(Slideout * s, EWin * ewin, Win win)
 
    SlideoutCalcSize(s);
    EGetGeometry(win, NULL, NULL, NULL, &w, &h, NULL, NULL);
-   ETranslateCoordinates(win, VRoot.win, 0, 0, &x, &y, NULL);
+   ETranslateCoordinates(win, VROOT, 0, 0, &x, &y, NULL);
 
    sw = EoGetW(s);
    sh = EoGetH(s);
@@ -109,7 +107,7 @@ SlideoutShow(Slideout * s, EWin * ewin, Win win)
      case 2:
 	xx = x + ((w - sw) >> 1);
 	yy = y - sh;
-	if ((yy < 0) && (sh < VRoot.h))
+	if ((yy < 0) && (sh < WinGetH(VROOT)))
 	  {
 	     pdir = s->direction;
 	     s->direction = 1;
@@ -121,7 +119,7 @@ SlideoutShow(Slideout * s, EWin * ewin, Win win)
      case 3:
 	xx = x + ((w - sw) >> 1);
 	yy = y + h;
-	if (((yy + sh) > VRoot.h) && (sh < VRoot.h))
+	if (((yy + sh) > WinGetH(VROOT)) && (sh < WinGetH(VROOT)))
 	  {
 	     pdir = s->direction;
 	     s->direction = 0;
@@ -133,7 +131,7 @@ SlideoutShow(Slideout * s, EWin * ewin, Win win)
      case 0:
 	xx = x - sw;
 	yy = y + ((h - sh) >> 1);
-	if ((xx < 0) && (sw < VRoot.w))
+	if ((xx < 0) && (sw < WinGetW(VROOT)))
 	  {
 	     pdir = s->direction;
 	     s->direction = 1;
@@ -145,7 +143,7 @@ SlideoutShow(Slideout * s, EWin * ewin, Win win)
      case 1:
 	xx = x + w;
 	yy = y + ((h - sh) >> 1);
-	if (((xx + sw) > VRoot.w) && (sw < VRoot.w))
+	if (((xx + sw) > WinGetW(VROOT)) && (sw < WinGetW(VROOT)))
 	  {
 	     pdir = s->direction;
 	     s->direction = 0;
@@ -188,7 +186,7 @@ SlideoutShow(Slideout * s, EWin * ewin, Win win)
 	for (i = 0; i < s->num_objs; i++)
 	   EChangeWindowAttributes(EobjGetWin(s->objs[i]), CWWinGravity, &att);
 	EoMoveResize(s, xx, yy, 1, 1);
-	ESync();
+	ESync(ESYNC_SLIDEOUT);
 	EoMap(s, 2);
 	EobjSlideSizeTo(EoObj(s), xx + sw, yy, xx, yy, 1, sh, sw, sh,
 			Conf.place.slidespeedmap);
@@ -200,7 +198,7 @@ SlideoutShow(Slideout * s, EWin * ewin, Win win)
 	for (i = 0; i < s->num_objs; i++)
 	   EChangeWindowAttributes(EobjGetWin(s->objs[i]), CWWinGravity, &att);
 	EoMoveResize(s, xx, yy, 1, 1);
-	ESync();
+	ESync(ESYNC_SLIDEOUT);
 	EoMap(s, 2);
 	EobjSlideSizeTo(EoObj(s), xx, yy, xx, yy, 1, sh, sw, sh,
 			Conf.place.slidespeedmap);
@@ -212,7 +210,7 @@ SlideoutShow(Slideout * s, EWin * ewin, Win win)
 	for (i = 0; i < s->num_objs; i++)
 	   EChangeWindowAttributes(EobjGetWin(s->objs[i]), CWWinGravity, &att);
 	EoMoveResize(s, xx, yy, 1, 1);
-	ESync();
+	ESync(ESYNC_SLIDEOUT);
 	EoMap(s, 2);
 	EobjSlideSizeTo(EoObj(s), xx, yy + sh, xx, yy, sw, 1, sw, sh,
 			Conf.place.slidespeedmap);
@@ -224,7 +222,7 @@ SlideoutShow(Slideout * s, EWin * ewin, Win win)
 	for (i = 0; i < s->num_objs; i++)
 	   EChangeWindowAttributes(EobjGetWin(s->objs[i]), CWWinGravity, &att);
 	EoMoveResize(s, xx, yy, 1, 1);
-	ESync();
+	ESync(ESYNC_SLIDEOUT);
 	EoMap(s, 2);
 	EobjSlideSizeTo(EoObj(s), xx, yy, xx, yy, sw, 1, sw, sh,
 			Conf.place.slidespeedmap);
