@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -35,8 +35,7 @@
 #include "timers.h"
 #include "xwin.h"
 
-struct _snapshot
-{
+struct _snapshot {
    char               *name;
    char               *win_title;
    char               *win_name;
@@ -57,7 +56,7 @@ struct _snapshot
    int                 layer;
    char                sticky;
    char                shaded;
-   unsigned int        flags;
+   unsigned int        flags[2];
    char               *cmd;
    int                *groups;
    int                 num_groups;
@@ -102,22 +101,15 @@ SnapshotDestroy(Snapshot * sn)
    if (sn->used)
       sn->used->snap = NULL;
 
-   if (sn->name)
-      Efree(sn->name);
-   if (sn->win_title)
-      Efree(sn->win_title);
-   if (sn->win_name)
-      Efree(sn->win_name);
-   if (sn->win_class)
-      Efree(sn->win_class);
-   if (sn->win_role)
-      Efree(sn->win_role);
-   if (sn->border_name)
-      Efree(sn->border_name);
-   if (sn->cmd)
-      Efree(sn->cmd);
-   if (sn->groups)
-      Efree(sn->groups);
+   Efree(sn->name);
+   Efree(sn->win_title);
+   Efree(sn->win_name);
+   Efree(sn->win_class);
+   Efree(sn->win_role);
+   Efree(sn->border_name);
+   Efree(sn->cmd);
+   Efree(sn->groups);
+
    Efree(sn);
 }
 
@@ -311,8 +303,7 @@ SnapshotEwinGet(EWin * ewin, unsigned int match_flags)
 static void
 SnapEwinBorder(Snapshot * sn, const EWin * ewin)
 {
-   if (sn->border_name)
-      Efree(sn->border_name);
+   Efree(sn->border_name);
    sn->border_name = NULL;
    if (ewin->previous_border)
       sn->border_name = Estrdup(BorderGetName(ewin->previous_border));
@@ -345,8 +336,8 @@ SnapEwinLocation(Snapshot * sn, const EWin * ewin)
    if (!EoIsSticky(ewin))
      {
 	DeskGetArea(EoGetDesk(ewin), &ax, &ay);
-	sn->x += ((ax - sn->area_x) * VRoot.w);
-	sn->y += ((ay - sn->area_y) * VRoot.h);
+	sn->x += ((ax - sn->area_x) * WinGetW(VROOT));
+	sn->y += ((ay - sn->area_y) * WinGetH(VROOT));
      }
 }
 
@@ -379,7 +370,7 @@ SnapEwinSkipLists(Snapshot * sn, const EWin * ewin)
 static void
 SnapEwinFlags(Snapshot * sn, const EWin * ewin)
 {
-   sn->flags = EwinFlagsEncode(ewin);
+   EwinFlagsEncode(ewin, sn->flags);
 }
 
 static void
@@ -389,8 +380,7 @@ SnapEwinCmd(Snapshot * sn, const EWin * ewin)
        strcmp(ewin->icccm.wm_machine, Mode.wm.machine_name))
       return;
 
-   if (sn->cmd)
-      Efree(sn->cmd);
+   Efree(sn->cmd);
    sn->cmd = Estrdup(ewin->icccm.wm_command);
 }
 
@@ -406,8 +396,8 @@ SnapEwinGroups(Snapshot * sn, const EWin * ewin, char onoff)
 
    if (!ewin->groups)
      {
-	if (sn->groups)
-	   Efree(sn->groups);
+	Efree(sn->groups);
+	sn->groups = NULL;
 	sn->num_groups = 0;
 	return;
      }
@@ -427,9 +417,7 @@ SnapEwinGroups(Snapshot * sn, const EWin * ewin, char onoff)
 		     sn = SnapshotEwinGet(gwins[i], SNAP_MATCH_DEFAULT);
 		  if (sn)
 		    {
-		       if (sn->groups)
-			  Efree(sn->groups);
-
+		       Efree(sn->groups);
 		       sn->groups = EMALLOC(int, num_groups);
 
 		       sn->num_groups = num_groups;
@@ -447,8 +435,8 @@ SnapEwinGroups(Snapshot * sn, const EWin * ewin, char onoff)
 		  sn = gwins[i]->snap;
 		  if (sn)
 		    {
-		       if (sn->groups)
-			  Efree(sn->groups);
+		       Efree(sn->groups);
+		       sn->groups = NULL;
 		       sn->num_groups = 0;
 		    }
 	       }
@@ -561,12 +549,10 @@ SnapshotEwinRemove(EWin * ewin)
 /*
  * Snapshot dialogs
  */
-typedef struct
-{
+typedef struct {
    Window              client;
 
-   struct
-   {
+   struct {
       char                title;
       char                name;
       char                clss;
@@ -957,8 +943,7 @@ SnapshotEwinDialog(const EWin * ewin)
  * _another_ global var, or a wrapper struct to pass data to the 
  * callback funcs besides the dialog itself -- this is much easier */
 
-typedef struct _remwinlist
-{
+typedef struct _remwinlist {
    Snapshot           *snap;
    char                remove;
 } RememberWinList;
@@ -1138,7 +1123,7 @@ Real_SaveSnapInfo(int dumval __UNUSED__, void *dumdat __UNUSED__)
       if (sn->use_flags & SNAP_USE_DESK)
 	 fprintf(f, "DESKTOP: %i\n", sn->desktop);
       if (sn->use_flags & SNAP_USE_POS)
-	 fprintf(f, "RES: %i %i\n", VRoot.w, VRoot.h);
+	 fprintf(f, "RES: %i %i\n", WinGetW(VROOT), WinGetH(VROOT));
       if (sn->use_flags & SNAP_USE_SIZE)
 	 fprintf(f, "WH: %i %i\n", sn->w, sn->h);
       if (sn->use_flags & SNAP_USE_POS)
@@ -1156,7 +1141,7 @@ Real_SaveSnapInfo(int dumval __UNUSED__, void *dumdat __UNUSED__)
 	   fprintf(f, "SKIPFOCUS: %i\n", sn->skipfocus);
 	}
       if (sn->use_flags & SNAP_USE_FLAGS)
-	 fprintf(f, "FLAGS: %#x\n", sn->flags);
+	 fprintf(f, "FLAGS: %#x %#x\n", sn->flags[0], sn->flags[1]);
 #if USE_COMPOSITE
       if (sn->use_flags & SNAP_USE_OPACITY)
 	 fprintf(f, "OPACITY: %i %i\n", sn->opacity, sn->focused_opacity);
@@ -1214,8 +1199,8 @@ LoadSnapInfo(void)
    if (!f)
       return;
 
-   res_w = VRoot.w;
-   res_h = VRoot.h;
+   res_w = WinGetW(VROOT);
+   res_h = WinGetH(VROOT);
    while (fgets(buf, sizeof(buf), f))
      {
 	s = strchr(buf, ':');
@@ -1227,8 +1212,8 @@ LoadSnapInfo(void)
 	   continue;
 	if (!strcmp(buf, "NEW"))
 	  {
-	     res_w = VRoot.w;
-	     res_h = VRoot.h;
+	     res_w = WinGetW(VROOT);
+	     res_h = WinGetH(VROOT);
 	     sn = SnapshotCreate(s);
 	  }
 	else if (sn)
@@ -1303,7 +1288,7 @@ LoadSnapInfo(void)
 		  sn->x = a;
 		  sn->y = b;
 		  /* we changed reses since we last used this snapshot file */
-		  if (res_w != VRoot.w)
+		  if (res_w != WinGetW(VROOT))
 		    {
 		       if (sn->use_flags & SNAP_USE_SIZE)
 			 {
@@ -1311,15 +1296,16 @@ LoadSnapInfo(void)
 			       sn->x = 0;
 			    else
 			       sn->x =
-				  (sn->x * (VRoot.w - sn->w)) / (res_w - sn->w);
+				  (sn->x * (WinGetW(VROOT) - sn->w)) /
+				  (res_w - sn->w);
 			 }
 		       else
 			 {
-			    if (sn->x >= VRoot.w)
-			       sn->x = VRoot.w - 32;
+			    if (sn->x >= WinGetW(VROOT))
+			       sn->x = WinGetW(VROOT) - 32;
 			 }
 		    }
-		  if (res_h != VRoot.h)
+		  if (res_h != WinGetH(VROOT))
 		    {
 		       if (sn->use_flags & SNAP_USE_SIZE)
 			 {
@@ -1327,12 +1313,13 @@ LoadSnapInfo(void)
 			       sn->y = 0;
 			    else
 			       sn->y =
-				  (sn->y * (VRoot.h - sn->h)) / (res_h - sn->h);
+				  (sn->y * (WinGetH(VROOT) - sn->h)) /
+				  (res_h - sn->h);
 			 }
 		       else
 			 {
-			    if (sn->y >= VRoot.h)
-			       sn->y = VRoot.h - 32;
+			    if (sn->y >= WinGetH(VROOT))
+			       sn->y = WinGetH(VROOT) - 32;
 			 }
 		    }
 		  sn->area_x = c;
@@ -1371,7 +1358,8 @@ LoadSnapInfo(void)
 	     else if (!strcmp(buf, "FLAGS"))
 	       {
 		  sn->use_flags |= SNAP_USE_FLAGS;
-		  sn->flags = strtoul(s, NULL, 0);
+		  sn->flags[0] = sn->flags[1] = 0;
+		  sscanf(s, "%i %i", sn->flags, sn->flags + 1);
 	       }
 	     else if (!strcmp(buf, "GROUP"))
 	       {
@@ -1459,8 +1447,8 @@ SnapshotEwinApply(EWin * ewin)
 	if (!EoIsSticky(ewin))
 	  {
 	     DeskGetArea(EoGetDesk(ewin), &ax, &ay);
-	     ewin->client.x += ((sn->area_x - ax) * VRoot.w);
-	     ewin->client.y += ((sn->area_y - ay) * VRoot.h);
+	     ewin->client.x += ((sn->area_x - ax) * WinGetW(VROOT));
+	     ewin->client.y += ((sn->area_y - ay) * WinGetH(VROOT));
 	  }
 	EMoveResizeWindow(EwinGetClientWin(ewin), ewin->client.x,
 			  ewin->client.y, ewin->client.w, ewin->client.h);
@@ -1505,7 +1493,7 @@ SnapshotEwinApply(EWin * ewin)
 #if USE_COMPOSITE
    if (use_flags & SNAP_USE_OPACITY)
      {
-	sn->opacity = OpacityFix(sn->opacity, 100);
+	sn->opacity = OpacityFix(sn->opacity, 0);
 	sn->focused_opacity = OpacityFix(sn->focused_opacity, 0);
 	ewin->ewmh.opacity = OpacityFromPercent(sn->opacity);
 	ewin->props.focused_opacity = OpacityFromPercent(sn->focused_opacity);
@@ -1684,7 +1672,7 @@ _SnapShow(void *data, void *prm)
 	 ("         skiptask: %d    skipfocus: %d    skipwinlist: %d\n",
 	  sn->skiptask, sn->skipfocus, sn->skipwinlist);
    if (sn->use_flags & SNAP_USE_FLAGS)
-      IpcPrintf("            flags: %#x\n", sn->flags);
+      IpcPrintf("            flags: %#x %#x\n", sn->flags[0], sn->flags[1]);
    IpcPrintf("\n");
 }
 
