@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -391,8 +391,7 @@ TextstateTextFitMB(TextState * ts, char **ptext, int *pw, int textwidth_limit)
    Efree(text);
    *ptext = new_line;
  done:
-   if (wc_line)
-      Efree(wc_line);
+   Efree(wc_line);
    EwcClose();
 }
 
@@ -402,8 +401,7 @@ TextstateTextFitMB(TextState * ts, char **ptext, int *pw, int textwidth_limit)
  */
 extern const FontOps FontOpsXfs;
 
-typedef struct
-{
+typedef struct {
    XFontSet            font;
    int                 ascent;
    Win                 win;
@@ -520,8 +518,7 @@ const FontOps       FontOpsXfs = {
  */
 extern const FontOps FontOpsXfont;
 
-typedef struct
-{
+typedef struct {
    XFontStruct        *font;
    Win                 win;
    Drawable            draw;
@@ -650,8 +647,7 @@ TsTextDraw(TextState * ts, int x, int y, const char *text, int len)
    ts->ops->TextDraw(ts, x, y, text, len);
 }
 
-typedef struct
-{
+typedef struct {
    const char         *type;
    const FontOps      *ops;
    char                checked;
@@ -754,9 +750,8 @@ TextStateLoadFont(TextState * ts)
 		goto fallback;
 	  }
 #endif
-	ts->ops = fhp->ops;
-	ts->ops->Load(ts, name);
-	goto done;
+	if (fhp->ops->Load(ts, name) == 0)
+	   goto done;
      }
  fallback:
 #if FONT_TYPE_XFS
@@ -829,12 +824,12 @@ TextstateTextFit(TextState * ts, char **ptext, int *pw, int textwidth_limit)
 void
 TextstateTextDraw(TextState * ts, Win win, Drawable draw, const char *text,
 		  int x, int y, int w, int h, const EImageBorder * pad,
-		  int fsize __UNUSED__, int justification)
+		  int fsize __UNUSED__, int justh, int justv)
 {
    const char         *str;
    char              **lines;
    int                 i, num_lines;
-   int                 textwidth_limit, offset_x, offset_y;
+   int                 textwidth_limit, textheight_limit, offset_x, offset_y;
    int                 xx, yy, ww, hh, ascent;
    Pixmap              drawable;
 
@@ -866,6 +861,7 @@ TextstateTextDraw(TextState * ts, Win win, Drawable draw, const char *text,
 	     h -= pad->top + pad->bottom;
 	  }
 	textwidth_limit = w;
+	textheight_limit = h;
      }
    else
      {
@@ -877,11 +873,12 @@ TextstateTextDraw(TextState * ts, Win win, Drawable draw, const char *text,
 	     w -= pad->top + pad->bottom;
 	  }
 	textwidth_limit = h;
+	textheight_limit = w;
      }
 
 #if 0
-   Eprintf("TextstateTextDraw %d,%d %dx%d(%d): %s\n", x, y, w, h,
-	   textwidth_limit, text);
+   Eprintf("TextstateTextDraw %d,%d %dx%d(%dx%d): %s\n", x, y, w, h,
+	   textwidth_limit, textheight_limit, text);
 #endif
 
    xx = x;
@@ -901,9 +898,11 @@ TextstateTextDraw(TextState * ts, Win win, Drawable draw, const char *text,
 	     if (ww > textwidth_limit)
 		ts->ops->TextFit(ts, &lines[i], &ww, textwidth_limit);
 
+	     if (justv && num_lines == 1 && textheight_limit > 0)
+		yy += (textheight_limit - hh) / 2;
 	     if (i == 0)
 		yy += ascent;
-	     xx = x + (((textwidth_limit - ww) * justification) >> 10);
+	     xx = x + (((textwidth_limit - ww) * justh) >> 10);
 
 	     im = TextImageGet(win, draw, xx - 1, yy - 1 - ascent,
 			       ww + 2, hh + 2, ts);
@@ -932,9 +931,11 @@ TextstateTextDraw(TextState * ts, Win win, Drawable draw, const char *text,
 	     if (ww > textwidth_limit)
 		ts->ops->TextFit(ts, &lines[i], &ww, textwidth_limit);
 
+	     if (justv && num_lines == 1 && textheight_limit > 0)
+		yy += (textheight_limit - hh) / 2;
 	     if (i == 0)
 		yy += ascent;
-	     xx = x + (((textwidth_limit - ww) * justification) >> 10);
+	     xx = x + (((textwidth_limit - ww) * justh) >> 10);
 
 	     if (ts->style.orientation != FONT_TO_RIGHT)
 		drawable = ECreatePixmap(win, ww + 2, hh + 2, 0);
@@ -977,7 +978,7 @@ TextstateTextDraw(TextState * ts, Win win, Drawable draw, const char *text,
 void
 TextDraw(TextClass * tclass, Win win, Drawable draw, int active, int sticky,
 	 int state, const char *text, int x, int y, int w, int h, int fsize,
-	 int justification)
+	 int justh)
 {
    TextState          *ts;
 
@@ -988,6 +989,5 @@ TextDraw(TextClass * tclass, Win win, Drawable draw, int active, int sticky,
    if (!ts)
       return;
 
-   TextstateTextDraw(ts, win, draw, text, x, y, w, h, NULL, fsize,
-		     justification);
+   TextstateTextDraw(ts, win, draw, text, x, y, w, h, NULL, fsize, justh, 0);
 }
