@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2005-2007 Kim Woelders
+ * Copyright (C) 2005-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -33,8 +33,7 @@
 
 typedef struct _windowmatch WindowMatch;
 
-struct _windowmatch
-{
+struct _windowmatch {
    char               *name;
    /* Match criteria */
    char                match;
@@ -42,8 +41,8 @@ struct _windowmatch
    char                prop;
    char                qual;
    char               *value;
-   Constraints         width;
-   Constraints         height;
+   int                 width_min, width_max;
+   int                 height_min, height_max;
    /* Match actions */
    char               *args;
    Border             *border;
@@ -109,8 +108,8 @@ WindowMatchCreate(const char *name)
    ecore_list_prepend(wm_list, b);
 
    b->name = Estrdup(name);
-   b->width.max = 99999;
-   b->height.max = 99999;
+   b->width_max = 99999;
+   b->height_max = 99999;
 
    return b;
 }
@@ -123,12 +122,9 @@ WindowMatchDestroy(WindowMatch * wm)
 
    ecore_list_node_remove(wm_list, wm);
 
-   if (wm->name)
-      Efree(wm->name);
-   if (wm->value)
-      Efree(wm->value);
-   if (wm->args)
-      Efree(wm->args);
+   Efree(wm->name);
+   Efree(wm->value);
+   Efree(wm->args);
 
    Efree(wm);
 }
@@ -192,7 +188,7 @@ WindowMatchConfigLoad(FILE * fs)
 		wm->match = MATCH_TYPE_SIZE;
 	     else
 		wm->match = MATCH_TYPE_SIZE_H;
-	     sscanf(s, "%*s %u %u", &(wm->width.min), &(wm->width.max));
+	     sscanf(s, "%*s %u %u", &(wm->width_min), &(wm->width_max));
 	     break;
 	  case WINDOWMATCH_HEIGHT:
 	     if (!wm)
@@ -201,7 +197,7 @@ WindowMatchConfigLoad(FILE * fs)
 		wm->match = MATCH_TYPE_SIZE;
 	     else
 		wm->match = MATCH_TYPE_SIZE_V;
-	     sscanf(s, "%*s %u %u", &(wm->height.min), &(wm->height.max));
+	     sscanf(s, "%*s %u %u", &(wm->height_min), &(wm->height_max));
 	     break;
 
 	  case WINDOWMATCH_TRANSIENT:
@@ -329,24 +325,24 @@ WindowMatchDecode(const char *line)
 	num = sscanf(value, "%u-%ux%u-%u", &w1, &w2, &h1, &h2);
 	if (num < 4)
 	   goto case_error;
-	wm->width.min = w1;
-	wm->width.max = w2;
-	wm->height.min = h1;
-	wm->height.max = h2;
+	wm->width_min = w1;
+	wm->width_max = w2;
+	wm->height_min = h1;
+	wm->height_max = h2;
 	break;
      case MATCH_TYPE_SIZE_H:
 	num = sscanf(value, "%u-%u", &w1, &w2);
 	if (num < 2)
 	   goto case_error;
-	wm->width.min = w1;
-	wm->width.max = w2;
+	wm->width_min = w1;
+	wm->width_max = w2;
 	break;
      case MATCH_TYPE_SIZE_V:
 	num = sscanf(value, "%u-%u", &h1, &h2);
 	if (num < 2)
 	   goto case_error;
-	wm->height.min = h1;
-	wm->height.max = h2;
+	wm->height_min = h1;
+	wm->height_max = h2;
 	break;
 
      case MATCH_TYPE_PROP:
@@ -431,16 +427,16 @@ WindowMatchEncode(WindowMatch * wm, char *buf, int len)
 
      case MATCH_TYPE_SIZE:
 	value = s;
-	sprintf(s, "%u-%ux%u-%u", wm->width.min, wm->width.max,
-		wm->height.min, wm->height.max);
+	sprintf(s, "%u-%ux%u-%u", wm->width_min, wm->width_max,
+		wm->height_min, wm->height_max);
 	break;
      case MATCH_TYPE_SIZE_H:
 	value = s;
-	sprintf(s, "%u-%u", wm->width.min, wm->width.max);
+	sprintf(s, "%u-%u", wm->width_min, wm->width_max);
 	break;
      case MATCH_TYPE_SIZE_V:
 	value = s;
-	sprintf(s, "%u-%u", wm->height.min, wm->height.max);
+	sprintf(s, "%u-%u", wm->height_min, wm->height_max);
 	break;
 
      case MATCH_TYPE_PROP:
@@ -508,18 +504,18 @@ WindowMatchEwinTest(const WindowMatch * wm, const EWin * ewin)
 	return matchregexp(wm->value, EwinGetIcccmClass(ewin));
 
      case MATCH_TYPE_SIZE:
-	match = (ewin->client.w >= wm->width.min &&
-		 ewin->client.w <= wm->width.max &&
-		 ewin->client.h >= wm->height.min &&
-		 ewin->client.h <= wm->height.max);
+	match = (ewin->client.w >= wm->width_min &&
+		 ewin->client.w <= wm->width_max &&
+		 ewin->client.h >= wm->height_min &&
+		 ewin->client.h <= wm->height_max);
 	break;
      case MATCH_TYPE_SIZE_H:
-	match = (ewin->client.w >= wm->width.min &&
-		 ewin->client.w <= wm->width.max);
+	match = (ewin->client.w >= wm->width_min &&
+		 ewin->client.w <= wm->width_max);
 	break;
      case MATCH_TYPE_SIZE_V:
-	match = (ewin->client.h >= wm->height.min &&
-		 ewin->client.h <= wm->height.max);
+	match = (ewin->client.h >= wm->height_min &&
+		 ewin->client.h <= wm->height_max);
 	break;
 
      case MATCH_TYPE_PROP:
@@ -576,8 +572,7 @@ WindowMatchEobjTest(const WindowMatch * wm, const EObj * eo)
 }
 #endif
 
-typedef struct
-{
+typedef struct {
    int                 type;
    const EWin         *ewin;
 } wmatch_type_data;
@@ -659,7 +654,7 @@ WindowMatchEwinOpsAction(EWin * ewin, int op, const char *args)
 	return;
 
      case EWIN_OP_TITLE:
-	_EFREE(EwinGetIcccmName(ewin));
+	Efree(EwinGetIcccmName(ewin));
 	EwinGetIcccmName(ewin) = Estrdup(args);
 	break;
 
@@ -715,7 +710,7 @@ WindowMatchEwinOpsAction(EWin * ewin, int op, const char *args)
 
      case EWIN_OP_OPACITY:
 	a = atoi(args);
-	ewin->ewmh.opacity = OpacityFromPercent(OpacityFix(a, 100));
+	ewin->ewmh.opacity = OpacityFromPercent(OpacityFix(a, 0));
 	break;
 
      case EWIN_OP_FOCUSED_OPACITY:
@@ -869,35 +864,43 @@ WindowMatchEobjOpsParse(EObj * eo, const char *ops)
    return err;
 }
 
-void
-WindowMatchEwinOps(EWin * ewin)
+static void
+_WindowMatchEwinFunc(void *_wm, void *_ew)
 {
-   const WindowMatch  *wm;
+   const WindowMatch  *wm = (WindowMatch *) _wm;
+   EWin               *ew = (EWin *) _ew;
 
-   ECORE_LIST_FOR_EACH(wm_list, wm)
-   {
-      if (wm->op != MATCH_OP_WINOP || !WindowMatchEwinTest(wm, ewin))
-	 continue;
+   if (wm->op != MATCH_OP_WINOP || !WindowMatchEwinTest(wm, ew))
+      return;
 
-      /* Match found - do the ops */
-      WindowMatchEobjOpsParse(EoObj(ewin), wm->args);
-   }
+   /* Match found - do the ops */
+   WindowMatchEobjOpsParse(EoObj(ew), wm->args);
+}
+
+void
+WindowMatchEwinOps(EWin * ew)
+{
+   ecore_list_for_each(wm_list, _WindowMatchEwinFunc, ew);
 }
 
 #if USE_COMPOSITE
+static void
+_WindowMatchEobjFunc(void *_wm, void *_eo)
+{
+   const WindowMatch  *wm = (WindowMatch *) _wm;
+   EObj               *eo = (EObj *) _eo;
+
+   if (wm->op != MATCH_OP_WINOP || !WindowMatchEobjTest(wm, eo))
+      return;
+
+   /* Match found - do the ops */
+   WindowMatchEobjOpsParse(eo, wm->args);
+}
+
 void
 WindowMatchEobjOps(EObj * eo)
 {
-   const WindowMatch  *wm;
-
-   ECORE_LIST_FOR_EACH(wm_list, wm)
-   {
-      if (wm->op != MATCH_OP_WINOP || !WindowMatchEobjTest(wm, eo))
-	 continue;
-
-      /* Match found - do the ops */
-      WindowMatchEobjOpsParse(eo, wm->args);
-   }
+   ecore_list_for_each(wm_list, _WindowMatchEobjFunc, eo);
 }
 #endif
 
