@@ -1,6 +1,6 @@
 /*
  * Copyright (C) 2000-2007 Carsten Haitzler, Geoff Harrison and various contributors
- * Copyright (C) 2004-2007 Kim Woelders
+ * Copyright (C) 2004-2008 Kim Woelders
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to
@@ -40,6 +40,8 @@
 #define DEBUG_XWIN   0
 #define DEBUG_PIXMAP 0
 
+EDisplay            Dpy;
+
 #if USE_COMPOSITE
 static Visual      *argb_visual = NULL;
 static Colormap     argb_cmap = None;
@@ -108,6 +110,12 @@ WinGetCmap(const Win win)
 }
 #endif
 
+void
+EXInit(void)
+{
+   memset(&Dpy, 0, sizeof(Dpy));
+}
+
 static              Win
 EXidCreate(void)
 {
@@ -128,8 +136,7 @@ EXidDestroy(Win win)
 #endif
    if (win->rects)
       XFree(win->rects);
-   if (win->cbl.lst)
-      Efree(win->cbl.lst);
+   Efree(win->cbl.lst);
    Efree(win);
 }
 
@@ -392,7 +399,8 @@ ECreateArgbWindow(Win parent, int x, int y, int w, int h, Win cwin)
 	  {
 	     argb_visual = EVisualFindARGB();
 	     argb_cmap =
-		XCreateColormap(disp, VRoot.xwin, argb_visual, AllocNone);
+		XCreateColormap(disp, WinGetXwin(VROOT), argb_visual,
+				AllocNone);
 	  }
 	depth = 32;
 	vis = argb_visual;
@@ -412,7 +420,7 @@ ECreateWindowVD(Win parent, int x, int y, int w, int h,
    if (!vis || depth == 0)
       return 0;
 
-   cmap = XCreateColormap(disp, VRoot.xwin, vis, AllocNone);
+   cmap = XCreateColormap(disp, WinGetXwin(VROOT), vis, AllocNone);
 
    return ECreateWindowVDC(parent, x, y, w, h, vis, depth, cmap);
 }
@@ -505,7 +513,7 @@ ECreateFocusWindow(Win parent, int x, int y, int w, int h)
 
    attr.backing_store = NotUseful;
    attr.override_redirect = False;
-   attr.colormap = VRoot.cmap;
+   attr.colormap = WinGetCmap(VROOT);
    attr.border_pixel = 0;
    attr.background_pixel = 0;
    attr.save_under = False;
@@ -729,8 +737,8 @@ ECreateWinFromXwin(Window xwin)
    win->w = w;
    win->h = h;
    win->depth = depth;
-   win->visual = VRoot.vis;
-   win->cmap = VRoot.cmap;
+   win->visual = WinGetVisual(VROOT);
+   win->cmap = WinGetCmap(VROOT);
 
    return win;
 }
@@ -936,7 +944,7 @@ EGetGeometry(Win win, Window * root_return, int *x, int *y,
    if (depth)
       *depth = win->depth;
    if (root_return)
-      *root_return = VRoot.xwin;
+      *root_return = WinGetXwin(VROOT);
 
    return 1;
 }
@@ -1033,11 +1041,13 @@ EGetWindowBackgroundPixmap(Win win)
 void
 EFreeWindowBackgroundPixmap(Win win)
 {
-   if (!win || !win->bgpmap || !win->bg_owned)
+   if (!win || !win->bgpmap)
       return;
 
-   EFreePixmap(win->bgpmap);
+   if (win->bg_owned)
+      EFreePixmap(win->bgpmap);
    win->bgpmap = 0;
+   win->bg_owned = 0;
 }
 
 void
@@ -1061,11 +1071,58 @@ ESetWindowBackground(Win win, unsigned int col)
    XSetWindowBackground(disp, win->xwin, col);
 }
 
+void
+ESelectInput(Win win, unsigned int event_mask)
+{
+   XSelectInput(disp, win->xwin, event_mask);
+}
+
+void
+EChangeWindowAttributes(Win win, unsigned int mask, XSetWindowAttributes * attr)
+{
+   XChangeWindowAttributes(disp, win->xwin, mask, attr);
+}
+
+void
+ESetWindowBorderWidth(Win win, unsigned int bw)
+{
+   XSetWindowBorderWidth(disp, win->xwin, bw);
+}
+
+void
+ERaiseWindow(Win win)
+{
+   XRaiseWindow(disp, win->xwin);
+}
+
+void
+ELowerWindow(Win win)
+{
+   XLowerWindow(disp, win->xwin);
+}
+
+void
+EXRestackWindows(Window * windows, int nwindows)
+{
+   XRestackWindows(disp, windows, nwindows);
+}
+
+void
+EClearWindow(Win win)
+{
+   XClearWindow(disp, win->xwin);
+}
+
+void
+EClearArea(Win win, int x, int y, unsigned int w, unsigned int h)
+{
+   XClearArea(disp, win->xwin, x, y, w, h, False);
+}
+
 int
-ETranslateCoordinates(Win src_w, Win dst_w,
-		      int src_x, int src_y,
-		      int *dest_x_return,
-		      int *dest_y_return, Window * child_return)
+ETranslateCoordinates(Win src_w, Win dst_w, int src_x, int src_y,
+		      int *dest_x_return, int *dest_y_return,
+		      Window * child_return)
 {
    Window              child;
 
@@ -1091,7 +1148,7 @@ EXQueryPointer(Window xwin, int *px, int *py, Window * pchild,
    unsigned int        mask;
 
    if (xwin == None)
-      xwin = VRoot.xwin;
+      xwin = WinGetXwin(VROOT);
 
    if (!px)
       px = &root_x;
@@ -1111,13 +1168,13 @@ EQueryPointer(Win win, int *px, int *py, Window * pchild, unsigned int *pmask)
 {
    Window              xwin;
 
-   xwin = (win) ? win->xwin : VRoot.xwin;
+   xwin = (win) ? win->xwin : WinGetXwin(VROOT);
 
    return EXQueryPointer(xwin, px, py, pchild, pmask);
 }
 
 void
-ESelectInputChange(Win win, long set, long clear)
+ESelectInputChange(Win win, unsigned long set, unsigned long clear)
 {
    XWindowAttributes   xwa;
 
@@ -1146,6 +1203,30 @@ EDrawableCheck(Drawable draw, int grab)
    return ok;
 }
 
+KeyCode
+EKeysymToKeycode(KeySym keysym)
+{
+   return XKeysymToKeycode(disp, keysym);
+}
+
+KeyCode
+EKeynameToKeycode(const char *name)
+{
+   return XKeysymToKeycode(disp, XStringToKeysym(name));
+}
+
+const char         *
+EKeycodeToString(KeyCode keycode, int ix)
+{
+   return XKeysymToString(XKeycodeToKeysym(disp, keycode, ix));
+}
+
+Atom
+EInternAtom(const char *name)
+{
+   return XInternAtom(disp, name, False);
+}
+
 #define DEBUG_SHAPE_OPS 0
 #define DEBUG_SHAPE_PROPAGATE 0
 
@@ -1400,7 +1481,7 @@ EShapePropagate(Win win)
    unsigned int        num_rects;
    int                 k, rn;
    int                 x, y, w, h;
-   XRectangle         *rects, *rl;
+   XRectangle         *rects, *rectsn, *rl;
 
    if (!win || win->w <= 0 || win->h <= 0)
       return 0;
@@ -1438,7 +1519,11 @@ EShapePropagate(Win win)
 	if (rn > 0)
 	  {
 	     rl = xch->rects;
-	     rects = EREALLOC(XRectangle, rects, num_rects + rn);
+	     rectsn = EREALLOC(XRectangle, rects, num_rects + rn);
+	     if (!rectsn)
+		goto bail_out;
+	     rects = rectsn;
+
 	     /* go through all clip rects in thsi window's shape */
 	     for (k = 0; k < rn; k++)
 	       {
@@ -1459,7 +1544,10 @@ EShapePropagate(Win win)
 	else if (rn == 0)
 	  {
 	     /* Unshaped */
-	     rects = EREALLOC(XRectangle, rects, num_rects + 1);
+	     rectsn = EREALLOC(XRectangle, rects, num_rects + 1);
+	     if (!rectsn)
+		goto bail_out;
+	     rects = rectsn;
 
 	     rects[num_rects].x = x;
 	     rects[num_rects].y = y;
@@ -1488,6 +1576,11 @@ EShapePropagate(Win win)
      }
 
    return win->num_rect;
+
+ bail_out:
+   Efree(rects);
+   EShapeCombineMask(win, ShapeBounding, 0, 0, None, ShapeSet);
+   return 0;
 }
 
 void
@@ -1563,6 +1656,25 @@ EXCopyArea(Drawable src, Drawable dst, int sx, int sy, unsigned int w,
    EXFreeGC(gc);
 }
 
+void
+EXCopyAreaTiled(Drawable src, Pixmap mask, Drawable dst, int sx, int sy,
+		unsigned int w, unsigned int h, int dx, int dy)
+{
+   GC                  gc;
+   XGCValues           gcv;
+
+   gcv.fill_style = FillTiled;
+   gcv.tile = src;
+   gcv.ts_x_origin = sx;
+   gcv.ts_y_origin = sy;
+   gcv.clip_mask = mask;
+   gc = EXCreateGC(dst, GCFillStyle |
+		   GCTile | GCTileStipXOrigin | GCTileStipYOrigin | GCClipMask,
+		   &gcv);
+   XFillRectangle(disp, dst, gc, dx, dy, w, h);
+   EXFreeGC(gc);
+}
+
 GC
 EXCreateGC(Drawable draw, unsigned long mask, XGCValues * val)
 {
@@ -1588,6 +1700,12 @@ EXFreeGC(GC gc)
    return XFreeGC(disp, gc);
 }
 
+void
+EXSendEvent(Window xwin, long event_mask, XEvent * ev)
+{
+   XSendEvent(disp, xwin, False, event_mask, ev);
+}
+
 void
 EAllocColor(Colormap cmap, EColor * pec)
 {
@@ -1644,11 +1762,10 @@ EWindowGetShapePixmap(Win win)
  * Display
  */
 
-Display            *
+int
 EDisplayOpen(const char *dstr, int scr)
 {
    char                dbuf[256], *s;
-   Display            *dpy;
 
    if (scr >= 0)
      {
@@ -1667,12 +1784,12 @@ EDisplayOpen(const char *dstr, int scr)
 
 #ifdef USE_ECORE_X
    ecore_x_init(dstr);
-   dpy = ecore_x_display_get();
+   disp = ecore_x_display_get();
 #else
-   dpy = XOpenDisplay(dstr);
+   disp = XOpenDisplay(dstr);
 #endif
 
-   return dpy;
+   return (disp) ? 0 : -1;
 }
 
 void
@@ -1706,6 +1823,48 @@ EDisplayDisconnect(void)
    disp = NULL;
 }
 
+static int
+HandleXError(Display * dpy, XErrorEvent * ev)
+{
+   char                buf[64];
+
+   if (EDebug(1))
+     {
+	XGetErrorText(dpy, ev->error_code, buf, 63);
+	Eprintf("*** ERROR: xid=%#lx error=%i req=%i/%i: %s\n",
+		ev->resourceid, ev->error_code,
+		ev->request_code, ev->minor_code, buf);
+     }
+
+   Dpy.last_error_code = ev->error_code;
+
+   return 0;
+}
+
+static void         (*EXIOErrorFunc) (void) = NULL;
+
+static int
+HandleXIOError(Display * dpy __UNUSED__)
+{
+   disp = NULL;
+
+   if (EXIOErrorFunc)
+      EXIOErrorFunc();
+
+   return 0;
+}
+
+void
+EDisplaySetErrorHandlers(void (*fatal) (void))
+{
+   /* set up an error handler for then E would normally have fatal X errors */
+   XSetErrorHandler(HandleXError);
+
+   /* set up a handler for when the X Connection goes down */
+   EXIOErrorFunc = fatal;
+   XSetIOErrorHandler(HandleXIOError);
+}
+
 /*
  * Server
  */
@@ -1713,34 +1872,34 @@ EDisplayDisconnect(void)
 void
 EGrabServer(void)
 {
-   if (Mode.grabs.server_grabbed <= 0)
+   if (Dpy.server_grabbed <= 0)
      {
 	if (EDebug(EDBUG_TYPE_GRABS))
 	   Eprintf("EGrabServer\n");
 	XGrabServer(disp);
      }
-   Mode.grabs.server_grabbed++;
+   Dpy.server_grabbed++;
 }
 
 void
 EUngrabServer(void)
 {
-   if (Mode.grabs.server_grabbed == 1)
+   if (Dpy.server_grabbed == 1)
      {
 	XUngrabServer(disp);
 	XFlush(disp);
 	if (EDebug(EDBUG_TYPE_GRABS))
 	   Eprintf("EUngrabServer\n");
      }
-   Mode.grabs.server_grabbed--;
-   if (Mode.grabs.server_grabbed < 0)
-      Mode.grabs.server_grabbed = 0;
+   Dpy.server_grabbed--;
+   if (Dpy.server_grabbed < 0)
+      Dpy.server_grabbed = 0;
 }
 
 int
 EServerIsGrabbed(void)
 {
-   return Mode.grabs.server_grabbed;
+   return Dpy.server_grabbed;
 }
 
 void
@@ -1750,8 +1909,10 @@ EFlush(void)
 }
 
 void
-ESync(void)
+ESync(unsigned int mask)
 {
+   if (mask & Conf.testing.no_sync_mask)
+      return;
    XSync(disp, False);
 }
 
@@ -1768,7 +1929,7 @@ EVisualFindARGB(void)
    int                 i, num;
    Visual             *vis;
 
-   xvit.screen = VRoot.scr;
+   xvit.screen = Dpy.screen;
    xvit.depth = 32;
 #if __cplusplus
    xvit.c_class = TrueColor;
@@ -1827,7 +1988,7 @@ EGetTimestamp(void)
    if (win_ts == None)
      {
 	attr.override_redirect = False;
-	win_ts = XCreateWindow(disp, VRoot.xwin, -100, -100, 1, 1, 0,
+	win_ts = XCreateWindow(disp, WinGetXwin(VROOT), -100, -100, 1, 1, 0,
 			       CopyFromParent, InputOnly, CopyFromParent,
 			       CWOverrideRedirect, &attr);
 	XSelectInput(disp, win_ts, PropertyChangeMask);
